/********************************************************************************************
 *     LEGAL DISCLAIMER 
 *
 *     (Header of MediaTek Software/Firmware Release or Documentation)
 *
 *     BY OPENING OR USING THIS FILE, BUYER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES 
 *     THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE") RECEIVED 
 *     FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO BUYER ON AN "AS-IS" BASIS 
 *     ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES, EXPRESS OR IMPLIED, 
 *     INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR 
 *     A PARTICULAR PURPOSE OR NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY 
 *     WHATSOEVER WITH RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY, 
 *     INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND BUYER AGREES TO LOOK 
 *     ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO. MEDIATEK SHALL ALSO
 *     NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE RELEASES MADE TO BUYER'S SPECIFICATION 
 *     OR TO CONFORM TO A PARTICULAR STANDARD OR OPEN FORUM.
 *     
 *     BUYER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S ENTIRE AND CUMULATIVE LIABILITY WITH 
 *     RESPECT TO THE MEDIATEK SOFTWARE RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, 
 *     TO REVISE OR REPLACE THE MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE 
 *     FEES OR SERVICE CHARGE PAID BY BUYER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE. 
 *     
 *     THE TRANSACTION CONTEMPLATED HEREUNDER SHALL BE CONSTRUED IN ACCORDANCE WITH THE LAWS 
 *     OF THE STATE OF CALIFORNIA, USA, EXCLUDING ITS CONFLICT OF LAWS PRINCIPLES.  
 ************************************************************************************************/

//============================================================================
// Include files
//============================================================================
#include <asm/arch/nand_operation.h>

#include <emmc.h>


#include <common.h>
#include <malloc.h>

#include <mt85xx_gpio.h>

//============================================================================
// public functions
//============================================================================
#define ALIGN_L(data, align) (data & ~(align-1))
#define ALIGN_H(data, align) ((data + align -1) & ~(align-1))

//============================================================================
// public functions
//============================================================================
int mtk_emmc_read(u_long dstaddr, u_long noffset, u_long size)
{
	int r=0;
	struct emmc *eMMC=NULL;
	UINT32 retlen = 0;

	printf("[EMMC][%s] start=0x%x, len=0x%x, buf=0x%x\n", __func__,
				noffset, size, dstaddr);
	
	eMMC= find_emmc_device(0);
	if(eMMC == NULL)
	{
		printf("[EMMC][%s] Emmc Device == NULL \n", __func__);
		return -1;
	}
	
	if((noffset+size) > (uint64_t)(eMMC->capacity) << 20)
	{
		printf("[EMMC][%s] Parameter out of capacity\n", __func__);
		return -1;
	}
	if(EMMC_OK != emmc_read(eMMC, noffset, size, (UINT8*)dstaddr))
	{
	    printf("[EMMC][%s] Read fail! \n", __func__);
		return -1;
	}

	return r;
}

int mtk_emmc_erase(u_long noffset, u_long size)
{
	struct emmc *eMMC=NULL;

	eMMC= find_emmc_device(0);
	if(eMMC == NULL)
	{
		printf("[EMMC][%s] mtd == NULL \n", __func__);
		return -1;
	}

	if(0 != emmc_earse(eMMC, noffset, size))
	{
		printf("[EMMC][%s] Erase fail \n", __func__);
		return -1;
	}
	return 0;
}

int mtk_emmc_write_align(u_long srcaddr, u_long size, u_long noffset)
{
	struct emmc *eMMC=NULL;

	UINT32 retlen = 0;
	
	printf("[EMMC][%s] start=0x%x, len=0x%x, buf=0x%x\n", __func__,
					noffset, size, srcaddr);
	eMMC= find_emmc_device(0);
	if(eMMC == NULL)
	{
		printf("[EMMC][%s] mtd == NULL \n", __func__);
		return -1;
	}

		
	if((noffset+size) > (eMMC->capacity * eMMC->read_bl_len))
	{
		printf("[EMMC][%s] Parameter out of capacity\n", __func__);
		return -1;
	}
	
	if(EMMC_OK != emmc_write(eMMC, noffset, size, (UINT8*)srcaddr))
	{
	    printf("[EMMC][%s] Write align fail! \n", __func__);
		return -1;
	}		
	return 0;


}

int mtk_emmc_write(u_long srcaddr, u_long size, u_long noffset)
{
	struct emmc *eMMC=NULL;
	UINT32 retlen = 0;
	
	printf("[EMMC][%s] start=0x%x, len=0x%x, buf=0x%x\n", __func__,
					noffset, size, srcaddr);
		
	eMMC= find_emmc_device(0);
	if(eMMC == NULL)
	{
		printf("[EMMC][%s] mtd == NULL \n", __func__);
		return -1;
	}

		
	if((noffset+size) > (eMMC->capacity * eMMC->read_bl_len))
	{
		printf("[EMMC][%s] Parameter out of capacity\n", __func__);
		return -1;
	}
		
	if(EMMC_OK != emmc_write(eMMC, noffset, size, (UINT8*)srcaddr))
	{
		printf("[EMMC][%s] Write fail! \n", __func__);
		return -1;
	}	

	return 0;	
}
