/* Mtk internal MAC uboot driver
 *
 *
 */
#include <chip_ver.h>
#include <asm/arch/mt85xx_mac.h>

static unsigned char *pbuf = NULL;
static unsigned char *pbufOrg = NULL;
static u8  BCmacAddr[6] = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
u32 u32NetlogEnable = 0;

void vDmaInit(void)
{

    vWriteMAC(RW_BURST_TYPE, DMA_BUSMODE_INIT);
    vWriteMAC(RW_FC_CFG, MAC_FLOWCTRL_INIT);
    vWriteMAC(RW_EXTEND_CFG, SEND_PAUSE_RLS_1K);
    MmacDmaIntDisable(NULL);

}

// *********************************************************************
// Function    : i4MacPhyRead
// Description : MII PHY register read
// Parameter   : u4PhyAddr - phy device address
//               u4RegAddr - phy register offset
//               pu4Data - output read data
// Return      : 0 if success
// *********************************************************************
INT32 i4MacPhyRead(UINT32 u4PhyAddr, UINT32 u4RegAddr, UINT32 *pu4Data)
{
    UINT32 u4Data, u4TimeOut;

   // ASSERT(pu4Data != NULL);

   u4Data = 0;
   u4Data = (u4PhyAddr << PHY_ADDR_SHIFT) & PHY_ADDR;
   u4Data |= ((u4RegAddr << PHY_REG_SHIFT) & PHY_REG);
   u4Data |= RD_CMD;
   vWriteMAC(RW_PHY_CTRL0, u4Data);

    u4TimeOut = 0xffff;
    while((dReadMAC(RW_PHY_CTRL0) & RW_OK) == 0) // Wait for R/W command complete
    {
        u4TimeOut --;
        if(u4TimeOut == 0x00) {
            TR("MII read PHY timeout\n");
            return(NET_DRV_HW_ERROR);
        }
    }

     *pu4Data = (UINT32)(dReadMAC(RW_PHY_CTRL0) >> RW_DATA_SHIFT);        // Get PHY data
     SetBitMAC(RW_PHY_CTRL0, RW_OK);  //clear RW_OK

    return (NET_DRV_OK);
}

// *********************************************************************
// Function    : void i4MacPhyWrite(UINT32 u4PhyAddr, UINT32 u4RegAddr, UINT32 u4RegData)
// Description : MII PHY register write
// Parameter   : u4PhyAddr, u4RegAddr, u4RegData
// Return      : none
// *********************************************************************
INT32 i4MacPhyWrite(UINT32 u4PhyAddr, UINT32 u4RegAddr, UINT32 u4RegData)
{
    UINT32 u4Data, u4TimeOut;

   u4Data = (u4RegData & 0xFFFF) << RW_DATA_SHIFT;
   u4Data |= ((u4PhyAddr << PHY_ADDR_SHIFT) & PHY_ADDR);
   u4Data |= ((u4RegAddr << PHY_REG_SHIFT) & PHY_REG);
   u4Data |= WT_CMD;
   vWriteMAC(RW_PHY_CTRL0, u4Data);

    u4TimeOut = 0xffff;
    while((dReadMAC(RW_PHY_CTRL0) & RW_OK) == 0) // Wait for R/W command complete
    {
        u4TimeOut --;
        if(u4TimeOut == 0x00) {
            TR("MII write PHY timeout\n");
            return(NET_DRV_HW_ERROR);
        }
    }
    SetBitMAC(RW_PHY_CTRL0, RW_OK);  //clear RW_OK

    return(NET_DRV_OK);
}

#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8581)
u16 i4MacPhyRead45(u32 phyAddr, u32 phyReg , u32 devName)
{
	u32 phyCtl;
	u16 data;
	u32 delay=0x400000;

	vWriteMAC(RW_PHY_CTRL0, RW_OK); /* Clear previous read/write OK status (write 1 clear) */
	phyCtl = (  ((phyAddr & 0x1f) << 0) |  //PHY ID
	            ((phyReg & 0xffff) << 16) | //
	            ((devName & 0x1f) << 8)|
	            0x00 |(1<<5) /* set dev/address for prepare read*/
	          );

	vWriteMAC(RW_PHY_CTRL0, phyCtl);

	while (((dReadMAC(RW_PHY_CTRL0)&RW_OK)==0))
	{
		delay --;
        if(delay == 0x00)
        {
            TR("MII clause45 read PHY timeout\n");
            return(NET_DRV_HW_ERROR);
        }
	}; /* Wait for read/write ok bit is asserted */
    delay = 0x400000;
    
	vWriteMAC(RW_PHY_CTRL0, RW_OK); /* Clear previous read/write OK status (write 1 clear) */
	phyCtl = (  ((phyAddr & 0x1f) << 0) |
	            ((devName & 0x1f) << 8)|
	            0xc0 |(1<<5) /* set read Command */
	          );

	vWriteMAC(RW_PHY_CTRL0, phyCtl);

	while (((dReadMAC(RW_PHY_CTRL0)&RW_OK)==0))
	{
		delay --;
        if(delay == 0x00)
        {
            TR("MII clause45 read PHY timeout\n");
            return(NET_DRV_HW_ERROR);
        }
	}; /* Wait for read/write ok bit is asserted */
    data = ((dReadMAC(RW_PHY_CTRL0)>>16)&(0xffff));

	return data;
}

// *********************************************************************
// Function    : void i4MacPhyWrite45(u32 phyAddr, u32 phyReg, u32 devName, u16 value)
// Description : MII PHY register write clause45
// Parameter   : u4PhyAddr, phyReg, devName, value
// Return      : none
// *********************************************************************
void i4MacPhyWrite45(u32 phyAddr, u32 phyReg, u32 devName, u16 value)
{
	u32 phyCtl;
	u32 delay=0x400000;

	vWriteMAC(RW_PHY_CTRL0, RW_OK); /* Clear previous read/write OK status (write 1 clear) */
	phyCtl = (  ((phyAddr & 0x1f) << 0) |  //PHY ID
	            ((phyReg & 0xffff) << 16) | //
	            ((devName & 0x1f) << 8)|
	            0x00 |(1<<5) /* set dev/address for prepare read*/
	          );

	vWriteMAC(RW_PHY_CTRL0, phyCtl);

	while (((dReadMAC(RW_PHY_CTRL0)&RW_OK)==0))
	{
		delay --;
        if(delay == 0x00)
        {
            TR("MII clause45 write PHY timeout\n");
            return(NET_DRV_HW_ERROR);
        }
	}; /* Wait for read/write ok bit is asserted */
    delay = 0x400000;

	vWriteMAC(RW_PHY_CTRL0, RW_OK); /* Clear previous read/write OK status (write 1 clear) */
	phyCtl = (  ((phyAddr & 0x1f) << 0) |
	            ((devName & 0x1f) << 8)|
	            ((value & 0xffff) << 16)|
	            0x40 |(1<<5) /* set write Command */
	          );

	vWriteMAC(RW_PHY_CTRL0, phyCtl);

	while (((dReadMAC(RW_PHY_CTRL0)&RW_OK)==0))
	{
		delay --;
        if(delay == 0x00)
        {
            TR("MII clause45 read PHY timeout\n");
            return(NET_DRV_HW_ERROR);
        }
	}; /* Wait for read/write ok bit is asserted */

}
#endif

// *********************************************************************
// Function    : void i4MacIntPhyInit(UINT32 u4PhyAddr)
// Description : PHY device SW init
// Parameter   : u4PhyAddr
// Return      : none
// *********************************************************************
INT32 i4MacIntPhyInit(UINT32 u4PhyAddr)
{
    INT32 i4Ret,u4Data,u1InterPhyAddr;
	INT32 data1,data2;
	UINT32	TempLow,TempHi;

	/* Init internal PHY settings */
	//test page
	i4MacPhyWrite(u4PhyAddr, 0x1F, 0x2a30);
	i4MacPhyWrite(u4PhyAddr, 0x08, 0x0210); //disable gating of RCLK125 according to YT's suggest
	i4MacPhyWrite(u4PhyAddr, 0x0E, 0x35); //analog

	//clock edge choose
	i4MacPhyRead(u4PhyAddr,0x03,&data1);
    i4MacPhyRead(u4PhyAddr,0x1E,&data1);
	i4MacPhyWrite(u4PhyAddr, 0x03, data1&(~(1<<6))); /* PLL Control register[6]=TCLK125M_EDGE_SEL=0 , POSITIVE EDGE */
	i4MacPhyWrite(u4PhyAddr, 0x1E, data2&(~(1<<2))); /* ANARG Register[2]     =RG_TCK250_EDG_SEL=0, NEGATIVE EDGE */


    //extend page
    i4MacPhyWrite(u4PhyAddr, 0x1F, 0x01);
    i4MacPhyWrite(u4PhyAddr, 0x1b, 0x1a0); //enable auto-crossover as autoneg is disabled

	//work around for SLT fail issue for 8560/8580
	/////patch for SLT fail IC (Rx ADC not good)
	i4MacPhyWrite(u4PhyAddr, 0x1f, 0x2a30);
	i4MacPhyRead(u4PhyAddr,0x09,&data1);
	i4MacPhyWrite(u4PhyAddr, 0x09, ((1<<15)|(1<<14)|data1));

	i4MacPhyWrite(u4PhyAddr, 0x1f, 0x52b5);
	i4MacPhyWrite(u4PhyAddr, 0x10, 0xafa4);
	i4MacPhyRead(u4PhyAddr,0x11,&TempLow);
	i4MacPhyRead(u4PhyAddr,0x12,&TempHi);
	TempHi = (TempHi & (~(0x7f<<1)))|(0x2D<<1);
	i4MacPhyWrite(u4PhyAddr, 0x11, TempLow);
	i4MacPhyWrite(u4PhyAddr,0x12, TempHi);
	i4MacPhyWrite(u4PhyAddr, 0x10, 0x8fa4);
	i4MacPhyWrite(u4PhyAddr, 0x1f, 0x00);


    //main page
    i4MacPhyWrite(u4PhyAddr, 0x1F, 0x00);
    i4MacPhyWrite(u4PhyAddr, 0x17, 0x0020); //ACTI_EN for power save when link down
    i4Ret = i4MacPhyWrite(u4PhyAddr, 0x1F, 0x0); //return to main page.
    i4MacPhyWrite(u4PhyAddr, 0x00, 0x3100);

    if(i4Ret != NET_DRV_OK){
        TR("ethernet internal PHY init fail\n");
    }

    return(i4Ret);
}

INT32 i4InterPhyClockExtSource(BOOL fgExtClock)
{
  UINT32 u4Data;

	if(fgExtClock == TRUE) {
		i4MacPhyWrite(0x00, 0x1f, 0x2a30);
		i4MacPhyRead(0x00, 0x0f, &u4Data);
		u4Data |= (1<<8);
		i4MacPhyWrite(0x00, 0x0f, u4Data);

		i4MacPhyRead(0x00, 0x01, &u4Data);
		u4Data |= (1<<3);
		i4MacPhyWrite(0x00, 0x01, u4Data);
		i4MacPhyWrite(0x00, 0x1f, 0x0);
	}

    return (NET_DRV_OK);

}


// *********************************************************************
// Function    : i4MacPhyAddrGet
// Description : Get PHY physical address by r/w register result
// Parameter   : prMac
// Return      : 0 on success, others on failure
// *********************************************************************
u8  u8MacPhyAddrGet(void)
{
    return  0;
}

void vMacInit(void)
{
	u32 u4Data;//bit3~bit7
	u8  u8PhyAddr=0;

	u4Data= (dReadMAC(RW_INTERNAL_PHY_CTRL)& (~INTERNAL_PHY_ADDRESS_MASK));
	vWriteMAC(RW_INTERNAL_PHY_CTRL,u4Data|(u8PhyAddr <<INTERNAL_PHY_ADDRESS_POS));
	SetBitMAC(RW_PHY_CTRL1, AUTO_POLL_DIS);//disable Auto Polling when set PHY register
#if (CONFIG_CHIP_VER_CURR != CONFIG_CHIP_VER_MT8581)
	i4MacIntPhyInit(u8PhyAddr);  //init internal PHY
#endif
	ClrBitMAC(RW_PHY_CTRL1, AUTO_POLL_DIS);//enable Auto Polling after set PHY register

	/* Init MAC setting */
	vWriteMAC(RW_MAC_CFG, MAC_CFG_INIT);

	vWriteMAC(RW_ARL_CFG, MAC_FILTER_INIT);

	/* For first time to use the PHY */
	vWriteMAC(RW_PHY_CTRL1, PHY_CTRL_INIT | ((u8PhyAddr << PHY_ADDR_AUTO_SHIFT) & PHY_ADDR_AUTO));
}

static void StarMibReset(void)
{
    vWriteMAC(RW_C_RXOKPKT, 0);
    vWriteMAC(RW_C_RXOKBYTE, 0);
    vWriteMAC(RW_C_RXRUNT, 0);
    vWriteMAC(RW_C_RXLONG, 0);
    vWriteMAC(RW_C_RXDROP, 0);
    vWriteMAC(RW_C_RXCRC, 0);
    vWriteMAC(RW_C_RXARLDROP, 0);
    vWriteMAC(RW_C_RXVLANDROP, 0);
    vWriteMAC(RW_C_RXCSERR, 0);
    vWriteMAC(RW_C_RXPAUSE, 0);
    vWriteMAC(RW_C_TXOKPKT, 0);
    vWriteMAC(RW_C_TXOKBYTE, 0);
    vWriteMAC(RW_C_TXPAUSE, 0);
}

#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563)
static void vMtcmosPoweron(void)
{
	unsigned long val = 0, idx=0;
	#define MAX_POLL_TIME (100)

	val = PDWNC_READ32(REG_POWER_ON_ACK);
	if((val & ETH_PWR_ON_ACK_S) && (val & ETH_PWR_ON_ACK))
		return;

	MTCMOS_CTRL_SET(ETH_PWR_ON_S);
	/* polloing Ethernet Power on ACK: wait 0x2409c[1]==1 */
	while((idx++ < MAX_POLL_TIME) && !(PDWNC_READ32(REG_POWER_ON_ACK)&(0x1<<1)))
	{
	}

	if(idx > MAX_POLL_TIME)
		printf("ERROR: set ETH_PWR_ON_S,no ack\n");

	idx = 0;
	MTCMOS_CTRL_SET(ETH_PWR_ON);
	/* polloing Ethernet Power on 2 ACK: wait 0x2409c[2]==1 */
	while((idx++ < MAX_POLL_TIME) && !(PDWNC_READ32(REG_POWER_ON_ACK)&(0x1<<2)))
	{
	}
	if(idx > MAX_POLL_TIME)
		printf("ERROR: set ETH_PWR_ON,no ack\n");

	MTCMOS_CTRL_Clr(ETH_MEM_PD);

	MTCMOS_CTRL_Clr(ETH_PWR_CLKOFF);

	MTCMOS_CTRL_Clr(ETH_PWR_ISO);

	MTCMOS_CTRL_SET(ETH_PWR_RST_B);
}
#endif
static void vEnableEthPwrClk(void)
{
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8561)
	//set 24038[8,1:0] open ethernet power
	PDWNC_WRITE32(PDWNC_REG_SOFT_RESET, PDWNC_READ32(PDWNC_REG_SOFT_RESET)|(RG_ETHER_XPLL_PWD|ENET_PHY_RSTB|ENET_NIC_RSTB));
	PDWNC_WRITE32(PDWNC_REG_ETCLKCTL, PDWNC_READ32(PDWNC_REG_ETCLKCTL)&(~(ET_AFT_PWD)));
	TR("0x24038=0x%x,0x2404c=0x%x\n",PDWNC_READ32(PDWNC_REG_SOFT_RESET),PDWNC_READ32(PDWNC_REG_ETCLKCTL));
#elif (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580)
	PDWNC_WRITE32(PDWNC_REG_SOFT_RESET, PDWNC_READ32(PDWNC_REG_SOFT_RESET)|(ENET_PHY_RSTB|ENET_NIC_RSTB));
	PDWNC_WRITE32(PDWNC_REG_ETCLKCTL, PDWNC_READ32(PDWNC_REG_ETCLKCTL)&(~(ET_AFT_PWD)));
#elif (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563)
	if(!BIM_READ32(REG_RW_GPRB1))//8563 E2 and E1
	{
		/*8563 E2/E1 need set LDO as the highest value 1.25v to workaround ethernet cannot linkup issue*/
		PDWNC_WRITE32(PDWNC_REG_LDO_CFG,
			(PDWNC_READ32(PDWNC_REG_LDO_CFG)&(~RG_LDO_VSEL_MASK))|(RG_LDO_VSEL_1_25));
		TR("PDWNC_REG_LDO_CFG=0x%x\n",PDWNC_READ32(PDWNC_REG_LDO_CFG));
	}

	PDWNC_WRITE32(PDWNC_REG_ETCLKCTL,
		(PDWNC_READ32(PDWNC_REG_ETCLKCTL)&(~(ET_AFT_PWD)&
		(~ETH_TXT_TCLK_INV)&(~ETH_DACG_TCLK_INV))) | ETH_NIC_CKEN);

	/* ethernet module cpu clock start */
	PDWNC_WRITE32(REG_RW_CLKPDN, PDWNC_READ32(REG_RW_CLKPDN) & (~RW_ENET_PDN));

	PDWNC_WRITE32(PDWNC_REG_SOFT_RESET, PDWNC_READ32(PDWNC_REG_SOFT_RESET)|(ENET_PHY_RSTB|ENET_NIC_RSTB));
	TR("0x24038=0x%x,0x2404c=0x%x\n",PDWNC_READ32(PDWNC_REG_SOFT_RESET),PDWNC_READ32(PDWNC_REG_ETCLKCTL));

	vMtcmosPoweron();
#endif
}
/**********************************************************
 * device cfg related functions
 **********************************************************/
void hwcfgsetup(void)
{

	/* Init pdwnc to enable Ethernet engine accessible by ARM only:
	  * 0: Accessed by ARM
	 * 1: Accessed by T8032
	 */
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8581)
		vWriteMAC(0x300, dReadMAC(0x300)&(~(1<<16))); 
		PDWNC_WRITE32(0x048, PDWNC_READ32(0x048)&(~(1<<1)));
#endif
	PDWNC_WRITE32(PDWNC_REG_UP_CONF_OFFSET, PDWNC_READ32(PDWNC_REG_UP_CONF_OFFSET)&(~UP_CONF_ETEN_EN));
	
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8581)
	PDWNC_WRITE32(PDWNC_REG_SOFT_RESET, PDWNC_READ32(PDWNC_REG_SOFT_RESET)|0x103);

    ClrBitMAC(0x0308, 1);
	mdelay(1);
    SetBitMAC(0x0334, 1);
	mdelay(3);
    SetBitMAC(0x0308, 1);
	mdelay(4);

	i4MacPhyWrite45(0, 0x0104, 0x1f, 0x0608); //disable pll auto calibration
    i4MacPhyWrite45(0, 0x0424, 0x1f, 0x0000);
    i4MacPhyWrite45(0, 0x010d, 0x1f, 0x0000);
    i4MacPhyWrite45(0, 0x041c, 0x1f, 0x0000);
    i4MacPhyWrite45(0, 0x0424, 0x1f, 0x0001);
    i4MacPhyWrite45(0, 0x010d, 0x1f, 0x0001);
    i4MacPhyWrite45(0, 0x0424, 0x1f, 0x0011);
    i4MacPhyWrite45(0, 0x041c, 0x1f, 0x0010);
    i4MacPhyWrite45(0, 0x010d, 0x1f, 0x0005);
    i4MacPhyWrite45(0, 0x041c, 0x1f, 0x0110);

    i4MacPhyWrite(0, 0x00, 0x8000);
	mdelay(20);

    ClrBitMAC(0x0308, 1);
	mdelay(1);
    ClrBitMAC(0x0334, 1);
	mdelay(3);
    SetBitMAC(0x0308, 1);
	mdelay(4);
    
	TR("PHY PLL Init done.\n");

	i4MacPhyWrite45(0, 0x144, 0x1e, 0x8620);
        
    i4MacPhyWrite45(0, 0x40, 0x1e, 0x2000); 

    i4MacPhyWrite45(0, 0x27b, 0x1f, 0x0177);

    i4MacPhyWrite45(0, 0x41, 0x1e, 0x1111);

    i4MacPhyWrite45(0, 0x170, 0x1e, 0x1111);

    i4MacPhyWrite45(0, 0x267, 0x1f, 0x3333);

    i4MacPhyWrite45(0, 0x268, 0x1f, 0x0384);

    i4MacPhyWrite45(0, 0x269, 0x1f, 0x4448);

    i4MacPhyWrite45(0, 0x26a, 0x1f, 0x1119);

    i4MacPhyWrite45(0, 0x26b, 0x1f, 0x0094);

    i4MacPhyWrite45(0, 0x26c, 0x1f, 0x4444);

    i4MacPhyWrite45(0, 0x271, 0x1f, 0x7210);

    i4MacPhyWrite45(0, 0x272, 0x1f, 0x421c);

    i4MacPhyWrite45(0, 0x27c, 0x1f, 0x0808);

    i4MacPhyWrite45(0, 0x302, 0x1f, 0x0010);

    i4MacPhyWrite45(0, 0xc8, 0x1e, 0x4455);

    i4MacPhyWrite45(0, 0x28, 0x1e, 0x243f);
    
    i4MacPhyWrite45(0, 0x4, 0x1e, 0x0208);
    
    i4MacPhyWrite45(0, 0xa, 0x1e, 0x0008);
    
    i4MacPhyWrite45(0, 0x3c, 0x07, 0x0);

	TR("GBE PHY Setting done!\n");
#endif

	vEnableEthPwrClk();


	vWriteMAC(0x94, 0x08);
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563)
	/*workaroud for pdwnc hardware bug:if write pdwnc register, the written register followed by it will be write fail*/
	{
		u32 value = dReadMAC(0x94);
		TR("net 0x94 value is 0x%x\n", value);
	}
#endif
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8581)
		vWriteMAC(0x9C, 0x7F);		
		i4MacPhyWrite(0, 0x09, 0x0000);
		i4MacPhyWrite(0, 0x00, 0x3300);
#else
		vWriteMAC(0x9C, 0x1F); 
#endif

	vDmaInit();
	vMacInit();

	vWriteMAC(RW_ARL_CFG, MAC_FILTER_INIT);
	//  vWriteMAC(RW_PHY_CTRL1, PHY_CTRL_INIT | ((0 << PHY_ADDR_AUTO_SHIFT) & PHY_ADDR_AUTO));
	vWriteMAC(RW_DLY_INT_CFG, DMA_INT_ENABLE_ALL); ///clear all int mask

	StarNICPdSet(0) ;

	MmacDmaRxStop(NULL); 	 /* stop receiver, take ownership of all rx descriptors */
	MmacDmaTxStop(NULL); 	 /* stop transmitter, take ownership of all tx descriptors */

	/* init carrier to off */
	//netif_carrier_off(dev);


	SetBitMAC(RW_ETHPHY,MII_PAD_OE);
	SetBitMAC(RW_ETHPHY,FRC_SMI_ENB);
	SetBitMAC(RW_TEST1,TEST1_EXTEND_RETRY);

	/* Set Mac Configuration */
#ifdef CHECKSUM_OFFLOAD
	vWriteMAC(RW_MAC_CFG,
			  RX_CKS_EN |
			  CRC_STRIPPING |
			  MAX_LEN_1522 |
			  IPG /* 12 byte IPG */
			);
#else
	vWriteMAC( RW_MAC_CFG,
			  CRC_STRIPPING |
			  MAX_LEN_1522 |
			  IPG /* 12 byte IPG */
			);
#endif

	SetBitMAC(RW_FC_CFG, SEND_PAUSE_TH_2K|UC_PAUSE_DIS|BP_ENABLE);
	SetBitMAC(RW_EXTEND_CFG,SEND_PAUSE_RLS_1K);

	StarMibReset();

	/* Enable Hash Table BIST */
	SetBitMAC(RW_HASH_CTRL, HASH_TABLE_BIST_START);
	/* Reset Hash Table (All reset to 0) */
	SetBitMAC(RW_TEST1,RESTART_HASH_MBIST);

	iPhyCtrlInit(1);

}


/**********************************************************
 * Common functions
 **********************************************************/

int MmacInit( Mmac *Dev, u32 ConfigBase, u32 MacBase, u32 DmaBase, u32 PhyAddr )
{
	memset( Dev, 0, sizeof(*Dev) );

	Dev->configBase = ConfigBase;
	Dev->macBase = MacBase;
	Dev->dmaBase = DmaBase;
	Dev->phyAddr = PhyAddr;

	return 0;
}

int MmacMacInit( Mmac *Dev, u8 Addr[6], u8 Broadcast[6] )
{
	/* Set Mac Address */
	vWriteMAC(RW_MY_MAC_H, (Addr[0]<<8) | (Addr[1]<<0));
	vWriteMAC(RW_MY_MAC_L, (Addr[2]<<24) | (Addr[3]<<16) | (Addr[4]<<8) | (Addr[5]<<0));

	if(Dev->full_duplex) {
	  SetBitMAC(RW_PHY_CTRL1, FORCE_DUPLEX_FULL);
	} else {
	  ClrBitMAC(RW_PHY_CTRL1, FORCE_DUPLEX_FULL);
	}

	/* 100/10 Mbps selection */
	if(Dev->speed_100){
	  SetBitMAC(RW_PHY_CTRL1, FORCE_SPEED_100M);
	} else {
	  ClrBitMAC(RW_PHY_CTRL1, FORCE_SPEED_10M);
	}

	return 0;
}

/* =========================================================
   DMA related functions
   ========================================================= */
static void DescTxInit(TxDesc *txDesc, u32 isEOR)
{
	txDesc->buffer = 0;
	txDesc->ctrlLen = TX_COWN | (isEOR ? TX_EOR : 0);
	txDesc->vtag = 0;
	txDesc->reserve = 0;
}

static void DescRxInit(RxDesc *rxDesc, u32 isEOR)
{
	rxDesc->buffer = 0;
	rxDesc->ctrlLen = RX_COWN | (isEOR ? RX_EOR : 0);
	rxDesc->vtag = 0;
	rxDesc->reserve = 0;
}

static void DescTxTake(TxDesc *txDesc)     /* Take ownership */
{
	if (DescTxDma(txDesc)) {
		txDesc->ctrlLen |= TX_COWN;       /* Set CPU own */
	}
}

static void DescRxTake(RxDesc *rxDesc)     /* Take ownership */
{
	if (DescRxDma(rxDesc)){
		rxDesc->ctrlLen |= RX_COWN;       /* Set CPU own */
	}
}

int MmacDmaInit( Mmac *dev, TxDesc *tdesc, RxDesc *rdesc )
{
	int i;

	dev->txRingSize = NUM_TX_DESC;
	dev->rxRingSize = NUM_RX_DESC;

	dev->txdesc = (TxDesc *)((u32)tdesc & (~(DMA_ALIGN_SIZE-1)));
	dev->rxdesc = (RxDesc *)((u32)rdesc & (~(DMA_ALIGN_SIZE-1)));

	for (i=0; i<dev->txRingSize; i++) { DescTxInit(dev->txdesc + i, i==dev->txRingSize-1); }
	for (i=0; i<dev->rxRingSize; i++) { DescRxInit(dev->rxdesc + i, i==dev->rxRingSize-1); }

	for (i=0; i<dev->rxRingSize; i++) DescRxTake(dev->rxdesc + i);
	for (i=0; i<dev->txRingSize; i++) DescTxTake(dev->txdesc + i);

	dev->rxReadIdx = 0;
	dev->txWrIdx = 0 ;

	/* Set Tx/Rx descriptor address */
	vWriteMAC(RW_TX_BASE_ADDR, (u32)dev->txdesc);
	vWriteMAC(RW_TX_DPTR, dReadMAC(RW_TX_BASE_ADDR));

	vWriteMAC(RW_RX_BASE_ADDR, (u32)dev->rxdesc );
	vWriteMAC(RW_RX_DPTR, dReadMAC(RW_RX_BASE_ADDR));

	/* Enable Rx 2 byte offset (FIXME) */
 	/* Init DMA_CFG, Note: RX_OFFSET_2B_DIS is set to 0 */
	vWriteMAC(RW_DMA_CFG,RX_OFFSET_2B_DIS);

 	MmacDmaIntDisable(NULL);
	return 0;
}


int StarDmaTxSet(Mmac *dev, u32 buffer, u32 length, u32 extBuf)
{
	int descIdx = dev->txWrIdx;
	TxDesc *txDesc = dev->txdesc + descIdx;

	/* Error checking */
	if(DescTxDma(txDesc))  goto err;

	txDesc->buffer = buffer;
	txDesc->ctrlLen &= ~(TX_LEN_MASK<<TX_LEN_OFFSET);
	txDesc->ctrlLen |= ((((length < 60)?60:length) & TX_LEN_MASK) << TX_LEN_OFFSET)| TX_FS | TX_LS   /*Tx Interrupt Enable*/;
	txDesc->reserve = extBuf;
	txDesc->vtag = 0;
	//wmb();
	txDesc->ctrlLen &= ~TX_COWN;     /* Set HW own */

	TR(  "Mmac::send () decIdx=%d; buffer=%x; len=%x; \n",descIdx,buffer,length );

    dev->txWrIdx = DescTxLast(txDesc) ? 0 : descIdx + 1;
	return descIdx;

err:
	return -1;
}

void StarDmaRxSet(Mmac *dev, u32 buffer, u32 length, u32 extBuf, u32 dwIdx)
{
	RxDesc *rxDesc = dev->rxdesc + dwIdx;

	rxDesc->buffer = buffer;
	rxDesc->ctrlLen &= (~(RX_LEN_MASK<<RX_LEN_OFFSET)) ;
	rxDesc->ctrlLen |= ((length & RX_LEN_MASK) << RX_LEN_OFFSET);
	rxDesc->reserve = extBuf;
	//wmb();
	rxDesc->ctrlLen &= ~RX_COWN;       /* Set HW own */
}

int StarDmaRxGet(Mmac *dev /*, u32 *buffer, u32 *ctrlLen, u32 *extBuf */)
{
	int descIdx = dev->rxReadIdx;
	RxDesc *rxDesc = dev->rxdesc + descIdx;


	if (DescRxDma(rxDesc))   goto err;
	if (DescRxEmpty(rxDesc)) goto err;        /* descriptor is empty - cannot get */

	if (StarDmaRxValid(rxDesc->ctrlLen)) /* process good packets only */
    {
        u32 len = StarDmaRxLength(rxDesc->ctrlLen); /* ignore Ethernet CRC bytes */
        TR(  "receive good packet,addr=%x ; len=%d \n",rxDesc->buffer,len);
		NetReceive(rxDesc->buffer, len);
    }

	rxDesc->vtag = 0;
	rxDesc->reserve = 0;
	rxDesc->ctrlLen &= (RX_COWN | RX_EOR) ;
	rxDesc->ctrlLen |= ((RX_PACKET_BUFFER_SIZE & RX_LEN_MASK) << RX_LEN_OFFSET);
	rxDesc->ctrlLen &= ~RX_COWN;       /* Set HW own */

	dev->rxReadIdx = DescRxLast(rxDesc) ? 0 : descIdx + 1;

	return descIdx;
err:
	return -1;
}


void MmacRxDescInit(struct eth_device *dev, Private *pr, Mmac *mmac )
{
  int r = 0;
  int i = 0;
   /* prepare recieve descriptors */
  for(i = 0; i<NUM_RX_DESC ;i++)
  {
    if (pbuf == NULL) {
      TR("Mmac::(%s) - no memory for sk_buff\n", dev->name );
      break;
    }
	StarDmaRxSet( mmac, (u32)(pbuf+(i*RX_PACKET_BUFFER_SIZE)), RX_PACKET_BUFFER_SIZE, (u32)0,i );
    TR(  "Mmac::StarDmaRxSet OK  \n") ;
  }

}

void MmacDmaRxStart( Mmac *Dev )
{
   vDmaRxStartAndResetRXDesc;
}

void MmacDmaRxStop( Mmac *Dev )
{
	StarDmaRxDisable;
}

void MmacDmaTxStart( Mmac *Dev )
{
    vDmaTxStartAndResetTXDesc;
}

void MmacDmaTxStop( Mmac *Dev )
{
    StarDmaTxDisable;
}

void MmacDmaRxResume( Mmac *Dev )
{
	StarDmaRxEnable;
}

void MmacDmaTxResume( Mmac *Dev )
{
	StarDmaTxEnable;
}

/**********************************************************
 * DMA engine interrupt handling functions
 **********************************************************/
void MmacDmaIntEnable( Mmac *Dev )
{
	vWriteMAC(RW_INT_MASK, 0);               	// enable interrupts
}

void MmacDmaIntDisable( Mmac *Dev )
{
	vWriteMAC(RW_INT_MASK, DMA_INT_ENABLE_ALL);				  // disable interrupts
}

//***********************************************************
// 1. initial chip and board configuration.
// 2. initialize eth_device
//        -function pointer.
//        -priv date
// 3. register eth_device
// 4. mac addr...

int mmac_initialize(bd_t * bis)
{
    struct eth_device *dev;
    Private *pr = NULL;
    u32 macBase    = MAC_BASE;
    u32 dmaBase    = DMA_BASE;
    u32 phyAddr    = DEFAULT_PHY_ADDRESS;
    u8  macAddr[6] = DEFAULT_MAC_ADDRESS;

    TR(  "init MMAC driver\n");
    hwcfgsetup();

    dev = (struct eth_device *) malloc (sizeof *dev);
    if (!dev) return -1;//ENOMEM;

    if (!pbufOrg){
      pbufOrg = malloc((NUM_RX_DESC *RX_PACKET_BUFFER_SIZE) + DMA_ALIGN_SIZE);
      if (!pbufOrg){
         TR("Cannot allocate rx buffers\n");
         return -1;
      }
      pbuf = (uchar*)(((u32)pbufOrg + (u32)(DMA_ALIGN_SIZE-1)) & ~((u32)(DMA_ALIGN_SIZE-1)));
    }
    pr = malloc( sizeof(Private) + DMA_ALIGN_SIZE*2);
    if( pr == NULL ) return -1;//ENOMEM;

    TR("priv : 0x%08x\n",(u32)pr);
    if((u32)pr & (u32)0x0F)
    {
      pr = (Private*)(((u32)pr + (u32)(DMA_ALIGN_SIZE-1)) & ~((u32)(DMA_ALIGN_SIZE-1)));
    }

    dev->priv = (void *) pr;//devno;
    dev->init = mmac_init;
    dev->halt = mmac_stop;
    dev->send = mmac_send;
    dev->recv = mmac_receive;

    eth_register (dev);

    phyAddr = u8MacPhyAddrGet();
    TR(  "Mmac::PhyAddr=%x \n",phyAddr);

    MmacInit( &pr->mmac, 0, macBase, dmaBase, phyAddr ); /* init Mmac internal data */

    memcpy( dev->enetaddr, macAddr, 6 );       /* Set MAC address */
    TR("Mmac::initialize %x:%x:%x:%x:%x:%x\n", macAddr[0], macAddr[1],macAddr[2],macAddr[3],macAddr[4],macAddr[5]);

    phy_reset(&pr->mmac);
	TR(  "Mmac::initialize hank set pd\n");
#if (CONFIG_CHIP_VER_CURR != CONFIG_CHIP_VER_MT8581)
	StarNICPdSet(1) ;
#endif

    return 0;
}

static int mmac_receive( struct eth_device *dev )       /* handle received packets */
{
  Private *pr = NULL;
  Mmac *tc = NULL;
  int r = 0;
  int len;

  if(dev == NULL)
     return -1;

  pr = (Private *)dev->priv;
  if(pr == NULL)
        return -1;

  tc = &pr->mmac;
  if(tc == NULL)
      return -1;

  do   /* handle recieve descriptors */
  {
	r = StarDmaRxGet(tc);
    MmacDmaRxResume(NULL);

  } while( r >= 0 );

  return len;
}

void StarNICPdSet( u32 val)
{
    u32 data;
    int retry = 0;
    #define MAX_NICPDRDY_RETRY  10000

    data = dReadMAC(RW_MAC_CFG);
    if (val) {
        data |= NIC_PD;
		data &= ~NIC_PD_READY;
        vWriteMAC(RW_MAC_CFG, data);
        /* wait until NIC_PD_READY and clear it */
        do
        {
            if ((data = dReadMAC(RW_MAC_CFG)) & NIC_PD_READY){
                data |= NIC_PD_READY; /* clear NIC_PD_READY */
                vWriteMAC(RW_MAC_CFG, data);
                break;
            }
        } while(retry++ < MAX_NICPDRDY_RETRY);
        if (retry >= MAX_NICPDRDY_RETRY) {
            TR("Error NIC_PD_READY is not set in %d retries!\n", MAX_NICPDRDY_RETRY);
        }
    } else {
        data &= (~NIC_PD );
        vWriteMAC(RW_MAC_CFG, data);
    }
}

int iPhyCtrlInit(u32 enable)
{
	u32 data;

	data = FORCE_FC_TX |
	       FORCE_FC_RX |
	       FORCE_DUPLEX |
	       FORCE_SPEED_100M |
	       AN_EN;

	if (enable) {
		vWriteMAC( RW_PHY_CTRL1,
		            data  /*| ( PHY_ADDR_AUTO)*/ );
	} else {
		vWriteMAC( RW_PHY_CTRL1,
		            data | AUTO_POLL_DIS );
	}

	return 0;
}

static u32 link_delay_limit = 50;
static int mmac_init(struct eth_device *dev, bd_t *bis)
{
	Private *pr = NULL;
	Mmac *tc = NULL;
	u8 hw_addr[6];
	u32 delay_times=0;

	if(dev == NULL)
		return -1;

	pr = (Private *)dev->priv;
	if(pr == NULL)
		return -1;

	tc = &pr->mmac;
	if(tc == NULL)
		return -1;

	eth_getenv_enetaddr("ethaddr", hw_addr);
	memcpy(dev->enetaddr, hw_addr, 6);

#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563)
	/*IC before ECO, if 0x269, it will be ECO IC*/
	if(IO_READ(CKGEN_BASE+0) == 0x268)
	{
	    /*workaround for 8563 pdwnc issue, Disable FE to prevent it operating pdwnc register*/
	    if(0 != PDWNC_READ32(0x34))
	        PDWNC_WRITE32(0x34,0);
	    printf("8563 IC before ECO\n");
	    vWriteMAC(0x1B4, 0);

	}
#endif
	TR(  "open ethernet \n" );
	StarNICPdSet(0) ;
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8581)
  i4MacPhyWrite(0, 0x09, 0x0000);
  i4MacPhyWrite(0, 0x00, 0x3300);
  mdelay(50);
#endif

	do
	{
		if(check_link(dev, tc) == 1)
			break;
		delay_times++;
		mdelay(100);
	}while(delay_times<link_delay_limit || (!link_delay_limit));

	TR( "delay %d ms  wait link up\n" ,delay_times*100);
	check_media(dev,pr, tc);

#if CFG_UBOOT_CUSTOM_JSN
	MmacDmaRxStop(tc);
	MmacDmaTxStop(tc);
#endif

#if CFG_UBOOT_CUSTOM_JSN
	StarNICPdSet(0) ;
#endif

	/* set and active a timer process */
	/* Init Mmac DMA Engine */
	if( MmacDmaInit( tc, pr->tdesc, pr->rdesc ) != 0 ){
		TR(  "Mmac::open(%s) - cannot init DMA\n", dev->name );
		return -1;//ENODEV;
	}

	MmacRxDescInit(dev,pr, tc);

	/* Init Mmac MAC module */
	if( MmacMacInit( tc, dev->enetaddr, BCmacAddr ) != 0 ) {
		TR(  "Mmac::open(%s) - cannot init MAC\n", dev->name );
		return -1;//ENODEV;
	}

	MmacDmaRxStart(tc);     /* start receiver */
	MmacDmaTxStart(tc);     /* start transmitter, it must go to suspend immediately */

	return 0;
}

static void mmac_stop( struct eth_device *dev )
{
  Private *pr = NULL;
  Mmac *tc = NULL;

  if(dev == NULL)
       return;

  TR(  "stop()\n");

  pr = (Private *)dev->priv;

  if(pr == NULL)
        return;

  tc = &pr->mmac;

  if(tc == NULL)
     return;

  MmacDmaIntDisable(tc);  /* disable all interrupts */
  MmacDmaRxStop(tc);      /* stop receiver, take ownership of all rx descriptors */
  MmacDmaTxStop(tc);      /* stop transmitter, take ownership of all tx descriptors */
  StarNICPdSet(1) ;

  return;
}

static int mmac_send(struct eth_device *dev, volatile void *packet, int length)
{
	Private *pr = NULL;
	Mmac *tc = NULL;
	u32 data1;
	u32 status;
	u32 len,i=0;

	int r = 0;

	if(dev == NULL)
		return -1;

	if(packet == NULL)
		return -1;

	pr = (Private *)dev->priv;

	if(pr == NULL)
	 return -1;

	tc = &pr->mmac;
	if(tc == NULL)
	 return -1;

	if(length > ETH_MAX_FRAME_SIZE) {
		TR(  "Mmac::send (%s) - length too large \n", dev->name );
		return -1;
	}

	r = StarDmaTxSet( tc, (u32)packet , length, (u32)packet );
	MmacDmaTxResume(dev);

	TR("exit (%s)\n",__FUNCTION__);

	return r ;

}

static int phy_reset(Mmac *Dev)
{
	INT32 i4Ret;

    i4Ret = i4MacPhyWrite(Dev->phyAddr, MII_PHY_CTRL_REG, SW_RESET);
    if(i4Ret != NET_DRV_OK)
    {
        TR("ethernet PHY reset fail 1 \n");
    }

    return(i4Ret);
}

int check_link (struct eth_device *dev, Mmac *mmac)
{
    int re_link = 0;
    int cur_link;

	cur_link = (dReadMAC(RW_PHY_CTRL1)& LINK_ST) ? TRUE : FALSE;

	if (cur_link && (!mmac->link_status)) {
        TR(  "  - link up\n");
        mmac->link_status = cur_link;
        re_link = 1;
	}
    else if (!cur_link) {
        TR(  "  - no link!\n");
        mmac->link_status = cur_link;
    }

 	return cur_link;
}

void check_media(struct eth_device *dev, Private *pr, Mmac *mmac)
{
	int PHY_lpa;
	int negotiated;
	int duplex;
	int i;
	UINT32 u4CtrlReg0,u4CtrlReg1;
	UINT32 u4AnlparReg5,u4AnlparReg4;

	if (check_link(dev, mmac) == 0){
		TR("check_media fail \n");
		return;
	}

	if(dReadMAC(RW_PHY_CTRL1)& DULPLEX_ST){
		mmac->full_duplex = TRUE;
	} else {
		mmac->full_duplex = FALSE;
	}

	if((dReadMAC(RW_PHY_CTRL1)& SPEED_ST) == SPEED_ST_10M){
		mmac->speed_100 = FALSE;
		mmac->speed_10  = TRUE ;
		TR("10Mbps Link\n");
	} else if((dReadMAC(RW_PHY_CTRL1)& SPEED_ST) == SPEED_ST_100M){
		mmac->speed_100 = TRUE;
		mmac->speed_10  = FALSE ;
		TR("100Mbps Link\n");
	}

}


/* link_delay_limit=0 is special case, means that limit is infinity */
static int do_net_set (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    if(strcmp (argv[1], "logon") == 0) {
        u32NetlogEnable =1;
        printf("set neton\n");
    } else if(strcmp (argv[1], "logoff") == 0){
        u32NetlogEnable = 0;
        printf("set netoff\n");
    } else if(strcmp (argv[1], "delayLimit") == 0){
        if(argc>2) {
            link_delay_limit = simple_strtoul(argv[2],NULL,10);
        }

        printf("link_delay_limit %d\n", link_delay_limit);
	}
}

U_BOOT_CMD(
	net,	5,	1,	do_net_set,
	"net   - net related setting\n",
	"net logon : enable net log\n"
	"net : log delay_limit\n"
);

