/*
 * (C) Copyright 2000
 * Rob Taylor, Flying Pig Systems. robt@flyingpig.com.
 *
 * (C) Copyright 2004
 * ARM Ltd.
 * Philippe Robin, <philippe.robin@arm.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/* Simple U-Boot driver for the PrimeCell PL011 UARTs on the IntegratorCP */
/* Should be fairly simple to make it work with the PL010 as well */

#include <common.h>
#include <asm/arch/x_bim.h>

#ifdef CFG_MT85XX_SERIAL
#include <asm/arch/serial_mt85xx.h>
#include <asm/arch/x_typedef.h>
#include "uart_config.h"

DECLARE_GLOBAL_DATA_PTR;

//-----------------------------------------------------------------------------
// Static variables
//-----------------------------------------------------------------------------
static UINT32 _u4SerIntMode = 0;

static CHAR _aszUartIntBuf[UART_INT_BUF_SIZE];
static UINT32 _aszUartIntBufRead = 0;
static UINT32 _aszUartIntBufWrite = 0;

static UINT8 _u1DebugPort = DBG_PORT;
static BOOL _fgIsMT5371 = TRUE;
static BOOL _fgSetoutbyte = TRUE;

UINT32 dwRWTEST1 = 0x11111111;
UINT32 dwRWTEST2 = 0x22222222;
UINT32 dwZITEST1;
UINT32 dwZITEST2;

static void mt85xx_putc (UINT8 c);
static int mt85xx_getc ();
static int mt85xx_tstc();
static UINT8 _BspSerGetRxDataCnt()
{
  UINT8 u1RxCnt;
  u1RxCnt = (UINT8)(REG_SER_STATUS & SER_READ_ALLOW);
  //u1RxCnt = 1;
  return u1RxCnt;
}

//-----------------------------------------------------------------------------
/** _BspSerIntGetChar() provides polling mode to get the character.
 *  _BspSerIntGetChar() will return 0xff if no character input, otherwise it
 *  poll and get the input character.
 *  @return the char which polled, or 0xff no character.
 */
//-----------------------------------------------------------------------------
static UINT8 _BspSerIntGetChar(void)
{
  UINT8 uReturnValue;

  if (_aszUartIntBufRead == _aszUartIntBufWrite)
  {
    return NULL;
  }

  uReturnValue = (UINT8)_aszUartIntBuf[_aszUartIntBufRead];

  _aszUartIntBufRead = (_aszUartIntBufRead + 1) % UART_INT_BUF_SIZE;

  return uReturnValue;
}

//-----------------------------------------------------------------------------
/** UINT8 SerSetoutbyte() set outbyte function enable/disable
 *  @parameter fgSet TRUE: enable outbyte(), FALSE: disable outbyte()
 */
//-----------------------------------------------------------------------------
void SerSetoutbyte(BOOL fgSet)
{
  _fgSetoutbyte = fgSet;
}

#if 0
#define IC_BEFORE_8563 ((CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8561) \
                      ||(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580) \
                      ||(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560) \
                      ||(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8555) \
                      ||(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8550))
#endif
void SerTransparent(void)
{
#if CONFIG_UBOOT_8X_LOG_DEFAULT 
      REG_SER_STATUS = 0x3E2;        // Set to transparent mode
#else
      REG_SER_STATUS = 0xE2;        // Set to transparent mode
#endif
  SerSetoutbyte(TRUE);
}

void SerIsrEnable(void)
{
  //Enable RS232 interrupt
  REG_SER_INT_EN = REG_SER_INT_EN_TX_D_ACEPTED | REG_SER_INT_EN_READ_ALLOW;
  
  //Enable UART interrupt
//  VERIFY(BIM_EnableIrq(VECTOR_RS232_1));
  
  //_u4SerIntMode = 1;  
}
//-----------------------------------------------------------------------------
/** void putchar(UINT8) put char, poll mode
 *
 */
//-----------------------------------------------------------------------------
void SerPollPutChar(UINT8 cc)
{
#if (!CONFIG_DISABLE_UART_LOG)&&(!CONFIG_UART_ROBUSTNESS)		
   BIM_GETHWSemaphore(HSMPHE_UART1, 0);
   
   while((REG_SER_STATUS & SER_WRITE_ALLOW) != SER_WRITE_ALLOW) {}
   
   REG_SER_PORT = cc;
   
   BIM_ReleaseHWSemaphore(HSMPHE_UART1);
#endif
}
//-----------------------------------------------------------------------------
/** void SerInit(void) init Serial port routine.
 *
 */
//-----------------------------------------------------------------------------
int serial_init(void)
{
 
    SerTransparent();   
    /* Enable RS232 interrupt */
//    REG_SER_INT_EN = REG_SER_INT_EN_TX_D_ACEPTED | REG_SER_INT_EN_READ_ALLOW;
//    REG_SER_IRQ = REG_SER_IRQ | REG_SER_IRQ_EN;
    SerIsrEnable();
  
	return 0;
}

void serial_putc (const char c)
{
	if (c == '\n')
		mt85xx_putc ('\r');

	mt85xx_putc (c);
}

#if CFG_UBOOT_CUSTOM_JSN
extern BOOL b_disable_log;
#endif
void serial_puts (const char *s)
{
#if CFG_UBOOT_CUSTOM_JSN
	if(b_disable_log)
		return 0;
#endif		
	while (*s) {
		serial_putc (*s++);
	}
}

int serial_getc (void)
{
	return mt85xx_getc ();
}

int serial_tstc (void)
{
	return mt85xx_tstc ();
}

void serial_setbrg (void)
{
    switch(gd->baudrate)
    {
        case 115200:
            REG_SER_STATUS = 0xE2;
            break;
        case 230400:
            REG_SER_STATUS = 0x1E2;
            break;
        case 460800:
            REG_SER_STATUS = 0x2E2;
            break;
        case 921600:
            REG_SER_STATUS = 0x3E2;
            break;
        default:
            printf("The baudrate is not supported!!!\n");
            break;
    }
}

static void mt85xx_putc (UINT8 c)
{
	static CHAR prev = 0;

	if (!_fgSetoutbyte)
	{
		return;
	}

	if ((c < ' ') && (c != '\r') && (c != '\n') && (c != '\t') && (c != '\b'))
	{
		return;
	}

	if ((c == '\n') && (prev != '\r'))
	{
		SerPollPutChar('\r');
	}
	prev = c;

	SerPollPutChar((UINT8)c);
}

static int mt85xx_getc ()
{
	if (_u4SerIntMode)
	{
		return _BspSerIntGetChar();
	}

	// if data NOT ready, return 0xFF
	if (_BspSerGetRxDataCnt() == 0)
	{
		return NULL;
	}

	return REG_SER01_PORT;
}

static int mt85xx_tstc()
{
	if(_BspSerGetRxDataCnt() > 0)
                return 1;
        else
                return 0;
}

#if CONFIG_BL_WITH_UART_OFF || CONFIG_AWM_ROBUSTNESS_FINAL_SOLUTION
void serial_disable(int idisable)
{
    if(idisable)
    {
    	#if CONFIG_AWM_ENABLE_TX_DISABLE_RX
        
    	REG_SER_INT_EN = REG_SER_INT_EN & 0x7f;
		  REG_SER_INT_ENT = REG_SER_INT_ENT & 0xfc;
    	#else
        REG_RS232_KEY_PROTECT = REG_RS232_KEY_PROTECT & (~REG_POWER_ON_PROTECT_MASK) | REG_POWER_ON_PROTECT;

        //set 0x24048[21:20] = 0x10, to disable RS232 to match key and to auto power on
        REG_PWDNC_UART_DBG = REG_PWDNC_UART_DBG & (~REG_PWDNC_DBG_PIN_MASK) | REG_PWDNC_DBG_USE_UART;
		#endif		
    }
    else
    {
        REG_RS232_KEY_PROTECT = REG_RS232_KEY_PROTECT & (~REG_POWER_ON_PROTECT_MASK);
        
        //set 0x24048[21:20] = 0x0, to enable RS232 to match key and to auto power on
        REG_PWDNC_UART_DBG = REG_PWDNC_UART_DBG & (~REG_PWDNC_DBG_PIN_MASK);
    }
}
#endif

#endif

