/*
 * mt85xx partition operation functions
 */
#ifndef MT85XX_PART_OPER_C
#define MT85XX_PART_OPER_C


/*
 * header file
 */
#include <common.h>
//#include <nand.h>
#include <malloc.h>
#include <asm/arch/x_typedef.h>

#include <asm/errno.h>
#include <asm/io.h>

#include <linux/mtd/mtd.h>
//#include <linux/mtd/nand.h>
//#include <linux/mtd/nand_ecc.h>
#include <linux/bitops.h>
#include <asm/arch/mt85xx_part_tbl.h>
#include "linux/mtd/mt85xx_part_oper.h"
#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>
#include <linux/err.h>
#ifdef CONFIG_EMMC_BOOT
#include <emmc.h>
#include <mtkfs.h>
#else
#endif
/* proc */

#define MAX_PAGE_SIZE	8192
#define ISUBIVOLUME(binInfo) ((0x0000F000 & binInfo)>> 12) 	//bit[1512]

#define ASSERT(e) \
        if(!(e)){ \
          while(1); \
        }
#define NAND_LOG_ERROR 0
#define NAND_LOG_INFO  1
#define NAND_LOG_DEBUG 2
INT32 NAND_LOG_LEVEL = NAND_LOG_DEBUG; //default log level

void NAND_LOG(int level, char *s, ...)
{
  if(level<= NAND_LOG_LEVEL)
  {
	va_list args;
	int i;
	char printbuffer[400];
	char *log="[NAND]";

	va_start (args, log);

	/* For this to work, printbuffer must be larger than
	 * anything we ever want to print.
	 */
	i = vsprintf (printbuffer, log, args);
	va_end (args);
	/* Print the string */
	puts (printbuffer);
   /*////////////////////*/
	va_start (args, s);

	/* For this to work, printbuffer must be larger than
	 * anything we ever want to print.
	 */
	i = vsprintf (printbuffer, s, args);
	va_end (args);
	/* Print the string */
	puts (printbuffer);
  }
}
UINT32 get_part_id_by_name(const char* part_name)
{
  PART_TBL_ITEM *r_part_tbl;
  if (part_name == NULL)
  {
    printk("get_part_id_by_name,name is NULL\n");
    return 0;
  }
  r_part_tbl = mt85xx_get_part_info_by_name(part_name);
  return r_part_tbl->u4_id;
}
UINT32 get_part_info_by_name(const char* part_name)
{
  PART_TBL_ITEM *r_part_tbl;
  if (part_name == NULL)
  {
    printk("get_part_info_by_name,name is NULL\n");
    return 0;
  }
  r_part_tbl = mt85xx_get_part_info_by_name(part_name);
  return r_part_tbl->u4_info;
}
UINT32 get_part_addr_by_name(const char* part_name)
{
  PART_TBL_ITEM *r_part_tbl;
  if (part_name == NULL)
  {
    printk("get_part_addr_by_name,name is NULL\n");
    return 0;
  }
  r_part_tbl = mt85xx_get_part_info_by_name(part_name);
  return r_part_tbl->u4_offset;
}
UINT32 get_part_size_by_name(const char* part_name)
{
  PART_TBL_ITEM *r_part_tbl;
  if (part_name == NULL)
  {
    printk("get_part_size_by_name,name is NULL\n");
    return 0;
  }
  r_part_tbl = mt85xx_get_part_info_by_name(part_name);
  return r_part_tbl->u4_size;
}
UINT32 get_page_size(void)
{
  #ifdef CONFIG_EMMC_BOOT
  struct emmc *emmc;
  emmc = find_emmc_device(0);
  return emmc->read_bl_len;
  #else
  struct mtd_info *mtd;
  mtd = get_mtd_device(NULL, 0);
  return mtd->writesize;
  #endif
}
UINT32 get_block_size(void)
{
#ifdef CONFIG_EMMC_BOOT
  struct emmc *emmc;
printk("emmc getblock size\n");
  emmc = find_emmc_device(0);
  return emmc->erase_grp_size;
#else
  struct mtd_info *mtd;
  printf("nand getblock size\n");

  mtd = get_mtd_device(NULL, 0);
  return mtd->erasesize;
#endif
printf("block size end\n");

}
static void erase_buffer(void *buffer, size_t size)
{
	const uint8_t kEraseByte = 0xff;
	if (buffer != NULL && size > 0) 
	{
		memset(buffer, kEraseByte, size);
	}
}

#if ADAPT_MTD_LINUX
enum PIT_VER_T _pit_ver=V_ERR;
UINT32 *orig_pit=NULL;
UINT32 *adpt_pit=NULL;
UINT32 _u4_orig_part_nr=0;
UINT32 _u4_same=0;

UINT32 u4GetPartNum(UINT32 pit, UINT32 u4HeadSig, UINT32 u4TailSig)
{
  UINT32 i=0;
  V1_PIT_HEADER_T *v1part=NULL;
  PART_TBL_ITEM *v0Part=NULL;
  INT32 pitver=i4CheckPITVer((UINT32*)pit,u4HeadSig);
  UINT32 len=0;
  if(pitver == V0)
  {
	for(i=0; i<PART_TBL_ITEM_NR_MAX; i++)
	{
	  v0Part = (PART_TBL_ITEM*)(pit+i*sizeof(PART_TBL_ITEM));
	  if((v0Part->u4_id     != u4TailSig) ||
	  	 (v0Part->u4_info   != u4TailSig) ||
	  	 (v0Part->u4_offset != u4TailSig) ||
	  	 (v0Part->u4_size   != u4TailSig) )
	  	{
	  	  continue;
	  	}
	    /*end sig found*/
		len = i;
	}
  }
  else if(pitver == V1)
  {
    v1part = (V1_PIT_HEADER_T*)pit;
	len = v1part->u4_item_num;
  }
  else
  {
    len = 0;
  }
  NAND_LOG(NAND_LOG_DEBUG,"pit part num 0x%x\n", len);
  return len;
}

UINT32 u4GetItemLen(UINT32 pit, UINT32 u4HeadSig, UINT32 u4TailSig)
{
	V1_PIT_HEADER_T *v1part=NULL;
	INT32 pitver=i4CheckPITVer((UINT32*)pit,u4HeadSig);
    UINT32 len=0;
	
    if(pitver == V0)
    {
      len = sizeof(PART_TBL_ITEM);
    }
	else if (pitver == V1)
	{
	  v1part=(V1_PIT_HEADER_T*)pit;
	  len = v1part->u4_item_len;
	}
	else
	{
	  len = 0;
	}
	
	//NAND_LOG(NAND_LOG_DEBUG,"pit item len 0x%x\n", len);
    return len;
}


INT32 u4SetPartHead(UINT32 pit, UINT32 pitver, UINT32 u4HeadSig)
{
  UINT32 *phead = (UINT32*)pit;
  int i=0;
  if (pitver == V0)
  {
    for (i=0; i<8; i++)
    {
      *(phead+i) = u4HeadSig;
    }
  }
  else if (pitver == V1)
  {
    *phead = u4HeadSig;
    *(phead+1) = u4HeadSig;
  }
  else
  {
    NAND_LOG(NAND_LOG_ERROR,"set part head failed!\n");
    return -1;
  }
  NAND_LOG(NAND_LOG_DEBUG,"set part head success!\n");
  return 0;
}
INT32 u4SetPartTail(UINT32 pit, UINT32 pitlen, UINT32 pitver, UINT32 u4TailSig)
{
  UINT32 *ptail = (UINT32*)(pit + sizeof(PART_TBL_ITEM)*(pitlen+1));
  int i=0;
  if (pitver == V0)
  {
    for (i=0; i<8; i++)
    {
      *(ptail+i) = u4TailSig;
    }
  }
  else if (pitver == V1)
  {
    ;//do nothing, v1 version has no tail.
  }
  else
  {
    NAND_LOG(NAND_LOG_ERROR,"set part tail failed!\n");
    return -1;
  }
  
  NAND_LOG(NAND_LOG_DEBUG,"set part tail success!\n");
  return 0;
}

INT32 i4CheckPITVer(UINT32 *phead, UINT32 u4HeadSig)
{
  if ((*(phead) != u4HeadSig) || (*(phead+1) != u4HeadSig))
  {
   // NAND_LOG(NAND_LOG_DEBUG, "no part head sig found -1\n");
  	return -1;
  }
  else
  {
    if (*(phead+2) != u4HeadSig || *(phead+3) != u4HeadSig)
    {
      NAND_LOG(NAND_LOG_DEBUG, "v1 pit found\n");
	  return V1;
    }
	else
	{
	  NAND_LOG(NAND_LOG_DEBUG, "v0 pit found\n");
	  return V0;
	}
  	return 0;
  }
}

PART_TBL_ITEM* pr_get_pit_item_by_idx(UINT32 pit, UINT32 idx, UINT32 pitver)
{
  V1_PIT_HEADER_T* hdr = (V1_PIT_HEADER_T*)pit;
  if (pitver == V0)
  {
  	return (PART_TBL_ITEM*)(pit + sizeof(PART_TBL_ITEM)*(idx+1));
  }
  return (PART_TBL_ITEM*)(pit + hdr->u4_header_len + (idx * hdr->u4_item_len));
}

PART_TBL_ITEM* mt85xx_get_part_item_by_name(UINT32 pit, const char* part_name, UINT32 pitver)
{
  UINT32 i=0;
  PART_TBL_ITEM *item=NULL;
  for(i=0 ; i<u4_part_nr; i++)
  {
    item = pr_get_pit_item_by_idx(pit, i, pitver);
	if(strcmp(item->pc_name, part_name))
    {
            continue;
    }
	
	NAND_LOG(NAND_LOG_DEBUG, "get part item by name success\n");
	return item;
  }
  
  NAND_LOG(NAND_LOG_ERROR, "get part item by name error\n");
  return NULL;
}

UINT32 get_part_id_by_name_new(UINT32 pit, const char* part_name, UINT32 pitver)
{
  PART_TBL_ITEM *temp=NULL;
  temp = mt85xx_get_part_item_by_name(pit, part_name, pitver);
  if (temp)
    return temp->u4_id;
  else
  	return 0;
}
INT32 i4CopyPartItem(UINT32 pdest, UINT32 psource, UINT32 pitver)
{
  UINT32 u4ItemLen=0,u4CheckLen=0;
  
  if (pitver == V0)
  {
	u4CheckLen = 32;
  }
  else if (pitver == V1)
  {
	u4CheckLen = 40;
  }
  u4ItemLen = u4GetItemLen(psource, HEAD_SIG, TAIL_SIG);
  memcpy((u_char*)pdest, (u_char*)psource, u4CheckLen);
  return 0;
}
INT32 i4SetEnv(UINT32 pit, UINT32 u4PartCnt) 
{
    INT32 i = 0;	
    INT32 i4Ret = 0;
    char _bootargs_buf[1024] = "";
    char _tmp_buf[1024] = "";
	PART_TBL_ITEM *item=NULL;
    /* mtdparts */
    strcat(_bootargs_buf, "mtdparts=mt85xx_nand:");
    for (i = 0; (i < u4PartCnt); i++) 
    {
      item = pr_get_pit_item_by_idx(pit, i, _pit_ver);
      if( ISUBIVOLUME(item->u4_info) != 4 )
      {
        i4Ret = sprintf(_tmp_buf, "%dK(%s),", item->u4_size >> 10,item->pc_name);	
        if (-1 == i4Ret) 
		{
          NAND_LOG(NAND_LOG_ERROR, "[mt85xx_part_oper]creat szMtdPartsCmd failed\n");	
          return i4Ret;
        }
        strcat(_bootargs_buf, _tmp_buf);	
      }
    }
	NAND_LOG(NAND_LOG_INFO, "mt85xx_part_oper.c vSetEnv set mtdparts = %s \n", _bootargs_buf);	 
    i4Ret = setenv("mtdparts", _bootargs_buf);
    if (-1 == i4Ret) {
       NAND_LOG(NAND_LOG_ERROR, "mt85xx_part_oper set mtdparts failed, mtdparts = %s \n", _bootargs_buf);	
       return i4Ret;
    }
    i4Ret =  setenv("mtdids", "nand0=mt85xx_nand");
    if (-1 == i4Ret) {
       NAND_LOG(NAND_LOG_ERROR, "mt85xx_part_oper set mtdids failed\n");	
       return i4Ret;
    }
	NAND_LOG(NAND_LOG_INFO, "mt85xx_part_oper set env OK\n");

    return i4Ret;  
}
#if 0
INT32 i4SetMtdTbl(UINT32 pit, UINT32 u4PartCnt)
{
  UINT32 i=0;
  PART_TBL_ITEM *item=NULL;
  UINT32 pitver= i4CheckPITVer((UINT32*)pit, ADPT_HEAD_SIG);
  
  i4SetEnv(pit, u4PartCnt);
  for (i = 0; i < u4PartCnt; i++) 
  {
    item = pr_get_pit_item_by_idx(pit, i, pitver);
   // if( ISUBIVOLUME(item->u4_info) != 4 )
    {
	   if(mt85xx_init_mtd_table(item->pc_name) != 0)	
      {
        NAND_LOG(NAND_LOG_ERROR, "[Set Mtd Table]%s not be added!\n", item->pc_name);	
        return -1;
      }
    }
  }
	//set mtd success
	return 0;
}
#endif
INT32 i4FindPartTbl(UINT32 *pit, UINT32 *pitnr, UINT32 headsig, UINT32 tailsig)
{
    int r=0;
    UINT32 i=0;
	UINT32 internal=get_block_size();
	EMMC_LOG(EMMC_DBG, "[i4FindPartTbl] block_size=0x%x\n",internal);
	if(!pit)
	{
  	  EMMC_LOG(EMMC_ERR, "[i4FindPartTbl] pit NULL.\n");
  	  r = -1;
  	  goto err;
	}
    for(i=SEARCH_PART_TBL_START; i<SEARCH_PART_TBL_END; i+=internal)
    {
        /* read nand */
		r = i4NFBPartitionRead(0,(UINT32)i,(UINT32)pit, PART_TBL_ITEM_NR_MAX*sizeof(NEW_PART_TBL_ITEM));
        if(r)
        {
            EMMC_LOG(EMMC_ERR, "[i4FindPartTbl] NFBPartitionRead error!!\n");
            continue;
        }

        /* check head, the head sig is stored in the begining 8 byte of pc_name */
	    if ((i4CheckPITVer(pit, headsig)) < 0)
	    {
		  continue;
	    }
        r = 0;
		*pitnr = u4GetPartNum((UINT32*)pit, headsig, tailsig);
		EMMC_LOG(EMMC_DBG, "[i4FindPartTbl] pit found!!\n");
		goto close;
		
    }
    /* not found */
err:
    EMMC_LOG(EMMC_ERR, "[i4FindPartTbl] can't find pit!!\n");
	r = -1;
close:
	return r;

}
INT32 i4SearchAdptPIT(void)
{
  int r=0;
  UINT32 pitlen=0;
  /*find partition table in nand*/
  if(_u4_same)
  {
    if(!adpt_pit)
    {
      adpt_pit = kzalloc(sizeof(NEW_PART_TBL_ITEM)*PART_TBL_ITEM_NR_MAX, GFP_KERNEL);
      if(!adpt_pit)
      {
         NAND_LOG(NAND_LOG_ERROR, "[i4SearchAdptPIT] allocate adpt_pit memo failed.\n");
  	   return -1;
      }
    }
    r = i4FindPartTbl(adpt_pit, &pitlen, ADPT_HEAD_SIG, ADPT_TAIL_SIG);
    if(r)
    {
      NAND_LOG(NAND_LOG_INFO, "[i4SearchAdptPIT] error.\n");
  	  return -1;
    }
	#if 0
    if(i4SetMtdTbl((UINT32)adpt_pit, pitlen))
    {
      r = -1;
      NAND_LOG(NAND_LOG_ERROR, "mount adaptive mtd failed\n");
  	  return -1;
    }
	#endif
  }
  
  NAND_LOG(NAND_LOG_INFO, "[i4SearchAdptPIT] success.\n");
  return (INT32)adpt_pit;
}
INT32 i4GetAdptPit(void)
{
  return i4SearchAdptPIT();
}

//INT32 i4StoreAdptPit(void)
INT32 i4StoreAdptPit(UINT32 bin_pit)
{
  UINT32 u4Id=0;
  UINT32 offset=0;
  UINT32 itemnr=0;

  if (_pit_ver == V0)
  {
    itemnr = u4_part_nr + 2;
  }
  else if (_pit_ver == V1)
  {
    itemnr = u4_part_nr + 1;
  }
  else
  {
    NAND_LOG(NAND_LOG_DEBUG, "store adpt pit failed, due to part version error\n");
    return -1;
  }
  	  
  if (!_u4_same)
  {
    NAND_LOG(NAND_LOG_DEBUG, "store adpt pit in part_info_1 partition\n");
    u4Id = get_part_id_by_name_new((UINT32)adpt_pit, "part_info_1", _pit_ver);
	if (u4Id != 0)
	{
      /*store adaptive PIT part_info_1 parition.*/
	  if (get_block_size() > ADPT_PIT_OFFSET)
	    offset = get_block_size();
  	  offset += pr_get_pit_item_by_idx((UINT32)bin_pit, u4Id-1, _pit_ver)->u4_offset;
	  if (i4NFBPartitionWrite(0, offset, (UINT32)adpt_pit, 
      	                 (UINT32)(itemnr * sizeof(NEW_PART_TBL_ITEM))))
      {
        NAND_LOG(NAND_LOG_ERROR, "store part_info_1 adpt pit failed\n");
    	  return -1;
      }
	}
	else
	{
	  NAND_LOG(NAND_LOG_INFO, "no partition part_info_1 \n");
	}
    NAND_LOG(NAND_LOG_DEBUG, "store adpt pit in part_info_2 partition\n");
    u4Id = get_part_id_by_name_new((UINT32)adpt_pit, "part_info_2", _pit_ver);
	if (u4Id != 0)
	{
	  if (get_block_size() > ADPT_PIT_OFFSET)
	    offset = get_block_size();
  	  offset += pr_get_pit_item_by_idx((UINT32)bin_pit, u4Id-1, _pit_ver)->u4_offset;
      /*store adaptive PIT to part_info_2 parition.*/
      if (i4NFBPartitionWrite(0, offset, (UINT32)adpt_pit,
      	                 (UINT32)(itemnr * sizeof(NEW_PART_TBL_ITEM))))
      {
        NAND_LOG(NAND_LOG_ERROR, "store part_info_2 adpt pit failed\n");
    	  return -1;
      }
	}
    else
    {
      NAND_LOG(NAND_LOG_INFO, "no partition part_info_2 \n");
    }
    NAND_LOG(NAND_LOG_INFO, "store adpt pit success\n");
  }
  return 0;
}

INT32 i4SearchOrigPIT(void)
{
  int r=0;
  /*find partition table in nand*/
  if(!orig_pit)
  {
    orig_pit = kzalloc(sizeof(NEW_PART_TBL_ITEM)*PART_TBL_ITEM_NR_MAX, GFP_KERNEL);
	if(!orig_pit)
    {
       NAND_LOG(NAND_LOG_ERROR, "[i4SearchOrigPIT] allocate orig_pit memo failed.\n");
	   return -1;
    }
  }
  r = i4FindPartTbl(orig_pit, &_u4_orig_part_nr, HEAD_SIG, TAIL_SIG);
  if(r)
  {
    NAND_LOG(NAND_LOG_ERROR, "[i4SearchOrigPIT] ERROR.\n");
	return -1;
  }
  _pit_ver = i4CheckPITVer(orig_pit, HEAD_SIG);
  _u4_orig_part_nr = u4GetPartNum((UINT32)orig_pit, HEAD_SIG, TAIL_SIG);
  NAND_LOG(NAND_LOG_INFO, "[i4SearchOrigPIT] SUCCESS.\n");
  return (INT32)orig_pit;

}
INT32 i4CompareFlashPIT(UINT32 bin_pit, UINT32 len)
{
  INT32 i4Ret=0;
  UINT32 u4ItemLen=0, i=0;
  UINT32 u4CheckLen=0;

  orig_pit = kzalloc(sizeof(NEW_PART_TBL_ITEM)*PART_TBL_ITEM_NR_MAX, GFP_KERNEL);
  adpt_pit = kzalloc(sizeof(NEW_PART_TBL_ITEM)*PART_TBL_ITEM_NR_MAX, GFP_KERNEL);
  if(!orig_pit)
  {
     NAND_LOG(NAND_LOG_ERROR, "[i4CompareFlashPIT] allocate orig_pit memo failed.\n");
	 i4Ret = -1;
     goto err;
  }
  if(!adpt_pit)
  {
     NAND_LOG(NAND_LOG_ERROR, "[i4CompareFlashPIT] allocate adpt_pit memo failed.\n");
	 i4Ret = -1;
     goto err;
  }
  if((i4SearchOrigPIT())<0)
  {
    goto err;
  }
  
  if(i4CheckPITVer((UINT32*)bin_pit, HEAD_SIG) != _pit_ver)
  {
    /*the part version is differnt*/
    NAND_LOG(NAND_LOG_INFO, "[i4CompareFlashPIT] version different.\n");
    return 0;
  }
  
  if (_pit_ver == V0)
  {
	u4CheckLen = 32;
  }
  else if (_pit_ver == V1)
  {
	u4CheckLen = 40;
  }
  
  u4ItemLen = u4GetItemLen((UINT32)orig_pit, HEAD_SIG, TAIL_SIG);

  if (i4Ret == 0)
  {
    /*check whether the partition number are same or not . 
       if not , the compared result can return 'different' directly and doesn't need to compare item by item*/
    if (u4GetPartNum(bin_pit, HEAD_SIG, TAIL_SIG) != _u4_orig_part_nr)
    {
      NAND_LOG(NAND_LOG_INFO, "[i4CompareFlashPIT] partition number is different.\n");
      return 0;
    }
	/*compare item by item, from the 1st item to start
	note: the revered filed is un-necessary to compare, because the content in reserved filed is meaningless.*/
    for (i = 1 ; i < _u4_orig_part_nr ; i++)
    {
      i4Ret = memcmp(((UCHAR *)bin_pit + (u4ItemLen*i)) , ((UCHAR *)orig_pit + (u4ItemLen*i)), u4CheckLen);
      if (i4Ret != 0)
      {
        //compare failed.
        break;
      }
    }
	if (i4Ret == 0)
	{
	  if (u4_adpt == 1)
	  {
	    NAND_LOG(NAND_LOG_INFO, "[i4CompareFlashPIT] different, need to creat adpt pit!\n");
		return 0;
	  }
	  else
	  {
	    _u4_same = 1;
	    NAND_LOG(NAND_LOG_INFO, "[i4CompareFlashPIT] same.\n");
        return 1;
	  }
	}
	else
	{
	  NAND_LOG(NAND_LOG_INFO, "[i4CompareFlashPIT] different.\n");
      return 0;
	}
  }
err:
  NAND_LOG(NAND_LOG_ERROR, "[i4CompareFlashPIT] search original pit failed.\n");
  return -1;
}
INT32 i4SetNewFlashPIT(UINT32 bin_pit)
{
  UINT32 u4Id = 0;
  UINT32 u4Offset=0, u4Size=0, u4GoodCnt=0, u4BadCnt=0, u4Blockstart=0;  
  INT32 i4Ret = -1;
#ifdef CONFIG_EMMC_BOOT
	struct emmc *emmc;
#else
  struct mtd_info *mtd=NULL;
#endif
  PART_TBL_ITEM *prdest=NULL, *prdestup=NULL, *prsource=NULL;
  UINT32 u4DestOft=0;

  NAND_LOG(NAND_LOG_DEBUG, "[i4SetNewFlashPIT] set new flash pit\n");
  if(!adpt_pit)
  {
    adpt_pit = kzalloc(sizeof(NEW_PART_TBL_ITEM)*PART_TBL_ITEM_NR_MAX, GFP_KERNEL);
    if(!adpt_pit)
    {
      NAND_LOG(NAND_LOG_ERROR, "[i4SearchAdptPIT] allocate adpt_pit memo failed.\n");
  	  return -1;
    }
  }
#ifdef CONFIG_EMMC_BOOT
  emmc = find_emmc_device(0);
#else
  mtd = get_mtd_device(NULL, 0);
#endif
  _pit_ver = i4CheckPITVer((UINT32*) bin_pit, HEAD_SIG);
  if (_pit_ver == V0)
  {
	prdest = kzalloc(sizeof(PART_TBL_ITEM),GFP_KERNEL);
	prdestup = kzalloc(sizeof(PART_TBL_ITEM),GFP_KERNEL);
	prsource = kzalloc(sizeof(PART_TBL_ITEM),GFP_KERNEL);
  }
  else if (_pit_ver == V1)
  {
	prdest = kzalloc(sizeof(NEW_PART_TBL_ITEM),GFP_KERNEL);
	prdestup = kzalloc(sizeof(PART_TBL_ITEM),GFP_KERNEL);
    prsource = kzalloc(sizeof(PART_TBL_ITEM),GFP_KERNEL);
  }

  if (!prdest || !prdestup || !prsource)
  {
    NAND_LOG(NAND_LOG_ERROR, "[i4SetNewFlashPIT]allocate prdest memo failed\n");
    i4Ret = -1;
	goto closeall;
  }		
  u4_part_nr = u4GetPartNum(bin_pit, HEAD_SIG, TAIL_SIG);
  if (u4_part_nr == 0)
  {
    NAND_LOG(NAND_LOG_ERROR, "the size of part tbl you past is zero\n");
    i4Ret = -1;
    goto closeall;
  }

  /*scan each partition to creat a new adaptive table*/
  /*step 0: fill in the header of adapt pit, according to bin_pit*/
  i4CopyPartItem((UINT32)adpt_pit,(UINT32)bin_pit, _pit_ver);
  
  /*step 1: check 1st partition alone*/
  NAND_LOG(NAND_LOG_DEBUG, "step 1: check 1st partition\n");
  prdest = pr_get_pit_item_by_idx((UINT32)adpt_pit, 0, _pit_ver);
  prsource = pr_get_pit_item_by_idx((UINT32)bin_pit, 0, _pit_ver);
  u4Offset = prsource->u4_offset;
  u4Size = prsource->u4_size;
  u4BadCnt = 0;
  #if 0
  for (u4GoodCnt = 0; (u4GoodCnt * mtd->erasesize) < u4Size;)
  {
    u4Blockstart = u4Offset & (~mtd->erasesize + 1);
    u4Offset += mtd->erasesize;
	if(mtd->block_isbad(mtd, u4Blockstart))
	{
	  u4BadCnt ++;
      continue;
	}
	u4GoodCnt++;
  }
  #endif
  i4CopyPartItem((UINT32)prdest,(UINT32)prsource, _pit_ver);
  if( ISUBIVOLUME(prdest->u4_info) != 4 )
  {
#ifdef CONFIG_EMMC_BOOT
	prdest->u4_size = prsource->u4_size + u4BadCnt*emmc->erase_grp_size; 	
#else
    prdest->u4_size = prsource->u4_size + u4BadCnt*mtd->erasesize; 
#endif
  }
  /*step 2: check the 2nd~last partition*/
  NAND_LOG(NAND_LOG_DEBUG, "step 2: check 2nd~last partition\n");
  for (u4Id = 1; u4Id < u4_part_nr; u4Id ++) //scan flash to creat adaptive 
  {
    prdest = pr_get_pit_item_by_idx((UINT32)adpt_pit, u4Id, _pit_ver);
	prsource = pr_get_pit_item_by_idx((UINT32)bin_pit, u4Id, _pit_ver);
	prdestup = pr_get_pit_item_by_idx((UINT32)adpt_pit, u4Id-1, _pit_ver);
	if( ISUBIVOLUME(prsource->u4_info) != 4 ) // it is main partition
    {
      if (ISUBIVOLUME(prdestup->u4_info) != 4)//main partition
      {
	    prdest->u4_offset = prdestup->u4_offset + prdestup->u4_size;
      }
	  else //vol partition
	  {
	    prdestup = pr_get_pit_item_by_idx((UINT32)adpt_pit, u4_get_parent_part_id(prdestup)-1, _pit_ver);
	    prdest->u4_offset = prdestup->u4_offset + prdestup->u4_size;
	  }
    }
	else //it is vol partition
	{
	  if (ISUBIVOLUME(prdestup->u4_info) != 4)//main partition
	  {
	    prdest->u4_offset = prdestup->u4_offset;
	  }
	  else // vol partition
	  {
	    prdest->u4_offset = prdestup->u4_offset + prdestup->u4_size;
	  }
	}
    u4DestOft=u4Offset = prdest->u4_offset;
	u4Size = prsource->u4_size;
	u4BadCnt = 0;
	#if 0
	for (u4GoodCnt = 0; ((u4GoodCnt * mtd->erasesize) < u4Size);)
	{
	  u4Blockstart = u4Offset & (~mtd->erasesize + 1);
	  if (u4Offset >= (mtd->size - 12*mtd->erasesize)) //reserve at least 12 block to store bbt
	  {
	    NAND_LOG(NAND_LOG_ERROR, "[i4SetNewFlashPIT]Sorry, no enough block to use for \"%s\", please check PIT\n", prsource->pc_name);
        while(1);
	  }
	  u4Offset += mtd->erasesize;
	  if(mtd->block_isbad(mtd, u4Blockstart))
	  {
	    u4BadCnt ++;
		NAND_LOG(NAND_LOG_INFO, "[i4SetNewFlashPIT]bad block found at 0x%x\n", u4Blockstart);
		continue;
	  }
	  u4GoodCnt++;
	}
	#endif
	i4CopyPartItem((UINT32)prdest,(UINT32)prsource, _pit_ver);
	prdest->u4_offset = u4DestOft;
 #ifdef CONFIG_EMMC_BOOT
    prdest->u4_size = prsource->u4_size + u4BadCnt*emmc->erase_grp_size; 
 #else
	prdest->u4_size = prsource->u4_size + u4BadCnt*mtd->erasesize; 
 #endif
  }
  /*step 3: add the head&tail as adaptive pit, the head&tail style is same the binary pit, because it will be searched by other modules*/
  NAND_LOG(NAND_LOG_DEBUG, "step 3: set adpt pit head&tail\n");
  u4SetPartHead((UINT32)adpt_pit, _pit_ver, ADPT_HEAD_SIG);
  u4SetPartTail((UINT32)adpt_pit, u4_part_nr, _pit_ver, ADPT_TAIL_SIG);
  #if 0
  /*re-install mtd with new adaptive PIT */
  NAND_LOG(NAND_LOG_DEBUG, "step 5: re-init adpt mtd\n");
  if (i4SetMtdTbl((UINT32)adpt_pit, u4_part_nr))
  {
    i4Ret = -1;
    NAND_LOG(NAND_LOG_ERROR, "mount adaptive mtd failed\n");
	goto closeall;
  }
   
  NAND_LOG(NAND_LOG_DEBUG, "step 6: store adpt pit in part_info_1 partition\n");
  u4Id = get_part_id_by_name_new((UINT32)adpt_pit, "part_info_1", _pit_ver);
  /*store adaptive PIT part_info_1 parition.*/
   if (i4NFBPartitionWrite(u4Id, ADPT_PIT_OFFSET,(UINT32)adpt_pit,
  	                 (UINT32)((u4_part_nr+1) * sizeof(NEW_PART_TBL_ITEM))))
  {
    i4Ret = -1;
    NAND_LOG(NAND_LOG_ERROR, "store part_info_1 adpt pit failed\n");
	goto closeall;
  }
  NAND_LOG(NAND_LOG_DEBUG, "step 7: store adpt pit in part_info_2 partition\n");
  u4Id = get_part_id_by_name_new((UINT32)adpt_pit, "part_info_2", _pit_ver);
  /*store adaptive PIT to part_info_2 parition.*/
  if (i4NFBPartitionWrite(u4Id,ADPT_PIT_OFFSET,(UINT32)adpt_pit,
  	                 (UINT32)((u4_part_nr+1) * sizeof(NEW_PART_TBL_ITEM))))
  {
    i4Ret = -1;
    NAND_LOG(NAND_LOG_ERROR, "store part_info_2 adpt pit failed\n");
	goto closeall;
  }
  #else
  //if(i4StoreAdptPit())
  if(i4StoreAdptPit(bin_pit))
  {
	i4Ret = -1;
	goto closeall;
  }
  #endif
  i4Ret = 0;
  
closeall:
  if (i4Ret<0)
  {
    NAND_LOG(NAND_LOG_ERROR, "set new flash pit failed\n");
  }
  else
  {
    NAND_LOG(NAND_LOG_DEBUG, "set new flash pit success\n");
  }
  return i4Ret;
}
#endif

INT32 i4NFBPartitionRead(UINT32 u4DevId, UINT32 u8Offset, UINT32 u4MemPtr, UINT32 u4MemLen)
{
	struct emmc *emmc;
	PART_TBL_ITEM *pr_dst_part;
	UINT32 startaddr = 0;
	emmc = find_emmc_device(0);
	
	EMMC_LOG(EMMC_DBG, "[EMMC][%s]u4DevId=%d, start=0x%x, len=0x%x, buf=0x%x\n", __func__,
				u4DevId, u8Offset, u4MemLen, u4MemPtr);
	
	ASSERT(emmc != NULL);

	if (u4DevId > 0)
	{
		pr_dst_part = mt85xx_get_part_info_by_id(u4DevId);
		if (pr_dst_part == NULL)
		{
			EMMC_LOG(EMMC_ERR, "[EMMC][%s] Partition table have not initialized\n", __func__);
			return -1;
		}
		
		if((u8Offset+u4MemLen) > pr_dst_part->u4_size)
		{
			EMMC_LOG(EMMC_ERR, "[EMMC][%s] Parameter out of capacity\n", __func__);
			return -1;
		}
		startaddr = (pr_dst_part->u4_offset +  u8Offset);
	}
	else
	{
	
		if((u8Offset+u4MemLen) > emmc->capacity_byte)
		{
			EMMC_LOG(EMMC_ERR, "[EMMC][%s] Parameter out of capacity\n", __func__);
			return -1;
		}
		startaddr = u8Offset;
	}
	if(0 != emmc_read(emmc, startaddr , u4MemLen, (UINT8*)u4MemPtr))
	{
		EMMC_LOG(EMMC_ERR, "[EMMC][%s] Read fail! \n", __func__);
		return -1;
	}
	return 0;
}

INT32 i4NFBPartitionWrite(UINT32 u4DevId, UINT32 u8Offset , UINT32 u4MemPtr, UINT32 u4MemLen)
{
	struct emmc *emmc;
	PART_TBL_ITEM *pr_dst_part;
	UINT32 startaddr = 0;

	EMMC_LOG(EMMC_DBG, "[EMMC][%s]u4DevId=%d, start=0x%x, len=0x%x, buf=0x%x\n", __func__,
				u4DevId, u8Offset, u4MemLen, u4MemPtr);
	
	emmc = find_emmc_device(0);
	ASSERT(emmc != NULL);
	
	if (u4DevId > 0)
	{
		pr_dst_part = mt85xx_get_part_info_by_id(u4DevId);
		if (pr_dst_part == NULL)
		{
			EMMC_LOG(EMMC_ERR, "[EMMC][%s] Partition table have not initialized\n", __func__);
			return -1;
		}
		
		if((u8Offset+u4MemLen) > pr_dst_part->u4_size)
		{
			EMMC_LOG(EMMC_ERR, "[EMMC][%s] Parameter out of capacity\n", __func__);
			return -1;
		}
		startaddr = (pr_dst_part->u4_offset +  u8Offset);
	}
	else
	{
	
		if((u8Offset+u4MemLen) > emmc->capacity_byte)
		{
			EMMC_LOG(EMMC_ERR, "[EMMC][%s] Parameter out of capacity\n", __func__);
			return -1;
		}
		startaddr = u8Offset;
	}


	if(0 != emmc_write(emmc, startaddr, u4MemLen, (UINT8*)u4MemPtr))
	{
		EMMC_LOG(EMMC_ERR, "[EMMC][%s] Write fail! \n", __func__);
		return -1;
	}	
	return 0;
}
EXPORT_SYMBOL(get_part_id_by_name);
EXPORT_SYMBOL(get_part_info_by_name);
EXPORT_SYMBOL(get_part_addr_by_name);
EXPORT_SYMBOL(get_part_size_by_name);
EXPORT_SYMBOL(get_page_size);
EXPORT_SYMBOL(get_block_size);
EXPORT_SYMBOL(i4NFBPartitionRead);
EXPORT_SYMBOL(i4NFBPartitionWrite);

#endif

