/*
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 *  Copyright(C) 2006 NXP BV, All rights reserved.
 *  by Jean-Paul Saman
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <common.h>
#include <asm/io.h>
#include <asm/arch/mt85xx.h>
#include <asm/arch/sys_proto.h>
#include <asm/arch/sys_info.h>
#include <asm/mach-types.h>
#include <asm/arch/x_typedef.h>
#include <asm/arch/x_ckgen.h>
#include <mt85xx_gpio.h>
#include "gpio_config.h"
#include <chip_ver.h>
#include <upg_config.h>

#ifndef __UBOOT_GPIO_PINMUX__
#define __UBOOT_GPIO_PINMUX__
#endif
#include <asm/arch/mt85xx_gpio_pinmux.h>
#include <asm/arch/mt85xx_pinmux_table.h>

//add for pwm_sel whoes max function is 24.
#define MAX_FUNCTION_NUM 24

//-----------------------------------------------------------------------------
// definitions
//-----------------------------------------------------------------------------
#define TOTAL_GPIO_IDX          5
#define GPIO_INDEX_MASK         ((1 << 5) - 1)

//pdwnc
#define PDWNC_PAD_PINMUX1   0x0F4
#define PDWNC_PAD_PINMUX2   0x0F8
#define PDWNC_PAD_PINMUX3   0x0FC

#define PDWNC_PAD_PINMUX4   0x100
#define PDWNC_PAD_PINMUX5   0x10c
#define PDWNC_EGPIO_SEL     0x32c

#define PDWNC_GPIOEN        0x330
#define PDWNC_GPIOIN        0x334
#define PDWNC_GPIOOUT       0x338

#define PDWNC_GPIOEN1       0x33c
#define PDWNC_GPIOIN1       0x340
#define PDWNC_GPIOOUT1      0x344

#define GPIO_EN_WRITE(idx,val)          CKGEN_WRITE32((REG_RW_GPIO_EN_OFFSET+(4*(idx))), (val))
#define GPIO_OUT_WRITE(idx,val)         CKGEN_WRITE32((REG_RW_GPIO_OUT_OFFSET+(4*(idx))), (val))
#define GPIO_IN_WRITE(idx,val)          CKGEN_WRITE32((REG_RW_GPIO_IN_OFFSET+(4*(idx))), (val))
#define GPIO_EN_REG(idx)                CKGEN_REG32(REG_RW_GPIO_EN_OFFSET+(4*(idx)))
#define GPIO_OUT_REG(idx)               CKGEN_REG32(REG_RW_GPIO_OUT_OFFSET+(4*(idx)))
#define GPIO_IN_REG(idx)                CKGEN_REG32(REG_RW_GPIO_IN_OFFSET+(4*(idx)))

#define PINMUX_WRITE(idx,val)           CKGEN_WRITE32((REG_RW_PINMUX_OFFSET+(4*(idx))), (val))
#define PINMUX_REG(idx)                 CKGEN_REG32(REG_RW_PINMUX_OFFSET+(4*(idx)))

#define GPIO_PDWNC_READ(i4Addr)         (*((volatile UINT32*)(PDWNC_BASE + (i4Addr)))) 
#define GPIO_PDWNC_WRITE(i4Addr, u4Val) ((*((volatile UINT32*)(PDWNC_BASE + (i4Addr)))) = (u4Val))


#define PAD_CFG_WRITE(idx, value)   	CKGEN_WRITE32((REG_RW_PINMUX_OFFSET + ((idx)*4)), (value))    
#define PAD_CFG_REG(idx)				CKGEN_REG32((REG_RW_PINMUX_OFFSET + ((idx)*4)))
#define PAD_PWMCTL_WRITE(idx, value)   	CKGEN_WRITE32((REG_RW_PAD_PWMCTL_0 + ((idx)*4)), (value))
#define PAD_PWMCTL_REG(idx)             CKGEN_REG32(REG_RW_PAD_PWMCTL_0 + (4 * (idx)))

#if SUPPORT_JIG
extern BOOL fgGetJigMode0(void); //JIG_MODE0
extern BOOL fgGetJigMode1(void);  //JIG_MODE1
#endif


static inline unsigned gpio_get_addridx(UINT32 gpio_pinmux)
{
    return (gpio_pinmux / 32);
}

static inline unsigned gpio_get_addroffset(UINT32 gpio_pinmux)
{
    return (gpio_pinmux % 32);
}

/*****************************************
 * Routine: BSP_PinSet
 *****************************************/
INT32 BSP_PinSet(INT32 i4FuncSel, INT32 i4Func)
{
	UINT32 u4Mask, u4Offset, u4Idx;

	if(!(i4FuncSel& PDWNC_PINMUX_FLAG))
	{
		if ((i4Func < 0) ||
			(i4Func > MAX_FUNCTION_NUM) ||
			(i4Func > _au1PinmuxFunctionMasks[i4FuncSel]) ||
			(i4FuncSel < 0) ||
			(i4FuncSel >= MAX_PINMUX_SEL) ||
			(_au1PinmuxFunctionMasks[i4FuncSel] == 0))
		{
			return -1;
		}
	}
	
	if((i4FuncSel >= AUD_AMUTE2_SEL) && (i4FuncSel <= wpi2c_sel))
	
	{
		u4Offset = gpio_get_addroffset(i4FuncSel);
		u4Idx = gpio_get_addridx(i4FuncSel);
		u4Mask = ~((UINT32)_au1PinmuxFunctionMasks[i4FuncSel] << u4Offset);
	
		PAD_CFG_WRITE(u4Idx, ((PAD_CFG_REG(u4Idx) & u4Mask)|((UINT32)i4Func << u4Offset)));
	}
	
	else if ((i4FuncSel > wpi2c_sel) && (i4FuncSel <= MAX_PINMUX_SEL))
	
	{
	
		u4Offset = gpio_get_addroffset(i4FuncSel - 512);
		u4Idx = gpio_get_addridx(i4FuncSel - 512);
		u4Mask = ~((UINT32)_au1PinmuxFunctionMasks[i4FuncSel] << u4Offset);
		PAD_PWMCTL_WRITE(u4Idx, (((PAD_PWMCTL_REG(u4Idx) & u4Mask)|((UINT32)i4Func << u4Offset))));
	
	}
	else if(i4FuncSel & PDWNC_PINMUX_FLAG)/* pdwnc pinmux*/
	{	
		unsigned value = 0;
		unsigned reg = 0xF4+(i4FuncSel & PD_PM_OFFSET_MASK) >> PD_PM_OFFSET;
		unsigned startBit = (i4FuncSel & PD_PM_START_BIT_MASK) >> PD_PM_START_BIT;
		unsigned mask = (i4FuncSel & PD_PM_MASK_MASK) >> PD_PM_MASK;
		value = GPIO_PDWNC_READ(reg);
		GPIO_PDWNC_WRITE(reg, value & (~(mask << startBit)) | (i4Func << startBit));
	
	}
	return 0;
}

/*****************************************
 * Routine: BSP_PinGet
 *****************************************/
INT32 BSP_PinGet(INT32 i4FuncSel)
{	
	UINT32 u4Mask, u4Offset, u4Idx;

	if ((i4FuncSel < 0) ||
		(i4FuncSel >= MAX_PINMUX_SEL))
	{
		return -1;
	}
	if((i4FuncSel > wpi2c_sel) && (i4FuncSel <= MAX_PINMUX_SEL))
	{
		u4Offset = gpio_get_addroffset((i4FuncSel - 512));
		u4Idx = gpio_get_addridx((i4FuncSel - 512));
		u4Mask = _au1PinmuxFunctionMasks[i4FuncSel];

		return (((PAD_PWMCTL_REG(u4Idx))  >> (u4Offset)) & (u4Mask));
	}
	else if((i4FuncSel >= AUD_AMUTE2_SEL) && (i4FuncSel <= wpi2c_sel))
	{
		u4Offset = gpio_get_addroffset(i4FuncSel);
		u4Idx = gpio_get_addridx(i4FuncSel);
		u4Mask = _au1PinmuxFunctionMasks[i4FuncSel];

		return ((PAD_CFG_REG(u4Idx) >> (u4Offset)) & (u4Mask));
	}
	
}

/*****************************************
 * Routine: GPIO_InOut_Sel
 *****************************************/
void GPIO_InOut_Sel(INT32 i4GpioNum, INT32 i4Output)
{		
	UINT32 u4Val, u4Idx, u4Offset;

	/*set ckgen gpio input/output mode*/
	if(i4GpioNum < 0)
	{
		printf("GPIO number(%d) is invalid!\n",i4GpioNum);
		return;
	}
    else if(((UINT32)i4GpioNum >= PIN_GPIO0) && ((UINT32)i4GpioNum <= PIN_sys_usb20_gpio_dm_oe_2p))
	{
	  u4Offset = gpio_get_addroffset((UINT32)i4GpioNum);
	  u4Idx = gpio_get_addridx((UINT32)i4GpioNum);

      u4Val = GPIO_EN_REG(u4Idx);
      u4Val = (i4Output == OUTPUT) ? (u4Val | (1U << u4Offset)) : (u4Val & ~(1U << u4Offset));
      GPIO_EN_WRITE(u4Idx, u4Val);
	}

	else if((i4GpioNum & EGPIO_FLAG) && (i4GpioNum > PIN_GPIO0))
	{
		u4Offset = gpio_get_addroffset((UINT32)(i4GpioNum - PIN_E_START));
		u4Idx = gpio_get_addridx((UINT32)(i4GpioNum - PIN_E_START));
		
		u4Val = GPIO_PDWNC_READ(PDWNC_GPIOEN + (12*(u4Idx)));	  
		u4Val = (i4Output) ? (u4Val | (1U << u4Offset)) : (u4Val & ~(1U << u4Offset));
		GPIO_PDWNC_WRITE(PDWNC_GPIOEN + (12*(u4Idx)), u4Val);
	}
}  

/*****************************************
 * Routine: GPIO_Output
 *****************************************/
void GPIO_Output(INT32 i4GpioNum, INT32 i4High)
{
	UINT32 u4Val, u4Idx, u4Offset;
	if(i4GpioNum < 0)
	{
		printf("GPIO number(%d) is invalid!\n",i4GpioNum);
		return;
	}
    else if(((UINT32)i4GpioNum >= PIN_GPIO0) &&((UINT32)i4GpioNum <= PIN_sys_usb20_gpio_dm_oe_2p))
	{
	  u4Offset = gpio_get_addroffset((UINT32)i4GpioNum);
      u4Idx = gpio_get_addridx((UINT32)i4GpioNum);
      u4Val = GPIO_OUT_REG(u4Idx);
      u4Val = (i4High == 1) ? (u4Val | (1U << u4Offset)) : (u4Val & ~(1U << u4Offset));
      GPIO_OUT_WRITE(u4Idx, u4Val);
	}

	else if((i4GpioNum & EGPIO_FLAG) && (i4GpioNum > PIN_GPIO0))
	{
		u4Offset = gpio_get_addroffset((UINT32)(i4GpioNum - PIN_E_START));
		u4Idx = gpio_get_addridx((UINT32)(i4GpioNum - PIN_E_START));
		
	  	u4Val = GPIO_PDWNC_READ(PDWNC_GPIOOUT + (12*(u4Idx)));	  
	  	u4Val = (i4High) ? (u4Val | (1U << u4Offset)) : (u4Val & ~(1U << u4Offset));
	  	GPIO_PDWNC_WRITE(PDWNC_GPIOOUT + (12*(u4Idx)), u4Val);
	}
}

/*****************************************
 * Routine: GPIO_Input
 *****************************************/
INT32 GPIO_Input(INT32 i4GpioNum)
{
    UINT32 u4Val, u4Idx, u4Offset;
	if(i4GpioNum < 0)
	{
		printf("GPIO number(%d) is invalid!\n",i4GpioNum);
		return;
	}
    else if(((UINT32)i4GpioNum >= PIN_GPIO0) &&((UINT32)i4GpioNum <= PIN_sys_usb20_gpio_dm_oe_2p))
	{
		u4Idx = gpio_get_addridx((UINT32)i4GpioNum);
		u4Offset = gpio_get_addroffset((UINT32)i4GpioNum);
		u4Val = GPIO_IN_REG(u4Idx);
		return ((u4Val & (1U << u4Offset)) ? 1 : 0);
	}

	else if((i4GpioNum & EGPIO_FLAG) && (i4GpioNum > PIN_GPIO0))
	{
		u4Offset = gpio_get_addroffset((UINT32)(i4GpioNum - PIN_E_START));
		u4Idx = gpio_get_addridx((UINT32)(i4GpioNum - PIN_E_START));
		
		u4Val = GPIO_PDWNC_READ(PDWNC_GPIOIN + (12*(u4Idx)));
		return ((u4Val & (1U << u4Offset)) ? 1U : 0);
	}
	return ((INT32)0);	
}

static const unsigned long egpio_to_pinmux[PIN_E_MAX - PIN_E_START] = {
    PDWNC_GPIO0_PINMUX, PDWNC_GPIO1_PINMUX, PDWNC_GPIO2_PINMUX,
    PDWNC_GPIO3_PINMUX, PDWNC_GPIO4_PINMUX, PDWNC_GPIO5_PINMUX,
    PDWNC_GPIO6_PINMUX, PDWNC_GPIO7_PINMUX, PDWNC_GPIO8_PINMUX,
    PDWNC_VCLK_PINMUX,  PDWNC_VSTB_PINMUX,  PDWNC_VDATA_PINMUX,
    PDWNC_IR_PINMUX,    PDWNC_GPIO9_PINMUX, PDWNC_GPIO10_PINMUX,
    PDWNC_SCL_PINMUX,   PDWNC_SDA_PINMUX,   PDWNC_AMUTE_PINMUX,
    PDWNC_TRX20_PWR5V_PINMUX, PDWNC_RX20_HTPLG_PINMUX, PDWNC_RX20_PWR5V_PINMUX,
    PDWNC_RX_HDMISCK_PINMUX, PDWNC_RX_HDMISD_PINMUX, PDWNC_TX14_CEC_PINMUX,
    PDWNC_TX14_HDMISCK_PINMUX, PDWNC_TX14_HDMISD_PINMUX, PDWNC_TX14_HTPLG_PINMUX,
    PDWNC_TX20_CEC_PINMUX, PDWNC_TX20_HDMISCK_PINMUX, PDWNC_TX20_HDMISD_PINMUX,
    PDWNC_TX20_HTPLG_PINMUX, PDWNC_OPWRSB_PINMUX, PDWNC_UATXD_PINMUX,
    PDWNC_UARXD_PINMUX,
};


/*****************************************
 * Routine: GPIO_Release
 *****************************************/

void GPIO_Release(INT32 i4GpioNum)
{
	UINT32 unum;
    UINT32 utemp;
	if(i4GpioNum < 0)
	{
		printf("GPIO number(%d) is invalid!\n",i4GpioNum);
		return;
	}
	else if(((UINT32)i4GpioNum >= PIN_GPIO0) &&((UINT32)i4GpioNum <= PIN_VSTB))
    {  
        /*clear pdwnc gpio multifunction to make the pdwnc gpio as top*/
		for(unum =0;unum<34;unum++)
		  {
			 if((UINT32)i4GpioNum == _au2PDWNCIOMUXTable[unum][0])
			  {
			  utemp = GPIO_PDWNC_READ(_au2PDWNCIOMUXTable[unum][1]);
			  GPIO_PDWNC_WRITE(_au2PDWNCIOMUXTable[unum][1],(((~(_au2PDWNCIOMUXTable[unum][3]<<_au2PDWNCIOMUXTable[unum][2]))&utemp)|(_au2PDWNCIOMUXTable[unum][4]<<_au2PDWNCIOMUXTable[unum][2])));
			  break;
			 }
		  }
   
      for (unum = 1; ((unum < (2*_au1GPIOConfig[i4GpioNum][0])) && (unum < 2*MAX_PINMUX_FUNCTION)); unum = unum + 2)
      {
  	    if(BSP_PinGet(_au1GPIOConfig[i4GpioNum][unum]) == _au1GPIOConfig[i4GpioNum][unum+1])
    	{
    		if(_au1GPIOConfig[i4GpioNum][unum] == JTAG_SEL) {
				BSP_PinSet(JTAG_SEL, 7);			
			}
    	    else if(_au1GPIOConfig[i4GpioNum][unum+1] != 0) {
  	          BSP_PinSet(_au1GPIOConfig[i4GpioNum][unum], 0);
	        }

   	     }
	  
      }
	  
    }

	/*set pdwnc gpio mode*/
	else if((i4GpioNum & EGPIO_FLAG) && (i4GpioNum > PIN_GPIO0)) {
//		printf("GPIO_Release number(%d)(0x%x)!\n",i4GpioNum,i4GpioNum);
		UINT32 asPdwnGpio = 0;
		UINT32 pinmux = egpio_to_pinmux[i4GpioNum - PIN_E_START];
		if(((i4GpioNum >= PIN_E_GPIO0) && (i4GpioNum <= PIN_E_GPIO4)) ||
		   ((i4GpioNum >= PIN_E_GPIO7) && (i4GpioNum <= PIN_E_GPIO8)) ||
		   ((i4GpioNum >= PIN_E_GPIO9) && (i4GpioNum <= PIN_E_GPIO10)))
		{
			asPdwnGpio = 5;
		} 
		else if(((i4GpioNum >= PIN_E_GPIO5) && (i4GpioNum <= PIN_E_GPIO6))||
				(i4GpioNum == PIN_E_VCLK) )
		{
			asPdwnGpio = 4;
		}
		
		else if(((i4GpioNum >= PIN_E_VSTB) && (i4GpioNum <= PIN_E_IR))||
				((i4GpioNum >= PIN_E_RTC_SCL) && (i4GpioNum <= PIN_E_RTC_SDA))||
				((i4GpioNum >= PIN_E_TRX20_PWR5V) && (i4GpioNum <= PIN_E_OPWRSB)))
		{
			asPdwnGpio = 2;
		}
		else if(i4GpioNum == PIN_E_AMUTE)
		{
			asPdwnGpio = 1;
		}
		else if(((i4GpioNum >= PIN_E_UATXD) && (i4GpioNum <= PIN_E_UARXD)))
		{
			asPdwnGpio = 3;
		}
		BSP_PinSet(pinmux, asPdwnGpio);
	}

	
}

/*****************************************
 * Routine: GPIO_Config
 *****************************************/
void GPIO_Config(INT32 i4GpioNum, INT32 i4Output, INT32 i4High)
{
	GPIO_Release(i4GpioNum);
	if (i4Output == INPUT)
	{
		GPIO_InOut_Sel(i4GpioNum, INPUT);
	}
	else
	{
		GPIO_Output(i4GpioNum, i4High);
		GPIO_InOut_Sel(i4GpioNum, OUTPUT);
	}
}

/*****************************************
 * Routine: board_init
 * Description: Early hardware init.
 *****************************************/
int pinmux_init (void)
{
#if SUPPORT_JIG
    GPIO_Config(P_XICON_RESET, OUTPUT, HIGH);	//need to set output high to release SICON reset
    GPIO_Config(P_WLAN_RESET, OUTPUT, HIGH);
    GPIO_Config(USB_PCONT1, OUTPUT, LOW);
    GPIO_Config(P_PCONT2, OUTPUT, HIGH);
    GPIO_Config(P_PCONT3, OUTPUT, HIGH);
    GPIO_Config(P_PCONT4, OUTPUT, HIGH);
    GPIO_Config(P_LED, OUTPUT, HIGH);
#endif

#if CONFIG_PIN_LED0 != PIN_UNKNOWN
	GPIO_Config(CONFIG_PIN_LED0, OUTPUT, HIGH); /* init Ethernet PHY LED0 pin as output high */
#endif
#if CONFIG_PIN_LED1 != PIN_UNKNOWN
	GPIO_Config(CONFIG_PIN_LED1, OUTPUT, HIGH); /* init Ethernet PHY LED1 pin as output high */
#endif
#if CONFIG_PIN_LED2 != PIN_UNKNOWN
	GPIO_Config(CONFIG_PIN_LED2, OUTPUT, HIGH); /* init Ethernet PHY LED2 pin as output high */
#endif
#if CONFIG_PIN_LED3 != PIN_UNKNOWN
	GPIO_Config(CONFIG_PIN_LED3, OUTPUT, HIGH); /* init Ethernet PHY LED3 pin as output high */
#endif

    return 0;
}
#if CFG_UPG_LED_BLINK_JSN
//Add for Sony Panel LED blink when upgrade.
void v_set_upg_led(UINT8 bSetLed)
{
    static BOOL bLedStatus = LOW;

    if(0 == bSetLed)
    {
        printf ("do_upg set_upg_led_off\n");
        GPIO_Config(P_LED, OUTPUT, LOW); //Turn Panel LED OFF

        bLedStatus = LOW;
    }
    else if(1 == bSetLed)
    {
        printf ("do_upg set_upg_led_on\n");
        GPIO_Config(P_LED, OUTPUT, HIGH); //Turn Panel LED ON

        bLedStatus = HIGH;
    }
    else
    {
        //printf ("do_upg Reverse LED\n");
        bLedStatus = bLedStatus ? LOW : HIGH;
        GPIO_Config(P_LED, OUTPUT, bLedStatus); //Turn Panel LED ON or OFF

    }
}
#endif
/*****************************************
 * Routine: v_set_upg_status
 *****************************************/
#if (SUPPORT_UPG_LED_CUSTOMER_T)

#define UPG_PIN_RED     PIN_VIND1//PIN_ETTXD1
#define UPG_PIN_GREEN   PIN_VIND0//PIN_ETCRS

extern void udelay (unsigned long usec);

static void led_ctrl_red(BOOL b_on)
{
    GPIO_Output(UPG_PIN_RED, b_on ? LOW : HIGH);
}

static void led_ctrl_green(BOOL b_on)
{
    GPIO_Output(UPG_PIN_GREEN, b_on ? LOW : HIGH);
}

void v_set_upg_status(UPG_STATUS ustatus, u_long size)
{
    static u_long upg_time = 0;
	static int status = 0;
  
    switch(ustatus)
    {
        case UPG_ERASE:
		case UPG_PROGRAM:
		case UPG_READ:
		case UPG_CHECK:
            upg_time += size;
            if(upg_time >= 1000)
            {
                upg_time = 0;
            
                status = !status;
                led_ctrl_green(status);
            }
            break;
            
		case UPG_START:
	        GPIO_PDWNC_WRITE(0x188, 0x245);
            udelay(1000);
                
            GPIO_Config(UPG_PIN_RED, OUTPUT, HIGH);
            GPIO_Config(UPG_PIN_GREEN, OUTPUT, HIGH);
            
            led_ctrl_red(TRUE);
            led_ctrl_green(TRUE);
			return;

		case UPG_END_OK:
            led_ctrl_red(FALSE);
            led_ctrl_green(TRUE);
			return;

		case UPG_END_FAIL:
            led_ctrl_red(TRUE);
            led_ctrl_green(FALSE);
			return;

		default:
			return;
    }
}

#elif SUPPORT_UPG_FLASH_LED  
static UPG_STATUS upg_status = UPG_NONE;
void v_set_upg_status(UPG_STATUS ustatus, u_long size)
{
  #if (UPG_FLASH_LED_FORM == 0)
    static u_long upg_time = 0;
	static int status = 0;
    u_long time_now = 0;
  #endif
  
    switch(ustatus)
    {
      #if (UPG_FLASH_LED_FORM == 0)
        case UPG_ERASE:
			time_now = (size >= 0x40000) ? (3000 * (size/0x40000)) : (3000 /(0x40000/size));
			break;

		case UPG_PROGRAM:
			time_now = (size >= 0x800) ? (1000 * (size/0x800)) : (1000 /(0x800/size));
			break;

		case UPG_READ:
			time_now = (size >= 0x800) ? (30 * (size/0x800)) : (30 /(0x800/size));
			break;

		case UPG_CHECK:
			time_now = (size >= 0x20000) ? (40000 * (size/0x20000)) : (40000 /(0x20000/size));
			break;
      #endif
	   
		case UPG_START:
			upg_status = UPG_START;
		  #if (UPG_FLASH_LED_FORM == 1)
		    GPIO_Output(PIN_CON_RED_LED, HIGH);
			GPIO_Output(PIN_CON_GREEN_LED, HIGH);
		  #elif (UPG_FLASH_LED_FORM == 0)
			GPIO_Output(PIN_CON_RED_LED, LOW);
			GPIO_Output(PIN_CON_GREEN_LED, LOW);
		  #endif	
			return;

		case UPG_END_OK:
			upg_status = UPG_END_OK;
		  #if (UPG_FLASH_LED_FORM == 1)
		    GPIO_Output(PIN_CON_RED_LED, LOW);
			GPIO_Output(PIN_CON_GREEN_LED, HIGH);
		  #elif (UPG_FLASH_LED_FORM == 0)	
			GPIO_Output(PIN_CON_RED_LED, HIGH);
			GPIO_Output(PIN_CON_GREEN_LED, HIGH);
		  #endif 	
			return;

		case UPG_END_FAIL:
			upg_status = UPG_END_FAIL; 
		  #if (UPG_FLASH_LED_FORM == 1)
		    GPIO_Output(PIN_CON_RED_LED, HIGH);
			GPIO_Output(PIN_CON_GREEN_LED, LOW);
		  #elif (UPG_FLASH_LED_FORM == 0)	
			GPIO_Output(PIN_CON_RED_LED, LOW);
			GPIO_Output(PIN_CON_GREEN_LED, LOW);
		  #endif	
			return;

		default:
			return;
    }

  #if (UPG_FLASH_LED_FORM == 0)	
    upg_time += time_now;

	if(upg_time >= 200000)
	{
	    upg_time = 0;

		status = (status == 0) ? 1 : 0;

	    if(status == 1)
        {
    	    GPIO_Output(PIN_CON_GREEN_LED, HIGH);
        }
    	else
    	{
    		GPIO_Output(PIN_CON_GREEN_LED, LOW);
    	}	
	}
  #endif	
}

void v_show_upg_result(void)
{
  #if (UPG_FLASH_LED_FORM == 0)
    static int flash_count = 0;
	
    if(upg_status != UPG_END_FAIL)
		return;

	switch(flash_count)
	{
		case 1:
			GPIO_Output(PIN_CON_GREEN_LED, HIGH);
			break;

		case 2:
			GPIO_Output(PIN_CON_GREEN_LED, LOW);
			GPIO_Output(PIN_CON_RED_LED, HIGH);
			break;

		case 3:
			GPIO_Output(PIN_CON_RED_LED, LOW);
			break;

		case 9:
			flash_count = 0;
			return;

		default:
			break;
	}

	flash_count ++;
  #endif	
}
#endif

//================================================
// GPIO JIG Mode Private Function
//================================================
#if SUPPORT_JIG
void vEnableJigMode0(void)
{
    GPIO_Config(JIG_MODE0, INPUT, HIGH);        	    
}

void vEnableJigMode1(void)
{
    GPIO_Config(JIG_MODE1, INPUT, HIGH);    
}

BOOL fgGetJigMode0(void) //JIG_MODE0
{
    if(GPIO_Input(JIG_MODE0))   
    {
        return TRUE;
    }
    return FALSE;
}

BOOL fgGetJigMode1(void)  //JIG_MODE1
{
    if(GPIO_Input(JIG_MODE1))   
    {
        return TRUE;
    }
    return FALSE;
}

// for jig mode check
BOOL fg_ne_bl_chk_jig(void)
{
    vEnableJigMode0();
    vEnableJigMode1();
    
    if(fgGetJigMode0()==TRUE && fgGetJigMode1()==FALSE)
    {
        return TRUE;
    }
    return FALSE;
}
#endif

//=========================================================
// check gpio7 and PIN_ETTXER status, if connected, return true.   for pana fe upg request
//=========================================================
#if(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8550)
BOOL fgCheckFeCtlReg(void)
{
   GPIO_Config(PIN_ETTXER, OUTPUT, LOW);	
//   GPIO_Config(PIN_GPIO7, INPUT, HIGH);

   GPIO_Output(PIN_ETTXER, HIGH); 
   if(!GPIO_Input(PIN_GPIO7))
     return FALSE;
   GPIO_Output(PIN_ETTXER, LOW); 
   if(GPIO_Input(PIN_GPIO7))
     return FALSE;
   else
     return TRUE;
}
#elif (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560)
//=========================================================
// check PIN_VOUTD1 status, if PD, return true.   for pana fe upg request
//=========================================================
BOOL fgCheckFeCtlReg(void)
{
   GPIO_Config(PIN_AOSDATA4, INPUT, HIGH);

   if(GPIO_Input(PIN_AOSDATA4))
     return FALSE;
   else
     return TRUE;
}
#elif (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580)
BOOL fgCheckFeCtlReg(void)
{
    // return TRUE;
   GPIO_Config(PIN_FEGIO9, INPUT, HIGH);
   if(GPIO_Input(PIN_FEGIO9))
   	{
	  printf("Ylf-->PIN_FEGIO9 input =1\n");
     return FALSE;
   	}
   else
   	{
	  printf("Ylf-->PIN_FEGIO9 input =0\n");
     return TRUE;
   	}
}
#else
BOOL fgCheckFeCtlReg(void)
{
    return TRUE;
}
#endif 
//extern void UPG_LOG(int level, const char *fmt, ...);
BOOL fgCheckUSBProtectCircuit(void)
{
//#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580)    
//    return FALSE; //TRUE; alway enable usb power and update fe//set false because two type dvd.bin416/417
//#else 
    return TRUE; //TRUE; alway enable usb power and update fe
//#endif
    
}

//================================================
// Update UPG_STATUS for during upgrade FW.
//================================================
#if SUPPORT_UPG_STATUS
void vSetUpgStatus(BOOL fgVal)
{
	if (!fgVal)
	{
		GPIO_Output(UPG_STATUS_PIN, LOW);  
	}
	else
	{
		GPIO_Output(UPG_STATUS_PIN, HIGH);
	}
}
#endif
 
//================================================
// NAND Write Protect
//================================================
#if SUPPORT_WP
BOOL  NAND_COMMON_WriteProtect(BOOL bMode) 
{
    if ( TRUE == bMode) //write protect
    {
        GPIO_Output(NAND_WP, LOW);    
    }
    else //release write protection
    {
        GPIO_Output(NAND_WP, HIGH);
    }
    return TRUE;
}
#endif

