/*
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 *  Copyright(C) 2006 NXP BV, All rights reserved.
 *  by Jean-Paul Saman
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <common.h>
#include <asm/io.h>
#include <asm/arch/mt85xx.h>
#include <asm/arch/sys_proto.h>
#include <asm/arch/sys_info.h>
#include <asm/mach-types.h>
#include <asm/arch/x_typedef.h>
#include <asm/arch/x_ckgen.h>
#include <mt85xx_gpio.h>
#include "gpio_config.h"
#include <chip_ver.h>
#include <upg_config.h>

#ifndef __UBOOT_GPIO_PINMUX__
#define __UBOOT_GPIO_PINMUX__
#endif
#include <asm/arch/mt85xx_gpio_pinmux.h>
#include <asm/arch/mt85xx_pinmux_table.h>

//-----------------------------------------------------------------------------
// definitions
//-----------------------------------------------------------------------------
#define TOTAL_GPIO_IDX          5
#define GPIO_INDEX_MASK         ((1 << 5) - 1)

//pdwnc
#define PDWNC_PAD_PINMUX1   0x0F4
#define PDWNC_PAD_PINMUX2   0x0F8
#define PDWNC_PAD_PINMUX3   0x0FC

#define PDWNC_PAD_PINMUX4   0x108
#define PDWNC_PAD_PINMUX5   0x10c

#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563)
#define PDWNC_EGPIO_EN     0x320
#define PDWNC_EGPIO_IN     0x324
#define PDWNC_EGPIO_OUT     0x328
#endif

#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8561)
#define PDWNC_EGPIO_SEL     0x32c
#endif

#define PDWNC_GPIOIN        0x0D0
#define PDWNC_GPIOEN        0x0D4
#define PDWNC_GPIOOUT       0x0D8

#define GPIO_EN_WRITE(idx,val)          CKGEN_WRITE32((REG_RW_GPIO_EN_OFFSET+(4*(idx))), (val))
#define GPIO_OUT_WRITE(idx,val)         CKGEN_WRITE32((REG_RW_GPIO_OUT_OFFSET+(4*(idx))), (val))
#define GPIO_IN_WRITE(idx,val)          CKGEN_WRITE32((REG_RW_GPIO_IN_OFFSET+(4*(idx))), (val))
#define GPIO_EN_REG(idx)                CKGEN_REG32(REG_RW_GPIO_EN_OFFSET+(4*(idx)))
#define GPIO_OUT_REG(idx)               CKGEN_REG32(REG_RW_GPIO_OUT_OFFSET+(4*(idx)))
#define GPIO_IN_REG(idx)                CKGEN_REG32(REG_RW_GPIO_IN_OFFSET+(4*(idx)))

#define PINMUX_WRITE(idx,val)           CKGEN_WRITE32((REG_RW_PINMUX_OFFSET+(4*(idx))), (val))
#define PINMUX_REG(idx)                 CKGEN_REG32(REG_RW_PINMUX_OFFSET+(4*(idx)))

#define GPIO_PDWNC_READ(i4Addr)         (*((volatile UINT32*)(PDWNC_BASE + (i4Addr)))) 
#define GPIO_PDWNC_WRITE(i4Addr, u4Val) ((*((volatile UINT32*)(PDWNC_BASE + (i4Addr)))) = (u4Val))

/*****************************************
 * Routine: BSP_PinSet
 *****************************************/
INT32 BSP_PinSet(INT32 i4FuncSel, INT32 i4Func)
{
	UINT32 u4Mask;
    UINT32 u4Val;
	
	if ((i4Func < 0) ||
		(i4Func > MAX_PINMUX_FUNCTION) ||
		(i4Func > _au1PinmuxFunctionMasks[i4FuncSel]) ||
		(i4FuncSel < 0) ||
		(i4FuncSel > MAX_PINMUX_SEL) ||
		(_au1PinmuxFunctionMasks[i4FuncSel] == 0))
	{
		return -1;
	}

#if(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560)
if(i4FuncSel >= 256)
{
	u4Mask = ~((UINT32)_au1PinmuxFunctionMasks[i4FuncSel] << ((i4FuncSel - 256)%32));
	u4Val = CKGEN_REG32(REG_RW_PAD_PWMCTL_0+(4*((i4FuncSel - 256)/32)));
	u4Val &= u4Mask;
	u4Val |= ((UINT32)i4Func << ((i4FuncSel - 256)%32));

	CKGEN_WRITE32(REG_RW_PAD_PWMCTL_0+(4*((i4FuncSel - 256)/32)),u4Val);
}
else
#elif((CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8550) \
    ||(CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8555)  \
    ||(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8561)  \
    ||(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563)  \
    ||(CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8580))

#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563)
	if(i4FuncSel & PDWNC_PINMUX_FLAG) /* pdwnc pinmux*/
	{
		unsigned value = 0;
		unsigned reg = 0xF4+(i4FuncSel & PD_PM_OFFSET_MASK) >> PD_PM_OFFSET;
		unsigned startBit = (i4FuncSel & PD_PM_START_BIT_MASK) >> PD_PM_START_BIT;
		unsigned mask = (i4FuncSel & PD_PM_MASK_MASK) >> PD_PM_MASK;
		value = GPIO_PDWNC_READ(reg);
		GPIO_PDWNC_WRITE(reg, value & (~(mask << startBit)) | (i4Func << startBit));
		return 0;
	}
	else
#endif
	if(i4FuncSel >= SEL_FECTL_0)
	{
        u4Mask = ~((UINT32)_au1PinmuxFunctionMasks[i4FuncSel] << ((i4FuncSel - SEL_FECTL_0)%32));
		u4Val = CKGEN_REG32(REG_RW_PAD_FECTL_0+(4*((i4FuncSel - SEL_FECTL_0)/32)));
    	u4Val &= u4Mask;
    	u4Val |= ((UINT32)i4Func << ((i4FuncSel - SEL_FECTL_0)%32));

	    CKGEN_WRITE32(REG_RW_PAD_FECTL_0+(4*((i4FuncSel - SEL_FECTL_0)/32)),u4Val);
	}
	else
#endif
    {
    	u4Mask = ~((UINT32)_au1PinmuxFunctionMasks[i4FuncSel] << (i4FuncSel%32));
    	u4Val = PINMUX_REG(i4FuncSel/32);
    	u4Val &= u4Mask;
    	u4Val |= ((UINT32)i4Func << (i4FuncSel%32));
    	
        PINMUX_WRITE((i4FuncSel/32), (u4Val));
    }

	return 0;
}

/*****************************************
 * Routine: BSP_PinGet
 *****************************************/
INT32 BSP_PinGet(INT32 i4FuncSel)
{
    INT32 i4Data;

	if ((i4FuncSel < 0) ||
		(i4FuncSel > MAX_PINMUX_SEL))
	{
		return -1;
	}
	
#if(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560)
if(i4FuncSel >= 256)		
	i4Data = (CKGEN_REG32(REG_RW_PAD_PWMCTL_0+(4*((i4FuncSel - 256)/32))) 
				>> ((i4FuncSel - 256)%32)) & _au1PinmuxFunctionMasks[i4FuncSel];
else

#elif((CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8550)  \
        ||(CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8555)  \
        ||(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8561)  \
        ||(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563)  \
        ||(CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8580))
	if(i4FuncSel >= SEL_FECTL_0)		
		i4Data = (CKGEN_REG32(REG_RW_PAD_FECTL_0+(4*((i4FuncSel - SEL_FECTL_0)/32))) 
		            >> ((i4FuncSel - SEL_FECTL_0)%32)) & _au1PinmuxFunctionMasks[i4FuncSel];
	else
#endif 
    i4Data = (PINMUX_REG(i4FuncSel/32) >> (i4FuncSel%32)) & _au1PinmuxFunctionMasks[i4FuncSel];
	
    return (i4Data);
}

/*****************************************
 * Routine: GPIO_InOut_Sel
 *****************************************/
void GPIO_InOut_Sel(INT32 i4GpioNum, INT32 i4Output)
{		
	UINT32 u4Val;
	INT32 i4Idx;
   
	i4Idx = ((UINT32)i4GpioNum >> TOTAL_GPIO_IDX);
#if((CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560) \
		||(CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8580) \
		||(CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8581))
    if((UINT32)i4GpioNum <= PIN_USB_DM_P2)
#elif ((CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8561) || (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563))
    if((UINT32)i4GpioNum <= PIN_VFD_STB)
#else
	if((UINT32)i4GpioNum <= PIN_TCK)
#endif
	{	   
	  u4Val = GPIO_EN_REG(i4Idx);	 
	  u4Val = (i4Output) ? (u4Val | (1U << (i4GpioNum & GPIO_INDEX_MASK))) : (u4Val & ~(1U << (i4GpioNum & GPIO_INDEX_MASK)));
	  GPIO_EN_WRITE(i4Idx, u4Val);
	}
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563)
	else if(((UINT32)i4GpioNum & EGPIO_FLAG))
	{
		u4Val = GPIO_PDWNC_READ(PDWNC_EGPIO_EN);	  
		u4Val = (i4Output) ? (u4Val | (1U << (i4GpioNum - PIN_E_START))) : (u4Val & ~(1U << (i4GpioNum - PIN_E_START)));
		GPIO_PDWNC_WRITE(PDWNC_EGPIO_EN, u4Val);
	}
#else
	else if((UINT32)i4GpioNum <= TOTAL_GPIO_NUM)
	{
	  u4Val = GPIO_PDWNC_READ(PDWNC_GPIOEN);	
	  u4Val = (i4Output) ? (u4Val | (1U << (i4GpioNum - PIN_VFD_STB))) : (u4Val & ~(1U << (i4GpioNum - PIN_VFD_STB)));
	  GPIO_PDWNC_WRITE(PDWNC_GPIOEN, u4Val);
	}
#endif

}  

/*****************************************
 * Routine: GPIO_Output
 *****************************************/
void GPIO_Output(INT32 i4GpioNum, INT32 i4High)
{
	UINT32 u4Val;
	INT32 i4Idx;
   
	i4Idx = ((UINT32)i4GpioNum >> TOTAL_GPIO_IDX);
   
#if((CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560) \
    ||(CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8580) \
    ||(CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8581)) 
	   if((UINT32)i4GpioNum <= PIN_USB_DM_P2)
#elif ((CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8561) || (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563))
       if((UINT32)i4GpioNum <= PIN_VFD_STB)
#else
	   if((UINT32)i4GpioNum <= PIN_TCK)
#endif

	{
	  u4Val = GPIO_OUT_REG(i4Idx);
	  u4Val = (i4High) ? (u4Val | (1U << (i4GpioNum & GPIO_INDEX_MASK))) : (u4Val & ~(1U << (i4GpioNum & GPIO_INDEX_MASK)));
	  GPIO_OUT_WRITE(i4Idx, u4Val);
	}
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563)
	else if(((UINT32)i4GpioNum & EGPIO_FLAG))
	{
	  u4Val = GPIO_PDWNC_READ(PDWNC_EGPIO_OUT);	  
	  u4Val = (i4High) ? (u4Val | (1U << (i4GpioNum - PIN_E_START))) : (u4Val & ~(1U << (i4GpioNum - PIN_E_START)));
	  GPIO_PDWNC_WRITE(PDWNC_EGPIO_OUT, u4Val);
	}
#else
	else if((UINT32)i4GpioNum <= TOTAL_GPIO_NUM)
	{
	  u4Val = GPIO_PDWNC_READ(PDWNC_GPIOOUT);	  
	  u4Val = (i4High) ? (u4Val | (1U << (i4GpioNum - PIN_VFD_STB))) : (u4Val & ~(1U << (i4GpioNum - PIN_VFD_STB)));
	  GPIO_PDWNC_WRITE(PDWNC_GPIOOUT, u4Val);
	}
#endif
}

/*****************************************
 * Routine: GPIO_Input
 *****************************************/
INT32 GPIO_Input(INT32 i4GpioNum)
{
	UINT32 u4Val;
	INT32 i4Idx;

#if((CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560) \
    ||(CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8580) \
    ||(CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8581))  
		if((UINT32)i4GpioNum <= PIN_USB_DM_P2)
#elif ((CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8561) || (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563))
        if((UINT32)i4GpioNum <= PIN_VFD_STB)
#else
		if((UINT32)i4GpioNum <= PIN_TCK)
#endif

	{
	  i4Idx = ((UINT32)i4GpioNum >> TOTAL_GPIO_IDX);
	  u4Val = GPIO_IN_REG(i4Idx);
	
	  return ((u4Val & (1U << (i4GpioNum & GPIO_INDEX_MASK))) ? 1U : 0);
	}
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563)
	else if(((UINT32)i4GpioNum & EGPIO_FLAG))
	{
		u4Val = GPIO_PDWNC_READ(PDWNC_EGPIO_IN);
		return ((u4Val & (1U << (i4GpioNum - PIN_E_START))) ? 1U : 0);
	}
#else
	else if((UINT32)i4GpioNum <= TOTAL_GPIO_NUM)
	{
	  u4Val = GPIO_PDWNC_READ(PDWNC_GPIOIN);
	  return ((u4Val & (1U << (i4GpioNum - PIN_VFD_STB))) ? 1U : 0);
	}
#endif
	return ((INT32)0);
}

#if (CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8563)
static const unsigned long egpio_to_pinmux[PIN_E_MAX - PIN_E_START] = {
	PDWNC_OPWRSB_PINMUX, PDWNC_GPIO8_PINMUX, PDWNC_GPIO7_PINMUX,
	PDWNC_GPIO6_PINMUX, PDWNC_GPIO5_PINMUX, PDWNC_GPIO4_PINMUX,
	PDWNC_GPIO3_PINMUX, PDWNC_GPIO2_PINMUX, PDWNC_GPIO1_PINMUX,
	PDWNC_GPIO0_PINMUX, PDWNC_VCLK_PINMUX, PDWNC_VSTB_PINMUX,
	PDWNC_VDATA_PINMUX, PDWNC_IR_PINMUX, PDWNC_UATXD_PINMUX,
	PDWNC_LED1_PINMUX, PDWNC_LED0_PINMUX, PDWNC_HDMI_CEC_RX_PINMUX,
	PDWNC_HDMI_SDA_RX_PINMUX, PDWNC_HDMI_SCL_RX_PINMUX, PDWNC_MHL_SENCE_PINMUX,
	PDWNC_MHL_SENCE_PINMUX, PDWNC_HDMI_HPD_CBUS_RX_PINMUX, PDWNC_CEC_PINMUX,
	PDWNC_HTPLG_PINMUX, PDWNC_HDMISD_PINMUX, PDWNC_HDMISCK_PINMUX,
	PDWNC_CBUS_PINMUX, PDWNC_AMUTE_PINMUX,
};
#endif
/*****************************************
 * Routine: GPIO_Release
 *****************************************/
void GPIO_Release(INT32 i4GpioNum)
{
	UINT32 u4Num;
#if((CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8561)  \
        ||(CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8563)  \
        ||(CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8580))
	UINT32 u4Temp;
#endif
#if ((CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8561) || (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563))
    UINT32 u4Index;
#endif

#if((CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560) \
    ||(CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8580) \
    ||(CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8581))
		if((UINT32)i4GpioNum <= PIN_USB_DM_P2)
#elif ((CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8561) || (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563))
        if((UINT32)i4GpioNum <= PIN_VFD_STB)			
#else
		if((UINT32)i4GpioNum <= PIN_TCK)
#endif
	{
#if(CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8580)
    if((i4GpioNum>=PIN_USB_DP_P0)&&(i4GpioNum<=PIN_USB_DM_P2))
    {
         if((i4GpioNum ==PIN_USB_DP_P0)||(i4GpioNum ==PIN_USB_DM_P0))
         {
             u4Temp = CKGEN_READ32(0xF820);
		     CKGEN_WRITE32(0xF820,(0x01<<9)|u4Temp);
	     } 
	     else if((i4GpioNum ==PIN_USB_DP_P1)||(i4GpioNum ==PIN_USB_DM_P1))
	     {
             u4Temp = CKGEN_READ32(0xF920);
		     CKGEN_WRITE32(0xF920,(0x01<<9)|u4Temp);
	     }
	     else if((i4GpioNum ==PIN_USB_DP_P2)||(i4GpioNum ==PIN_USB_DM_P2))
         {
       	     u4Temp = CKGEN_READ32(0x35820);
		     CKGEN_WRITE32(0x35820,(0x01<<9)|u4Temp);
	     }
	}
    else if((i4GpioNum==PIN_FEDMO)||(i4GpioNum>=PIN_FEFMO)&&(i4GpioNum<=PIN_FEFMO4))
	{
	     GPIO_PDWNC_WRITE(0x30, 0x85);
	     GPIO_PDWNC_WRITE(0x34, 0x01);
	     for(u4Num = 0;u4Num<=100;u4Num++);//may be for memory barrier
	     CKGEN_WRITE32(0x5c704, 0x801);
	     u4Temp = CKGEN_READ32(0x6209c);
	     if(i4GpioNum == PIN_FEDMO)
		    CKGEN_WRITE32(0x6209c, (0x01<<17)|u4Temp);
	     else if(i4GpioNum == PIN_FEFMO)
		    CKGEN_WRITE32(0x6209c, (0x01<<16)|u4Temp);
	     else if(i4GpioNum == PIN_FEFMO2)
		    CKGEN_WRITE32(0x6209c, (0x01<<18)|u4Temp);
	     else if(i4GpioNum == PIN_FEFMO3)
		    CKGEN_WRITE32(0x6209c, (0x01<<23)|u4Temp);
	     else if(i4GpioNum == PIN_FEFMO4)
		    CKGEN_WRITE32(0x6209c, (0x01<<22)|u4Temp);
	  }
	  else
	  {
	      for(u4Num =0;u4Num<26;u4Num++)
		  {
		      if(i4GpioNum == _au2PDWNCIOMUXTable[u4Num][0])
			  {
			      //UPG_LOG(1, "PDWNCIOMUX gpio is %d\n",i4GpioNum);
			      u4Temp = GPIO_PDWNC_READ(_au2PDWNCIOMUXTable[u4Num][1]);          
			      GPIO_PDWNC_WRITE(_au2PDWNCIOMUXTable[u4Num][1],(((~(_au2PDWNCIOMUXTable[u4Num][3]<<_au2PDWNCIOMUXTable[u4Num][2]))&u4Temp)|(_au2PDWNCIOMUXTable[u4Num][4]<<_au2PDWNCIOMUXTable[u4Num][2])));
			      break;
			 }
		  }
	  }
#endif
#if	(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563)
    if (i4GpioNum == PIN_FECFREQ || (i4GpioNum >= PIN_FECMOD && i4GpioNum <= PIN_FOO) 
    	|| i4GpioNum >= PIN_TLO && i4GpioNum <= PIN_TRO)
    {
    	u4Temp = CKGEN_READ32(0x280);
    	if (i4GpioNum == PIN_FECFREQ) 
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 20) | u4Temp);
    	}
    	else if (i4GpioNum == PIN_FECMOD)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 22) | u4Temp);
    	} 
    	else if (i4GpioNum == PIN_FEDMO)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 26) | u4Temp);
    	}    	
    	else if (i4GpioNum == PIN_FEEJECT)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 15) | u4Temp);
    	}
    	else if (i4GpioNum == PIN_FEFG_)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 14) | u4Temp);
    	} 
    	else if (i4GpioNum == PIN_FEFMO)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 24) | u4Temp);
    	}    	
    	else if (i4GpioNum == PIN_FEFMO2)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 25) | u4Temp);
    	}   
    	else if (i4GpioNum == PIN_FEFMO3)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 30) | u4Temp);
    	}
    	else if (i4GpioNum == PIN_FEFMO4)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 31) | u4Temp);
    	} 
    	else if (i4GpioNum == PIN_FEGAINSW1)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 27) | u4Temp);
    	}    	
    	else if (i4GpioNum == PIN_FEGAINSW2)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 28) | u4Temp);
    	} 
    	else if (i4GpioNum == PIN_FEGAINSW3)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 29) | u4Temp);
    	}   
    	else if (i4GpioNum == PIN_FEGIO0)
    	{
    		CKGEN_WRITE32(0x280,(0x01) | u4Temp);
    	}
    	else if (i4GpioNum == PIN_FEGIO1)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 1) | u4Temp);
    	} 
    	else if (i4GpioNum == PIN_FEGIO10)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 10) | u4Temp);
    	}    	
    	else if (i4GpioNum == PIN_FEGIO11)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 11) | u4Temp);
    	}
    	else if (i4GpioNum == PIN_FEGIO3)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 3) | u4Temp);
    	}
    	else if (i4GpioNum == PIN_FEGIO4)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 4) | u4Temp);
    	} 
    	else if (i4GpioNum == PIN_FEGIO5)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 5) | u4Temp);
    	}    	
    	else if (i4GpioNum == PIN_FEGIO6)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 6) | u4Temp);
    	} 
    	else if (i4GpioNum == PIN_FEGIO7)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 7) | u4Temp);
    	}
    	else if (i4GpioNum == PIN_FEGIO9)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 9) | u4Temp);
    	} 
    	else if (i4GpioNum == PIN_FEOSCEN)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 21) | u4Temp);
    	}    	
    	else if (i4GpioNum == PIN_FETRAYIN_)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 17) | u4Temp);
    	}      	
    	else if (i4GpioNum == PIN_FETRAYOUT_)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 18) | u4Temp);
    	}    	
    	else if (i4GpioNum == PIN_FETRAYPWM)
    	{
    		CKGEN_WRITE32(0x280,(0x01 << 23) | u4Temp);
    	}  
    	else if (i4GpioNum == PIN_FOO)
    	{
    		CKGEN_WRITE32(0x284,(0x01) | u4Temp);
    	}  
    	else if (i4GpioNum == PIN_TLO)
    	{
    		CKGEN_WRITE32(0x284,(0x01 << 2) | u4Temp);
    	}  
    	else if (i4GpioNum == PIN_TRO)
    	{
    		CKGEN_WRITE32(0x284,(0x01 << 1) | u4Temp);
    	}      	   	    	   	      	     	 	
    }
    else if (i4GpioNum >= PIN_SD_CLK && i4GpioNum <= PIN_SD_DATA3)
    {
    	CKGEN_WRITE32(0xc4, (CKGEN_READ32(0xc4) & (~(0x1 << 16))));
    }
#endif	
	
	  for (u4Num = 1; ((u4Num < (2*_au1GPIOConfig[i4GpioNum][0])) && (u4Num < 2*MAX_PINMUX_FUNCTION)); u4Num = u4Num + 2)
	  {
 #if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580) 
	      if(_au1GPIOConfig[i4GpioNum][u4Num] == JTAG_SEL)
        {
           if((BSP_PinGet(_au1GPIOConfig[i4GpioNum][u4Num]) == _au1GPIOConfig[i4GpioNum][u4Num+1])
		   	   && (BSP_PinGet(_au1GPIOConfig[i4GpioNum][u4Num+2]) == _au1GPIOConfig[i4GpioNum][u4Num+3]))
           {
           	   BSP_PinSet(_au1GPIOConfig[i4GpioNum][u4Num], 0); 
			         BSP_PinSet(_au1GPIOConfig[i4GpioNum][u4Num+2], 0);
           }
        }
		else			
  #endif
		if(BSP_PinGet(_au1GPIOConfig[i4GpioNum][u4Num]) == _au1GPIOConfig[i4GpioNum][u4Num+1])
		{
			if(_au1GPIOConfig[i4GpioNum][u4Num+1] == 0)
			{//for SFCK, SFCS, SFDO, SFDI
			  BSP_PinSet(_au1GPIOConfig[i4GpioNum][u4Num], 1);
			}
			else
			{
			  BSP_PinSet(_au1GPIOConfig[i4GpioNum][u4Num], 0);
			}
		 }
	   }
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8561)
        if((i4GpioNum >= PIN_GPIO2) && (i4GpioNum <= PIN_GPIO7))
        {
            GPIO_PDWNC_WRITE(PDWNC_EGPIO_SEL, 0x00000001);
        }
		  
		  if(i4GpioNum == PIN_GPIO0)
		  {
			  u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX5);
			  u4Num = (u4Num & ~(1U << 5));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX5, u4Num);
		  }
	  
		  if(i4GpioNum == PIN_GPIO0 || i4GpioNum == PIN_VFD_DATA)
		  {
			  u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX3);
			  u4Num = (u4Num | (1U << 3));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX3, u4Num);
		  }
	  
		  if(i4GpioNum == PIN_GPIO1 || i4GpioNum == PIN_VFD_STB)
		  {
			  u4Num= GPIO_PDWNC_READ(PDWNC_PAD_PINMUX3);
			  u4Num = (u4Num | (1U << 2));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX3, u4Num);
		  }
		  
		  for(u4Temp = 0; u4Temp < 23; u4Temp ++)
		  {
			  if(_au2GPIORemapTable[u4Temp][0] == (unsigned short)i4GpioNum)
			  { 		  
				  if(_au2GPIORemapTable[u4Temp][1] != 32)
				  {
					  u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX1);
					  u4Num= (u4Num & ~(1U << _au2GPIORemapTable[u4Temp][1]));
					  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX1, u4Num);
				  }
				  
				  if(_au2GPIORemapTable[u4Temp][2] != 32)
				  {
					  u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX2);
					  u4Num = (u4Num & ~(1U << _au2GPIORemapTable[u4Temp][2]));
					  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX2, u4Num);
				  }
	  
				  if(_au2GPIORemapTable[u4Temp][3] != 32)
				  {
					  u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX5);
					  u4Num = (u4Num & ~(1U << _au2GPIORemapTable[u4Temp][3]));
					  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX5, u4Num);
				  }
				  
				  if(_au2GPIORemapTable[u4Temp][4] != 32)
				  {
					  u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX5);
					  u4Num = (u4Num | (1U << _au2GPIORemapTable[u4Temp][4]));
					  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX5, u4Num);
				  } 		  
				  break;
			  }
		  }
	  
		  for(u4Temp = 0; u4Temp < 8; u4Temp ++)
		  {
			  for(u4Index = 2; u4Index < 9; u4Index++)
			  {
				  if(_au2GPIOConfig[u4Temp][u4Index] == (unsigned short)i4GpioNum)
				  {
					  u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX4);
					  u4Num = (u4Num>>_au2GPIOConfig[u4Temp][0])&_au2GPIOConfig[u4Temp][1];
					  if(u4Num == u4Index-1)
					  {
						  u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX4);
						  u4Num = (u4Num & ~(_au2GPIOConfig[u4Temp][1] << _au2GPIOConfig[u4Temp][0]));
						  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX4, u4Num);
	  
						  if(u4Temp == 0)
						  {
							  u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX3);
							  u4Num = (u4Num | 1U);
							  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX3, u4Num);
						  }
						  else if(u4Temp == 1)
						  {
							  u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX3);
							  u4Num = (u4Num | (1U << 1));
							  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX3, u4Num);
						  }
					  }
					  break;
				  }
			  }
        }
        return;
#elif (CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8563)
    if (i4GpioNum >= PIN_GPIO0 && i4GpioNum <= PIN_AMUTE)
    {
      if(i4GpioNum == PIN_GPIO0)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX1);
			  u4Num = (u4Num & ~(7U << 26));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX1, u4Num);
      }    
      else if(i4GpioNum == PIN_GPIO1)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX1);
			  u4Num = (u4Num & ~(7U << 23));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX1, u4Num);
      }     
      else if(i4GpioNum == PIN_GPIO2)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX1);
			  u4Num = (u4Num & ~(7U << 20));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX1, u4Num);
      }     
      else if(i4GpioNum == PIN_GPIO3)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX1);
			  u4Num = (u4Num & ~(7U << 17));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX1, u4Num);
      }   
      else if(i4GpioNum == PIN_GPIO4)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX1);
			  u4Num = (u4Num & ~(7U << 14));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX1, u4Num);
      }     
      else if(i4GpioNum == PIN_GPIO5)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX1);
			  u4Num = (u4Num & ~(7U << 11));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX1, u4Num);
      }     
      else if(i4GpioNum == PIN_GPIO6)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX1);
			  u4Num = (u4Num & ~(7U << 8));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX1, u4Num);
      }     
      else if(i4GpioNum == PIN_GPIO7)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX1);
			  u4Num = (u4Num & ~(7U << 5));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX1, u4Num);
      }    
      else if(i4GpioNum == PIN_GPIO8)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX1);
			  u4Num = (u4Num & ~(7U << 2));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX1, u4Num);
      } 
      else if(i4GpioNum == PIN_AMUTE)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX3);
			  u4Num = (u4Num & ~(3U << 15));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX3, u4Num);
      }    	
    }
    else if (i4GpioNum >= PIN_HDMI_CEC_RX && i4GpioNum <= PIN_MHL_SENCE)
    {
      if(i4GpioNum == PIN_HDMI_CEC_RX)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX2);
			  u4Num = (u4Num & ~(7U << 16));
			  u4Num = u4Num | (1U << 16);
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX2, u4Num);
      } 
      else if(i4GpioNum == PIN_HDMI_HPD_CBUS_RX)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX2);
			  u4Num = (u4Num & ~(7U << 28));
			  u4Num = u4Num | (1U << 28);
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX2, u4Num);
      } 
      else if(i4GpioNum == PIN_HDMI_HTPLG)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX3);
			  u4Num = (u4Num & ~(7U << 3));
			  u4Num = u4Num | (1U << 3);
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX3, u4Num);
      }     
      else if(i4GpioNum == PIN_HDMI_SD)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX3);
			  u4Num = (u4Num & ~(7U << 6));
			  u4Num = u4Num | (1U << 6);
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX3, u4Num);
      }     
      else if(i4GpioNum == PIN_HDMI_SCK)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX3);
			  u4Num = (u4Num & ~(7U << 9));
			  u4Num = u4Num | (1U << 9);
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX3, u4Num);
      }     
      else if(i4GpioNum == PIN_HDMI_SCL_RX)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX2);
			  u4Num = (u4Num & ~(7U << 22));
			  u4Num = u4Num | (1U << 22);
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX2, u4Num);
      } 
      else if(i4GpioNum == PIN_HDMI_SDA_RX)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX2);
			  u4Num = (u4Num & ~(7U << 19));
			  u4Num = u4Num | (1U << 19);
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX2, u4Num);
      } 
      else if(i4GpioNum == PIN_IR)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX2);
			  u4Num = (u4Num & ~(3U << 4));
			  u4Num = u4Num | (1U << 4);
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX2, u4Num);
      }          
      else if(i4GpioNum == PIN_LED0)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX2);
			  u4Num = (u4Num & ~(7U << 13));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX2, u4Num);
      }   
      else if(i4GpioNum == PIN_LED1)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX2);
			  u4Num = (u4Num & ~(7U << 10));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX2, u4Num);
      } 
      else if(i4GpioNum == PIN_MHL_SENCE)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX2);
			  u4Num = (u4Num & ~(7U << 25));
			  u4Num = u4Num | (1U << 25);
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX2, u4Num);
      }             	
    }
    else if (i4GpioNum >= PIN_UARXD && i4GpioNum <= PIN_VFD_STB)
    {
      if(i4GpioNum == PIN_UARXD)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX2);
			  u4Num = (u4Num & ~(3U << 8));
			  u4Num = u4Num | (1U << 8);
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX2, u4Num);
      }     
      else if(i4GpioNum == PIN_UATXD)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX2);
			  u4Num = (u4Num & ~(3U << 6));
			  u4Num = u4Num | (1U << 6);
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX2, u4Num);
      }  
      else if(i4GpioNum == PIN_VFD_CLK)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX1);
			  u4Num = (u4Num & ~(7U << 29));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX1, u4Num);
      }  
      else if(i4GpioNum == PIN_VFD_DATA)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX2);
			  u4Num = (u4Num & ~(3U << 2));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX2, u4Num);
      } 
      else if(i4GpioNum == PIN_VFD_STB)
      {
        u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX2);
			  u4Num = (u4Num & ~(3U));
			  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX2, u4Num);
      }              	
    }
    else if(i4GpioNum == PIN_OPWRSB)
    {
      u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX1);
			u4Num = (u4Num & ~(3U));
			u4Num = u4Num | (1U);
			GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX1, u4Num);
    } 
    else if(i4GpioNum == PIN_HDMI_CEC)
    {
      u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX3);
			u4Num = (u4Num & ~(7U));
			u4Num = u4Num | (1U);
			GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX3, u4Num);
    }
    else if(i4GpioNum == PIN_CBUS)
    {
      u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX3);
			u4Num = (u4Num & ~(7U << 12));
			u4Num = u4Num | (1U << 12);
			GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX3, u4Num);
    } else if(i4GpioNum & EGPIO_FLAG){
    	UINT32 asPdwnGpio = 0;
		UINT32 pinmux = egpio_to_pinmux[i4GpioNum - PIN_E_START];
		if(((i4GpioNum > PIN_E_OPWRSB) && (i4GpioNum < PIN_E_VSTB)) ||
			((i4GpioNum >= PIN_E_LED1) && (i4GpioNum < PIN_E_MHL_SENCE))){
			asPdwnGpio = 6;
		} else {
			asPdwnGpio = 3;
		}
		BSP_PinSet(pinmux, asPdwnGpio);
	}
	
    return;
#endif	   
    }
#if(CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8580)
  else if(i4GpioNum <= PIN_HDMI_CEC)
#else
  else if(i4GpioNum <= TOTAL_GPIO_NUM)
#endif    
	{
#if(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560)
		  if(((UINT32)i4GpioNum  >= PIN_VFD_STB) && ((UINT32)i4GpioNum  <= PIN_GPIO1))
#elif(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580)
	  if(((UINT32)i4GpioNum >= PIN_VFD_STB) && ((UINT32)i4GpioNum <= PIN_LCDRD))
#endif  	
	  {
		u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX2);
		u4Num = (u4Num & ~(1U << (i4GpioNum - PIN_VFD_STB)));
		GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX2, u4Num);

		u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX3);
		u4Num = (u4Num | (1U << (i4GpioNum - PIN_VFD_STB)));
		GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX3, u4Num);

	 #if((CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560) \
	        || (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8561) \
	        || (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563))
    	if(((UINT32)i4GpioNum >= PIN_GPIO0) && ((UINT32)i4GpioNum <= PIN_GPIO1))
    	 {
    		u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX4);
    		u4Num = (u4Num & ~(1U << ((UINT32)i4GpioNum  - PIN_VFD_STB)));
    		GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX4, u4Num);
    	 }
     #endif	
	  }
 
  #if(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580)
	 if((i4GpioNum >= PIN_VFD_STB) && (i4GpioNum <= PIN_LCDRD))
	 {
	   u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX4);
		 u4Num = (u4Num & ~(1U << (i4GpioNum - PIN_VFD_STB)));
		 GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX4, u4Num);

	   u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX5);
		 u4Num = (u4Num & ~(1U << (i4GpioNum - PIN_VFD_STB)));
		 GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX5, u4Num);
	 }
  #endif	
	  u4Num = GPIO_PDWNC_READ(PDWNC_PAD_PINMUX1);
	  u4Num = (u4Num | (1U << (i4GpioNum - PIN_VFD_STB)));
	  GPIO_PDWNC_WRITE(PDWNC_PAD_PINMUX1, u4Num);
	}
#if(CONFIG_CHIP_VER_CURR==CONFIG_CHIP_VER_MT8580)
    else if(i4GpioNum <= TOTAL_GPIO_NUM)
    {
        if((i4GpioNum>=PIN_DDC_SCL_RX)&&(i4GpioNum<=PIN_PWR5V_RX))
        {
            if(BSP_PinGet(JTAG_4PIN_SEL)==0)
            {
			     BSP_PinSet(JTAG_4PIN_SEL,1);
		    }
            if(BSP_PinGet(JTAG_SEL)==2)
            {     
                if(BSP_PinGet(JTAG_PDWNC_HDMI_SEL)==3)
                {
                    BSP_PinSet(JTAG_SEL,0);  
				    BSP_PinSet(JTAG_PDWNC_HDMI_SEL,0);  
			     }
		    }
            if(i4GpioNum==PIN_PWR5V_RX)
            {
                u4Temp = GPIO_PDWNC_READ(0X358);
		 	    GPIO_PDWNC_WRITE(0X358,((~(0x11<<20))&u4Temp));
			    GPIO_PDWNC_WRITE(0X358,((0x01<<20)|u4Temp));
            }
		    else if(i4GpioNum==PIN_DDC_SDA_RX)
            {
                u4Temp = GPIO_PDWNC_READ(0X358);
		 	    GPIO_PDWNC_WRITE(0X358,((~(0x11<<16))&u4Temp));
			    GPIO_PDWNC_WRITE(0X358,((0x01<<16)|u4Temp));
            }
		    else if(i4GpioNum==PIN_DDC_SCL_RX)	
            {
                u4Temp = GPIO_PDWNC_READ(0X358);
		 	    GPIO_PDWNC_WRITE(0X358,((~(0x11<<12))&u4Temp));
			    GPIO_PDWNC_WRITE(0X358,((0x01<<12)|u4Temp));
            }
		 
	    }
    }
#endif	
}


/*****************************************
 * Routine: GPIO_Config
 *****************************************/
void GPIO_Config(INT32 i4GpioNum, INT32 i4Output, INT32 i4High)
{
	GPIO_Release(i4GpioNum);
	if (i4Output == INPUT)
	{
		GPIO_InOut_Sel(i4GpioNum, INPUT);
	}
	else
	{
		GPIO_Output(i4GpioNum, i4High);
		GPIO_InOut_Sel(i4GpioNum, OUTPUT);
	}
}

/*****************************************
 * Routine: board_init
 * Description: Early hardware init.
 *****************************************/
int pinmux_init (void)
{
	UINT32 val;
	val = 0;
	
  #if !SUPPORT_JIG
    //GPIO_Config(PIN_VOUTHSYNC, OUTPUT, LOW);
    GPIO_Config(PIN_VOUTVSYNC, OUTPUT, LOW);
  #else
    #if (CFG_UBOOT_CUSTOM_JSN)
    GPIO_Config(USB_PCONT1, OUTPUT, LOW);
    #else
    GPIO_Config(USB_PCONT1, OUTPUT, HIGH);
    #endif
    GPIO_Config(USB_PCONT2, OUTPUT, LOW); 
	GPIO_Config(USB_PCONT3, OUTPUT, HIGH);
    #if (CFG_UBOOT_CUSTOM_JSN && (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580))
    GPIO_Config(WIFI_PCONT, OUTPUT, HIGH);
    #endif

    #if CONFIG_MT8580_AUDIO_JSN//MSB11G BX-Audio
    GPIO_Config(RESET_SICON, OUTPUT, HIGH);
    GPIO_Config(PIN_MDIO,OUTPUT,HIGH);//PCONT3

    GPIO_Config(PIN_GPIO27, OUTPUT, HIGH);// BDP11G-BX Audio: PCONT2

    GPIO_Config(PIN_GPIO8, OUTPUT, HIGH);//WIFI_RESET, output H to release reset signal
    
    GPIO_Config(PIN_GPIO4, INPUT, LOW);//JIG_MD0
    GPIO_Config(PIN_GPIO5, INPUT, LOW);//JIG_MD1
    #endif
    
	GPIO_Config(PIN_FE_RST1, OUTPUT, HIGH);
	
    if (GPIO_Input(START_BIT))
    {
        GPIO_Config(VIDEO_MUTE, OUTPUT, HIGH);
    }
  #endif
  
  #if defined(CONFIG_CUSTOM_KLG) &&(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8563 || CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8553)
	  GPIO_Config(PIN_LED1, OUTPUT, HIGH); 
  #endif

  #if SUPPORT_AMUTE && !CFG_UBOOT_CUSTOM_JSN
    GPIO_Config(AUDIO_MUTE, OUTPUT, LOW);   
  #endif
  
  #if SUPPORT_WP
    GPIO_Config(NAND_WP, OUTPUT, HIGH); 
  #endif

#if CONFIG_DRV_CUSTOM_KLG
    GPIO_Config(PIN_LCDRD, OUTPUT, HIGH);  //For LG BDP model
#endif

  #if SUPPORT_UPG_STATUS
    GPIO_Config(UPG_STATUS_PIN, OUTPUT, LOW);
  #endif
  
  #if SUPPORT_UPG_FLASH_LED
    GPIO_Config(PIN_CON_RED_LED, OUTPUT, LOW);
    GPIO_Config(PIN_CON_GREEN_LED, OUTPUT, LOW);
  #endif
  #if  (CONFIG_CURRENT_MODEL_NAME == M9900)
   //GPIO_Config(PIN_GPIO6, OUTPUT, HIGH);//gk standby led
  #endif
  #if ((CONFIG_JXF_MODEL_NAME == BDP_160) && (!CONFIG_OPPO_3D_OUTPUT_LED_GPIO_CONTROL))
	 GPIO_Config(PIN_GPIO5, OUTPUT, HIGH);//add for pioneer bdp-160
  #endif
  
#if CONFIG_PIN_LED0 != PIN_UNKNOWN
	GPIO_Config(CONFIG_PIN_LED0, OUTPUT, HIGH); /* init Ethernet PHY LED0 pin as output high */
#endif
#if CONFIG_PIN_LED1 != PIN_UNKNOWN
	GPIO_Config(CONFIG_PIN_LED1, OUTPUT, HIGH); /* init Ethernet PHY LED1 pin as output high */
#endif

#if CONFIG_WFD_DONGLE_GPIO //define in gpio_config.h for wfd dongle.

    GPIO_Config(PIN_OPWRSB, INPUT, LOW);
    GPIO_Config(PIN_GPIO0, INPUT, LOW);
    GPIO_Config(PIN_GPIO1, INPUT, LOW);
    GPIO_Config(PIN_GPIO2, INPUT, LOW);
    GPIO_Config(PIN_GPIO3, INPUT, LOW);
    GPIO_Config(PIN_GPIO4, INPUT, LOW);
    GPIO_Config(PIN_GPIO5, INPUT, LOW);
    GPIO_Config(PIN_GPIO6, INPUT, LOW);
    GPIO_Config(PIN_GPIO7, INPUT, LOW);
    GPIO_Config(PIN_GPIO8, INPUT, LOW);
    
    GPIO_Config(PIN_VFD_CLK, INPUT, LOW);
    GPIO_Config(PIN_VFD_DATA, INPUT, LOW);
    GPIO_Config(PIN_VFD_STB, INPUT, LOW);

    GPIO_Config(PIN_LED0, INPUT, LOW);
    GPIO_Config(PIN_LED1, INPUT, LOW);

    GPIO_Config(PIN_HDMI_CEC_RX, INPUT, LOW);
    GPIO_Config(PIN_HDMI_SCL_RX, INPUT, LOW);
    GPIO_Config(PIN_HDMI_SDA_RX, INPUT, LOW);
    GPIO_Config(PIN_MHL_SENCE, INPUT, LOW);
    GPIO_Config(PIN_HDMI_HPD_CBUS_RX, INPUT, LOW);

    GPIO_Config(PIN_SCL, INPUT, LOW);
    GPIO_Config(PIN_SDA, INPUT, LOW);

    GPIO_Config(PIN_SD_DATA0, INPUT, LOW);
    GPIO_Config(PIN_SD_DATA1, INPUT, LOW);
    GPIO_Config(PIN_SD_DATA2, INPUT, LOW);
    GPIO_Config(PIN_SD_DATA3, INPUT, LOW);
    GPIO_Config(PIN_SD_CMD, INPUT, LOW);
    GPIO_Config(PIN_SD_CLK, INPUT, LOW);
    GPIO_Config(PIN_SD_DET, INPUT, LOW);

    GPIO_Config(PIN_GPIO20, INPUT, LOW);
    GPIO_Config(PIN_GPIO21, INPUT, LOW);
    GPIO_Config(PIN_GPIO22, INPUT, LOW);
    GPIO_Config(PIN_GPIO23, INPUT, LOW);
    GPIO_Config(PIN_GPIO24, INPUT, LOW);
    GPIO_Config(PIN_GPIO25, INPUT, LOW);

    GPIO_Config(PIN_AL0, INPUT, LOW);
    GPIO_Config(PIN_AR0, INPUT, LOW);
    GPIO_Config(PIN_AOLRCK, INPUT, LOW);
    GPIO_Config(PIN_AOSDATA0, INPUT, LOW);
    GPIO_Config(PIN_AOSDATA1, INPUT, LOW);
    GPIO_Config(PIN_AOSDATA2, INPUT, LOW);
    GPIO_Config(PIN_AOSDATA3, INPUT, LOW);
    GPIO_Config(PIN_SPDIF, INPUT, LOW);
    GPIO_Config(PIN_SPDIF_IN0, INPUT, LOW);
    GPIO_Config(PIN_SPDIF_IN1, INPUT, LOW);

    GPIO_PDWNC_WRITE(0x0f0, GPIO_PDWNC_READ(0x0f0) | 0x003f1fff);   
    CKGEN_WRITE32(0x11c, CKGEN_READ32(0x11c) | 0x80000008);
    CKGEN_WRITE32(0x148, CKGEN_READ32(0x148) | 0x00208208);
    CKGEN_WRITE32(0x128, CKGEN_READ32(0x128) | 0x82082000);   
    CKGEN_WRITE32(0x12C, CKGEN_READ32(0x12C) | 0x00000820);
    CKGEN_WRITE32(0x14c, CKGEN_READ32(0x14c) | 0x00000208);   
    CKGEN_WRITE32(0x100, CKGEN_READ32(0x100) | 1 << 27);   
    CKGEN_WRITE32(0x104, CKGEN_READ32(0x104) | 0x02002082);   
    CKGEN_WRITE32(0x108, CKGEN_READ32(0x108) | 0x08000808); 

#endif   
#if (CUSTOMER_MODEL_ID == 2)
    GPIO_Config(PIN_LED0, OUTPUT, HIGH); //samsung usb vbus
#endif
    return 0;
}
#if CFG_UPG_LED_BLINK_JSN
//Add for Sony Panel LED blink when upgrade.
void v_set_upg_led(UINT8 bSetLed)
{
    static BOOL bLedStatus = LOW;

    if(0 == bSetLed)
    {
        printf ("do_upg set_upg_led_off\n");
	    //8G LED OUTPUT LOW ON; INPUT HIGH OFF
        GPIO_Config(P_LED, INPUT, HIGH); //Turn Panel LED OFF
        bLedStatus = LOW;
    }
    else if(1 == bSetLed)
    {
        printf ("do_upg set_upg_led_on\n");
        //8G LED OUTPUT LOW ON; INPUT HIGH OFF
    #if CONFIG_CHIP_VER_MT8580 != CONFIG_CHIP_VER_CURR
        GPIO_Config(P_LED, OUTPUT, LOW); //Turn Panel LED ON
    #else
		#if !CONFIG_MT8580_AUDIO_JSN
        GPIO_Config(P_LED, OUTPUT, LOW); //Turn Panel LED ON
        #endif
    #endif

        bLedStatus = HIGH;
    }
    else
    {
    #if !CONFIG_MT8580_AUDIO_JSN
        //printf ("do_upg Reverse LED\n");
        bLedStatus = bLedStatus ? LOW : HIGH;
        if(bLedStatus)
        {
           //8G LED OUTPUT LOW ON; INPUT HIGH OFF
           GPIO_Config(P_LED, OUTPUT, LOW); //Turn Panel LED ON
        }
        else
        {
            //8G LED OUTPUT LOW ON; INPUT HIGH OFF
            GPIO_Config(P_LED, INPUT, HIGH); //Turn Panel LED OFF
        }
	#endif
    }
}
#endif
/*****************************************
 * Routine: v_set_upg_status
 *****************************************/
#if (SUPPORT_UPG_LED_CUSTOMER_T)

#define UPG_PIN_RED     PIN_VIND1//PIN_ETTXD1
#define UPG_PIN_GREEN   PIN_VIND0//PIN_ETCRS

extern void udelay (unsigned long usec);

static void led_ctrl_red(BOOL b_on)
{
    GPIO_Output(UPG_PIN_RED, b_on ? LOW : HIGH);
}

static void led_ctrl_green(BOOL b_on)
{
    GPIO_Output(UPG_PIN_GREEN, b_on ? LOW : HIGH);
}

void v_set_upg_status(UPG_STATUS ustatus, u_long size)
{
    static u_long upg_time = 0;
	static int status = 0;
  
    switch(ustatus)
    {
        case UPG_ERASE:
		case UPG_PROGRAM:
		case UPG_READ:
		case UPG_CHECK:
            upg_time += size;
            if(upg_time >= 1000)
            {
                upg_time = 0;
            
                status = !status;
                led_ctrl_green(status);
            }
            break;
            
		case UPG_START:
	        GPIO_PDWNC_WRITE(0x188, 0x245);
            udelay(1000);
                
            GPIO_Config(UPG_PIN_RED, OUTPUT, HIGH);
            GPIO_Config(UPG_PIN_GREEN, OUTPUT, HIGH);
            
            led_ctrl_red(TRUE);
            led_ctrl_green(TRUE);
			return;

		case UPG_END_OK:
            led_ctrl_red(FALSE);
            led_ctrl_green(TRUE);
			return;

		case UPG_END_FAIL:
            led_ctrl_red(TRUE);
            led_ctrl_green(FALSE);
			return;

		default:
			return;
    }
}

#elif SUPPORT_UPG_FLASH_LED  
static UPG_STATUS upg_status = UPG_NONE;
void v_set_upg_status(UPG_STATUS ustatus, u_long size)
{
  #if (UPG_FLASH_LED_FORM == 0)
    static u_long upg_time = 0;
	static int status = 0;
    u_long time_now = 0;
  #endif
  
    switch(ustatus)
    {
      #if (UPG_FLASH_LED_FORM == 0)
        case UPG_ERASE:
			time_now = (size >= 0x40000) ? (3000 * (size/0x40000)) : (3000 /(0x40000/size));
			break;

		case UPG_PROGRAM:
			time_now = (size >= 0x800) ? (1000 * (size/0x800)) : (1000 /(0x800/size));
			break;

		case UPG_READ:
			time_now = (size >= 0x800) ? (30 * (size/0x800)) : (30 /(0x800/size));
			break;

		case UPG_CHECK:
			time_now = (size >= 0x20000) ? (40000 * (size/0x20000)) : (40000 /(0x20000/size));
			break;
      #endif
	   
		case UPG_START:
			upg_status = UPG_START;
		  #if (UPG_FLASH_LED_FORM == 1)
		    GPIO_Output(PIN_CON_RED_LED, HIGH);
			GPIO_Output(PIN_CON_GREEN_LED, HIGH);
		  #elif (UPG_FLASH_LED_FORM == 0)
			GPIO_Output(PIN_CON_RED_LED, LOW);
			GPIO_Output(PIN_CON_GREEN_LED, LOW);
		  #endif	
			return;

		case UPG_END_OK:
			upg_status = UPG_END_OK;
		  #if (UPG_FLASH_LED_FORM == 1)
		    GPIO_Output(PIN_CON_RED_LED, LOW);
			GPIO_Output(PIN_CON_GREEN_LED, HIGH);
		  #elif (UPG_FLASH_LED_FORM == 0)	
			GPIO_Output(PIN_CON_RED_LED, HIGH);
			GPIO_Output(PIN_CON_GREEN_LED, HIGH);
		  #endif 	
			return;

		case UPG_END_FAIL:
			upg_status = UPG_END_FAIL; 
		  #if (UPG_FLASH_LED_FORM == 1)
		    GPIO_Output(PIN_CON_RED_LED, HIGH);
			GPIO_Output(PIN_CON_GREEN_LED, LOW);
		  #elif (UPG_FLASH_LED_FORM == 0)	
			GPIO_Output(PIN_CON_RED_LED, LOW);
			GPIO_Output(PIN_CON_GREEN_LED, LOW);
		  #endif	
			return;

		default:
			return;
    }

  #if (UPG_FLASH_LED_FORM == 0)	
    upg_time += time_now;

	if(upg_time >= 200000)
	{
	    upg_time = 0;

		status = (status == 0) ? 1 : 0;

	    if(status == 1)
        {
    	    GPIO_Output(PIN_CON_GREEN_LED, HIGH);
        }
    	else
    	{
    		GPIO_Output(PIN_CON_GREEN_LED, LOW);
    	}	
	}
  #endif	
}

void v_show_upg_result(void)
{
  #if (UPG_FLASH_LED_FORM == 0)
    static int flash_count = 0;
	
    if(upg_status != UPG_END_FAIL)
		return;

	switch(flash_count)
	{
		case 1:
			GPIO_Output(PIN_CON_GREEN_LED, HIGH);
			break;

		case 2:
			GPIO_Output(PIN_CON_GREEN_LED, LOW);
			GPIO_Output(PIN_CON_RED_LED, HIGH);
			break;

		case 3:
			GPIO_Output(PIN_CON_RED_LED, LOW);
			break;

		case 9:
			flash_count = 0;
			return;

		default:
			break;
	}

	flash_count ++;
  #endif	
}
#endif

//================================================
// GPIO JIG Mode Private Function
//================================================
#if SUPPORT_JIG
void vEnableJigMode0(void)
{
    GPIO_Config(JIG_MODE0, INPUT, HIGH);        	    
}

void vEnableJigMode1(void)
{
    GPIO_Config(JIG_MODE1, INPUT, HIGH);    
}

BOOL fgGetJigMode0(void) //JIG_MODE0
{
    if(GPIO_Input(JIG_MODE0))   
    {
        return TRUE;
    }
    return FALSE;
}

BOOL fgGetJigMode1(void)  //JIG_MODE1
{
    if(GPIO_Input(JIG_MODE1))   
    {
        return TRUE;
    }
    return FALSE;
}

// for jig mode check
BOOL fg_ne_bl_chk_jig(void)
{
    vEnableJigMode0();
    vEnableJigMode1();
    
    if(fgGetJigMode0()==TRUE && fgGetJigMode1()==FALSE)
    {
        return TRUE;
    }
    return FALSE;
}
#endif

//=========================================================
// check gpio7 and PIN_ETTXER status, if connected, return true.   for pana fe upg request
//=========================================================
#if(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8560)
//=========================================================
// check PIN_VOUTD1 status, if PD, return true.   for pana fe upg request
//=========================================================
BOOL fgCheckFeCtlReg(void)
{
   GPIO_Config(PIN_AOSDATA4, INPUT, HIGH);

   if(GPIO_Input(PIN_AOSDATA4))
     return FALSE;
   else
     return TRUE;
}
#elif (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580)
BOOL fgCheckFeCtlReg(void)
{
    // return TRUE;
   GPIO_Config(PIN_GPIO10, INPUT, HIGH);
   if(GPIO_Input(PIN_GPIO10))
   	{
  printf("Ylf-->PIN_GPIO10 input =1\n");
     return TRUE;
   	}
   else
   	{
  printf("Ylf-->PIN_GPIO10 input =0\n");
     return FALSE;
   	}
}
#else
BOOL fgCheckFeCtlReg(void)
{
    return TRUE;
}
#endif 
//extern void UPG_LOG(int level, const char *fmt, ...);
BOOL fgCheckUSBProtectCircuit(void)
{
//#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580)    
//    return FALSE; //TRUE; alway enable usb power and update fe//set false because two type dvd.bin416/417
//#else 
    return TRUE; //TRUE; alway enable usb power and update fe
//#endif
    
}

//================================================
// Update UPG_STATUS for during upgrade FW.
//================================================
#if SUPPORT_UPG_STATUS
void vSetUpgStatus(BOOL fgVal)
{
	if (!fgVal)
	{
		GPIO_Output(UPG_STATUS_PIN, LOW);  
	}
	else
	{
		GPIO_Output(UPG_STATUS_PIN, HIGH);
	}
}
#endif
 
//================================================
// NAND Write Protect
//================================================
#if SUPPORT_WP
BOOL  NAND_COMMON_WriteProtect(BOOL bMode) 
{
    if ( TRUE == bMode) //write protect
    {
        GPIO_Output(NAND_WP, LOW);    
    }
    else //release write protection
    {
        GPIO_Output(NAND_WP, HIGH);
    }
    return TRUE;
}
#endif

