#include <chip_ver.h>
#include <common.h>
#include <nand.h>
#include <linux/mtd/nand.h>
#include <malloc.h>
#include <asm/arch/x_typedef.h>

#include <asm/errno.h>
#include <asm/io.h>

#include <linux/mtd/NFI3.h>
#include <linux/mtd/NFIECC3.h>
#include <asm/arch/memory.h>
/*this value should be not changed until NAND HW changed, IC Designer will notify it.
the address input into NFI_STRADDR = allocate memory addr + NAND_HW_OFFSET*/
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8581)
#define NAND_HW_OFFSET  0x0
#else
#define NAND_HW_OFFSET  (PHYS_OFFSET + 0x10000000)
#endif

static int _data_buf_index = 0;
BOOL _fgUsingDMA = TRUE;
BOOL _fgDMABurstEn = TRUE;
BOOL _fgAUTO_FMT = TRUE;
BOOL _fgECCSWCorrect = FALSE;
u32 u4SectorSize = 512;
u32 u4SecSpareSize = 16;
u32 u4SectorNum = 1;
u32 page_addr_for_read = 0;


/* Define default oob placement schemes for large and small page devices */
static uint8_t scan_ff_pattern[] = { 0xff, 0xff };

static struct nand_bbt_descr mt85xx_largepage_bbt_descr = {
	.options = 0,
	.offs = 0,
	.len = 2,
	.pattern = scan_ff_pattern
};

static struct nand_bbt_descr mt85xx_smallpage_bbt_descr = {
	.options = 0,
	.offs = 5,
	.len = 1,
	.pattern = scan_ff_pattern
};

static struct nand_ecclayout mt85xx_oobinfo_512 = {
	.eccbytes = 10,
	.eccpos = { 6, 7, 8, 9, 10, 11, 12, 13, 14, 15},
	.oobfree = {{0, 6}}
};

static struct nand_ecclayout mt85xx_oobinfo_2048 = {
	.eccbytes = 40,
	.eccpos = { 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63},
	.oobfree = {{2, 22}}
};
static struct nand_ecclayout mt85xx_oobinfo_4096 = {
	.eccbytes = 78,
	.eccpos = {
	                    50, 51, 52, 53, 54, 55, 56, 57, 58, 59,
	                    60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71,
                             72, 73, 74, 75, 76, 77, 78, 79,
                             80, 81, 82, 83, 84, 85, 86, 87,
                             88, 89, 90, 91, 92, 93, 94, 95,
                             96, 97, 98, 99, 100, 101, 102, 103,
	                    104, 105, 106, 107, 108, 109, 110, 111,
	                    112, 113, 114, 115, 116, 117, 118, 119,
	                    120, 121, 122, 123, 124, 125, 126, 127},
	.oobfree = {
	                    {.offset = 2,
                               .length = 48}}
};

BOOL _fgRand_on = FALSE;
BOOL _fgRand_sel = TRUE;
BOOL _fgRandSeed_by_sec = TRUE;
/******************************************************************************
 * Samsang Randomizer Seed Table
 ******************************************************************************/
#define SAMSANG_NAND_RANDOM_SEED_NUM    (128)
const UINT16 u2Samsang_RandomSeed[SAMSANG_NAND_RANDOM_SEED_NUM] =
{
0x576A, 0x05E8, 0x629D, 0x45A3, 0x649C, 0x4BF0, 0x2342, 0x272E,
0x7358, 0x4FF3, 0x73EC, 0x5F70, 0x7A60, 0x1AD8, 0x3472, 0x3612,
0x224F, 0x0454, 0x030E, 0x70A5, 0x7809, 0x2521, 0x48F4, 0x5A2D,
0x492A, 0x043D, 0x7F61, 0x3969, 0x517A, 0x3B42, 0x769D, 0x0647,
0x7E2A, 0x1383, 0x49D9, 0x07B8, 0x2578, 0x4EEC, 0x4423, 0x352F,
0x5B22, 0x72B9, 0x367B, 0x24B6, 0x7E8E, 0x2318, 0x6BD0, 0x5519,
0x1783, 0x18A7, 0x7B6E, 0x7602, 0x4B7F, 0x3648, 0x2C53, 0x6B99,
0x0C23, 0x67CF, 0x7E0E, 0x4D8C, 0x5079, 0x209D, 0x244A, 0x747B,
0x350B, 0x0E4D, 0x7004, 0x6AC3, 0x7F3E, 0x21F5, 0x7A15, 0x2379,
0x1517, 0x1ABA, 0x4E77, 0x15A1, 0x04FA, 0x2D61, 0x253A, 0x1302,
0x1F63, 0x5AB3, 0x049A, 0x5AE8, 0x1CD7, 0x4A00, 0x30C8, 0x3247,
0x729C, 0x5034, 0x2B0E, 0x57F2, 0x00E4, 0x575B, 0x6192, 0x38F8,
0x2F6A, 0x0C14, 0x45FC, 0x41DF, 0x38DA, 0x7AE1, 0x7322, 0x62DF,
0x5E39, 0x0E64, 0x6D85, 0x5951, 0x5937, 0x6281, 0x33A1, 0x6A32,
0x3A5A, 0x2BAC, 0x743A, 0x5E74, 0x3B2E, 0x7EC7, 0x4FD2, 0x5D28,
0x751F, 0x3EF8, 0x39B1, 0x4E49, 0x746B, 0x6EF6, 0x44BE, 0x6DB7
};

/******************************************************************************
 * Toshiba Randomizer Seed Table
 ******************************************************************************/
#define TOSHIBA_NAND_RANDOM_SEED_NUM    (16)
const UINT32 u4Toshiba_RandomSeed[4][TOSHIBA_NAND_RANDOM_SEED_NUM] =
{
    { 0x0000056C, 0x00000FBC, 0x00001717, 0x00000A52,
      0x000007CF, 0x0000064E, 0x0000098C, 0x00001399,
      0x00001246, 0x00000150, 0x000011F1, 0x00001026,
      0x00001C45, 0x0000012A, 0x00000849, 0x00001D4C
    },
    { 0x0001BC77, 0x0000090C, 0x0001E1AD, 0x0000D564,
      0x0001CB3B, 0x000167A7, 0x0001BD54, 0x00011690,
      0x00014794, 0x00000787, 0x00017BAD, 0x000173AB,
      0x00008B63, 0x00003A3A, 0x0001CB9B, 0x0000AF6E
    },
    {
      0x0005D356, 0x0007F880, 0x0006DB67, 0x0006FBAC,
      0x00037CD1, 0x0000F1D2, 0x0002C2AF, 0x0001D310,
      0x0000F34F, 0x00073450, 0x00046EAA, 0x00079634,
      0x00042924, 0x000435D5, 0x00028350, 0x0000949E
    },
    {
      0x001F645D, 0x003D9E86, 0x007D7EA0, 0x006823DD,
      0x005C91F0, 0x00506BE8, 0x004B5FB7, 0x0027E53B,
      0x00347BC4, 0x003D8927, 0x005403F5, 0x0001B987,
      0x004BF708, 0x001A7BAA, 0x001E8309, 0x0000193A
    }
};

void _NFI_HW_RandomizerConfig(
	 struct mtd_info *mtd
    ,const UINT32 u4PageIdx
    ,const BOOL fgISRandomizerEnc
) {
    UINT32  u4PagesPerBlock =0;
    UINT32  u4SeedIdx =0, u4SeedIdx2 =0;

    u4PagesPerBlock = mtd->erasesize / mtd->writesize;

    // clear samsang seed to avoid Samsang/toshiab change side effect
    // Samsang Randomizer seed registers.
    *NFI_RANDOM_CNFG = 0;

    if (!_fgRand_on) {  // TRUE: Randomizer turn on, FALSE: Randomizer turn off
        return;
    }

    if (_fgRand_sel)   // 1: SAMSANG; 0: TOSHIBA
    {
        // Samsang Randomizer
        *NFI_CNFG |= CNFG_RAN_SEL;
        // Samsang randomizer seed only for each page.
        *NFI_CNFG &= (~CNFG_RAN_SEC_EN);  // 0: Randomizer seed for each page; 1: each sector

        //u4SeedIdx = u4PageIdx % u4PagesPerBlock;
        //pages_per_block general is 32/64/128/256
        u4SeedIdx = u4PageIdx & (u4PagesPerBlock-1);
        if (u4PagesPerBlock > SAMSANG_NAND_RANDOM_SEED_NUM) {
            //u4SeedIdx = u4SeedIdx % SAMSANG_NAND_RANDOM_SEED_NUM;
            // Current SAMSANG_NAND_RANDOM_SEED_NUM == 128
            u4SeedIdx = u4SeedIdx & (SAMSANG_NAND_RANDOM_SEED_NUM-1);
        }

        if (fgISRandomizerEnc)  // Randomizer encoder for program data to Nand.
        {
            //clear Toshiba seed to avoid Samsang/toshiab change side effect
            // TOSHIBA Randomizer seed registers.
            *NFI_TS_RANDOM_ENC_SEED_0_1 = 0;
            *NFI_TS_RANDOM_ENC_SEED_2   = 0;
            *NFI_TS_RANDOM_ENC_SEED_3   = 0;
            *NFI_TS_RANDOM_ENC_SEED_4_5 = 0;
            *NFI_TS_RANDOM_ENC_SEED_6   = 0;
            *NFI_TS_RANDOM_ENC_SEED_7   = 0;

            *NFI_RANDOM_CNFG = RANDOM_ENC_SEED(u2Samsang_RandomSeed[u4SeedIdx]) | RANDOM_ENC_EN;
        }
        else   // Randomizer decoder for read dat from Nand.
        {
            //clear Toshiba seed to avoid Samsang/toshiab change side effect
            // TOSHIBA Randomizer seed registers.
            *NFI_TS_RANDOM_DEC_SEED_0_1 = 0;
            *NFI_TS_RANDOM_DEC_SEED_2   = 0;
            *NFI_TS_RANDOM_DEC_SEED_3   = 0;
            *NFI_TS_RANDOM_DEC_SEED_4_5 = 0;
            *NFI_TS_RANDOM_DEC_SEED_6   = 0;
            *NFI_TS_RANDOM_DEC_SEED_7   = 0;

            *NFI_RANDOM_CNFG = RANDOM_DEC_SEED(u2Samsang_RandomSeed[u4SeedIdx]) | RANDOM_DEC_EN;
        }
    }  // end Samsang randomizer
    else {  // Toshiba randomizer
        *NFI_CNFG &= (~CNFG_RAN_SEL);

        if (_fgRandSeed_by_sec) {  // 0: Randomizer seed for each page; 1: each sector
            if (!_fgUsingDMA) {
                *NFI_CNFG |= CNFG_RAN_SEC_EN;
            }else {
                *NFI_CNFG &= (~CNFG_RAN_SEC_EN);
            }
        }else {
            *NFI_CNFG &= (~CNFG_RAN_SEC_EN);
        }

        //u4SeedIdx = u4PageIdx % TOSHIBA_NAND_RANDOM_SEED_NUM;
        // Current TOSHIBA_NAND_RANDOM_SEED_NUM == 16
        u4SeedIdx  = u4PageIdx & (TOSHIBA_NAND_RANDOM_SEED_NUM-1);
        u4SeedIdx2 = (u4PageIdx+1) & (TOSHIBA_NAND_RANDOM_SEED_NUM-1);

        if (fgISRandomizerEnc)  // Randomizer encoder for program data to Nand.
        {
            *NFI_TS_RANDOM_ENC_SEED_0_1 = ((u4Toshiba_RandomSeed[0][u4SeedIdx] & 0x00001FFF) | ((u4Toshiba_RandomSeed[1][u4SeedIdx] & 0x0001FFFF) <<15));
            *NFI_TS_RANDOM_ENC_SEED_2   =  (u4Toshiba_RandomSeed[2][u4SeedIdx] & 0x0007FFFF);
            *NFI_TS_RANDOM_ENC_SEED_3   =  (u4Toshiba_RandomSeed[3][u4SeedIdx] & 0x007FFFFF);
            *NFI_TS_RANDOM_ENC_SEED_4_5 = ((u4Toshiba_RandomSeed[0][u4SeedIdx2] & 0x00001FFF) | ((u4Toshiba_RandomSeed[1][u4SeedIdx2] & 0x0001FFFF) <<15));
            *NFI_TS_RANDOM_ENC_SEED_6   =  (u4Toshiba_RandomSeed[2][u4SeedIdx2] & 0x0007FFFF);
            *NFI_TS_RANDOM_ENC_SEED_7   =  (u4Toshiba_RandomSeed[3][u4SeedIdx2] & 0x007FFFFF);

            *NFI_RANDOM_CNFG = RANDOM_ENC_EN;
        }
        else   // Randomizer decoder for read dat from Nand.
        {
            *NFI_TS_RANDOM_DEC_SEED_0_1 = ((u4Toshiba_RandomSeed[0][u4SeedIdx] & 0x00001FFF) | ((u4Toshiba_RandomSeed[1][u4SeedIdx] & 0x0001FFFF) <<15));
            *NFI_TS_RANDOM_DEC_SEED_2   =  (u4Toshiba_RandomSeed[2][u4SeedIdx] & 0x0007FFFF);
            *NFI_TS_RANDOM_DEC_SEED_3   =  (u4Toshiba_RandomSeed[3][u4SeedIdx] & 0x007FFFFF);
            *NFI_TS_RANDOM_DEC_SEED_4_5 = ((u4Toshiba_RandomSeed[0][u4SeedIdx2] & 0x00001FFF) | ((u4Toshiba_RandomSeed[1][u4SeedIdx2] & 0x0001FFFF) <<15));
            *NFI_TS_RANDOM_DEC_SEED_6   =  (u4Toshiba_RandomSeed[2][u4SeedIdx2] & 0x0007FFFF);
            *NFI_TS_RANDOM_DEC_SEED_7   =  (u4Toshiba_RandomSeed[3][u4SeedIdx2] & 0x007FFFFF);

            *NFI_RANDOM_CNFG = RANDOM_DEC_EN;
        }
    }  // Toshiba randomizer

    return;
}

static int mt85xx_nand_device_ready(struct mtd_info *mtd)
{
	int result = !( *NFI_STA & STATUS_BUSY);

	return result;
}

UINT8 g_uNandVolSel = 0; /* 0 - 3.3v, 1 - 1.8v */
//should get from efuse.
static void gpio_init(void)
{
	// use defult driving, set pinmux
	NFI_SETBIT(PAD_CFG_0, NFI_SEL);
	// PUPD NFRBN/NFCEN
	NFI_CLRBIT(PAD_CTRL_16, NFI_NFRBN_PD);
	NFI_SETBIT(PAD_CTRL_16, NFI_NFRBN_PU);
	NFI_CLRBIT(PAD_CTRL_14, NFI_NFCEN_PD);
	NFI_SETBIT(PAD_CTRL_14, NFI_NFCEN_PU);

	// set TDSEL
	NFI_CLRBIT(PAD_CTRL_14, NFI_NFCEN_TDSEL_MASK);
	NFI_SETBIT(PAD_CTRL_14, NFI_NFCEN_TDSEL_SET);
	NFI_CLRBIT(PAD_CTRL_15, (NFI_NFWEN_TDSEL_MASK | NFI_NFREN_TDSEL_MASK));
	NFI_SETBIT(PAD_CTRL_15, (NFI_NFWEN_TDSEL_SET | NFI_NFREN_TDSEL_SET));
	NFI_CLRBIT(PAD_CTRL_16, (NFI_NFALE_TDSEL_MASK | NFI_NFRBN_TDSEL_MASK));
	NFI_SETBIT(PAD_CTRL_16, (NFI_NFALE_TDSEL_SET | NFI_NFRBN_TDSEL_SET));
	NFI_CLRBIT(PAD_CTRL_17, NFI_NFCLE_TDSEL_MASK);
	NFI_SETBIT(PAD_CTRL_17, NFI_NFCLE_TDSEL_SET);
	NFI_CLRBIT(PAD_CTRL_18, NFI_NFD0_TDSEL_MASK);
	NFI_SETBIT(PAD_CTRL_18, NFI_NFD0_TDSEL_SET);
	NFI_CLRBIT(PAD_CTRL_19, (NFI_NFD1_TDSEL_MASK | NFI_NFD2_TDSEL_MASK));
	NFI_SETBIT(PAD_CTRL_19, (NFI_NFD1_TDSEL_SET | NFI_NFD2_TDSEL_SET));
	NFI_CLRBIT(PAD_CTRL_20, (NFI_NFD3_TDSEL_MASK | NFI_NFD4_TDSEL_MASK));
	NFI_SETBIT(PAD_CTRL_20, (NFI_NFD3_TDSEL_SET | NFI_NFD4_TDSEL_SET));
	NFI_CLRBIT(PAD_DUTY_7, (NFI_NFD5_TDSEL_MASK | NFI_NFD6_TDSEL_MASK));
	NFI_SETBIT(PAD_DUTY_7, (NFI_NFD5_TDSEL_SET | NFI_NFD6_TDSEL_SET));
	NFI_CLRBIT(PAD_DUTY_8, (NFI_NFD7_TDSEL_MASK | NFI_NFD8_TDSEL_MASK));
	NFI_SETBIT(PAD_DUTY_8, (NFI_NFD7_TDSEL_SET | NFI_NFD8_TDSEL_SET));

	if (0 == g_uNandVolSel)
	{
		// set RDSEL for 3.3v nand
		NFI_CLRBIT(PAD_CTRL_14,  NFI_NFCEN_RDSEL_MASK);
		NFI_SETBIT(PAD_CTRL_14,  NFI_NFCEN_RDSEL_3_3V);
		NFI_CLRBIT(PAD_CTRL_15, (NFI_NFWEN_RDSEL_MASK | NFI_NFREN_RDSEL_MASK));
		NFI_SETBIT(PAD_CTRL_15, (NFI_NFWEN_RDSEL_3_3V | NFI_NFREN_RDSEL_3_3V));
		NFI_CLRBIT(PAD_CTRL_16, (NFI_NFALE_RDSEL_MASK | NFI_NFRBN_RDSEL_MASK));
		NFI_SETBIT(PAD_CTRL_16, (NFI_NFALE_RDSEL_3_3V | NFI_NFRBN_RDSEL_3_3V));
		NFI_CLRBIT(PAD_CTRL_17, NFI_NFCLE_RDSEL_MASK);
		NFI_SETBIT(PAD_CTRL_17, NFI_NFCLE_RDSEL_3_3V);
		NFI_CLRBIT(PAD_CTRL_18, NFI_NFD0_RDSEL_MASK);
		NFI_SETBIT(PAD_CTRL_18, NFI_NFD0_RDSEL_3_3V);
		NFI_CLRBIT(PAD_CTRL_19, (NFI_NFD1_RDSEL_MASK | NFI_NFD2_RDSEL_MASK));
		NFI_SETBIT(PAD_CTRL_19, (NFI_NFD1_RDSEL_3_3V | NFI_NFD2_RDSEL_3_3V));
		NFI_CLRBIT(PAD_CTRL_20, (NFI_NFD3_RDSEL_MASK | NFI_NFD4_RDSEL_MASK));
		NFI_SETBIT(PAD_CTRL_20, (NFI_NFD3_RDSEL_3_3V | NFI_NFD4_RDSEL_3_3V));
		NFI_CLRBIT(PAD_DUTY_7, (NFI_NFD5_RDSEL_MASK | NFI_NFD6_RDSEL_MASK));
		NFI_SETBIT(PAD_DUTY_7, (NFI_NFD5_RDSEL_3_3V | NFI_NFD6_RDSEL_3_3V));
		NFI_CLRBIT(PAD_DUTY_8, (NFI_NFD7_RDSEL_MASK | NFI_NFD8_RDSEL_MASK));
		NFI_SETBIT(PAD_DUTY_8, (NFI_NFD7_RDSEL_3_3V | NFI_NFD8_RDSEL_3_3V));
	}
}

#define NAND_DEFAULT_TIMEOUT     (0x0007FFFF)
#define NAND_ID_NUM    (8)
#define NAND_COMMON_READID_CMD             0x90
#define NFI_WAIT(condition_expression, timeout) \
    do {\
       timeout = NAND_DEFAULT_TIMEOUT;\
       while((condition_expression) && (--timeout) );\
    } while(0)


STATUS_E  NFI_ReadID(UINT8 u1DevIdAddr, UINT8 *p1DevID)
{
    STATUS_E  eRet = S_UNKNOWN_ERR;
    UINT32    u4Timeout = NAND_DEFAULT_TIMEOUT;
    UINT8     u1Idx =0;
    UINT32    u4ID[NAND_ID_NUM>>2];  // DWORD

    // reset the NFI core state machine, data FIFO and flushing FIFO
    *NFI_CON = (CON_NFI_RST | CON_FIFO_FLUSH);
    
    // Disable randomizer encoder and decoder
    *NFI_RANDOM_CNFG = 0; 
    
    // Single Byte read.    
    *NFI_CNFG = CNFG_RAN_SEL;  // Recover to default value
    *NFI_CNFG |= (CNFG_OP_SRD | CNFG_READ_EN);

    // always use 8bits I/O interface to read device id
    *NFI_PAGEFMT &= (~PAGEFMT_16BITS_IO_EN);

    // read id cmd
    *NFI_CMD = NAND_COMMON_READID_CMD;
    // wait til CMD is completely issued
    NFI_WAIT((*NFI_STA & STATUS_CMD), u4Timeout);

    // issue addr
    *NFI_COLADDR = (u1DevIdAddr &0xFF);
    *NFI_ROWADDR = 0;
    *NFI_ADDRNOB = 1;
    // wait til ADDR is completely issued
    NFI_WAIT((*NFI_STA & STATUS_ADDR), u4Timeout);

    // set single read 
    *NFI_CON = CON_NFI_SRD;
    // wait til DATA_READ is completely issued
    NFI_WAIT((*NFI_STA & STATUS_DATAR), u4Timeout);

    for(u1Idx=0; u1Idx<(NAND_ID_NUM>>2); u1Idx++)  // NAND_ID_NUM/4  (DWROD)
    {
        // Check the PIO bit is ready or not.
        NFI_WAIT(!GET_PIO_RDY, u4Timeout);
        u4ID[u1Idx] = *NFI_DATAR;
    }

    for (u1Idx=0; u1Idx<NAND_ID_NUM; u1Idx++)
        p1DevID[u1Idx] = *(UINT8*)((UINT8*)u4ID +u1Idx);

    eRet = S_DONE;
    return eRet;
}

static void NFI_Init(void)
{
	UINT32 clk;
	UINT8     u1DevID[NAND_ID_NUM] = {0};
	UINT8 	  *p1DeviceID = u1DevID;
	
	gpio_init();
	
	clk = *NFI_CLK_ENABLE;
	clk |= (0x1 << 2);
	*NFI_CLK_ENABLE = clk;

	clk = *NFI_CLK_SEL;
	clk &= 0xFFF8FFFF; //clear old setting
	clk |= 0x00040000;
	*NFI_CLK_SEL = clk;
	NFI_ReadID(0x00, p1DeviceID);
	printk("uboot: u1DevID[0] : 0x%x \n", u1DevID[0]);
	if (u1DevID[0] == 0x2C)
		*NFI_ACCCON = 0x10401033;
	else if((u1DevID[0] == 0x98) && (u1DevID[5] != 0x15))//exclude TC58NVF2S0FTA00
		*NFI_ACCCON = 0x10401122;//0x10401022;
	else if(u1DevID[0] == 0xc2)
		*NFI_ACCCON = 0x10401122;//0x10401022;
	else {
			clk &= 0xFFF8FFFF; //clear old setting
			clk |= 0x00050000;
			*NFI_CLK_SEL = clk;
			*NFI_ACCCON = 0x31C08222;
	}
	printk("uboot: after setting :*NFI_ACCCON : 0x%x , *NFI_CLK_SEL:0x%x\n", *NFI_ACCCON, *NFI_CLK_SEL);
    *NFI_CSEL = 0;
    // Reset the state machine and data FIFO, besides flushing FIFO
    *NFI_CON = (CON_NFI_RST | CON_FIFO_FLUSH);
}

static void NFI_Config(struct mtd_info *mtd)
{
	UINT32 u4PageSize  = mtd->writesize;	 // page size in Byte
	UINT32 u4SpareSize = mtd->oobsize;  // spare size in Byte
	UINT32 u4NFI_PageFmt = 0;
	UINT8  u1ECC_CorrectBits =0;
	UINT32 u4TmpValue = 0;
	UINT32 u4ECCBits = 0;

	// Configure NFI Page Format Control Registers. (NFI_PAGEFMT)
	// check if I/O interface is supported on this baseband chip,
	// MT8581 no support 16bits IO interface, So DBYTE_EN fix to 0.
	u4NFI_PageFmt = 0;

	u4NFI_PageFmt |= (PAGEFMT_FDM_NUM(FDM_BYTES) | PAGEFMT_FDM_ECC_NUM(FDM_ECC_BYTES));

	// Sector size = 512Bytes, only support Page size 512B, 2KB, 4KB.
	if (u4PageSize <=512) {
		u4SectorSize = NFI_SECTOR_SIZE_512;
	}else {  // Page 8K can't selet 512, DATA_AREA_PAGE_SIZE = 3: reserved if sel_sec_size is 1,
		u4SectorSize = NFI_SECTOR_SIZE_1K;
	}
	u4SectorNum = (UINT8)(u4PageSize / u4SectorSize);

	u4SpareSize = u4SpareSize / u4SectorNum;  // spare size per sector

	if (NFI_SECTOR_SIZE_512 == u4SectorSize)
	{
		u4NFI_PageFmt |= PAGEFMT_SECTOR_SIZE;  // 1: sector size 512B, 0: sector size 1024B(1KB).

		switch (u4PageSize) {
		case 512:  u4NFI_PageFmt |= PAGEFMT_512_2K; break;
		case 2048: u4NFI_PageFmt |= PAGEFMT_2K_4K;	break;
		case 4096: u4NFI_PageFmt |= PAGEFMT_4K_8K;	break;
		default:   u4NFI_PageFmt |= PAGEFMT_RESV_16K;
		}

		if ((u4SpareSize >=16) && (u4SpareSize <26)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_16_32;
			u4SecSpareSize = 16;
		}else if (u4SpareSize == 26) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_26_52;
			u4SecSpareSize = 26;
		}else if (u4SpareSize == 27) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_27_54;
			u4SecSpareSize = 27;
		}else if ((u4SpareSize >=28) && (u4SpareSize <32)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_28_56;
			u4SecSpareSize = 28;
		}else if ((u4SpareSize >=32) && (u4SpareSize <36)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_32_64;
			u4SecSpareSize = 32;
		}else if ((u4SpareSize >=36) && (u4SpareSize <40)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_36_72;
			u4SecSpareSize = 40;
		}else if ((u4SpareSize >=40) && (u4SpareSize <44)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_40_80;
			u4SecSpareSize = 44;
		}else if ((u4SpareSize >=44) && (u4SpareSize <48)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_44_88;
			u4SecSpareSize = 44;
		}else if (u4SpareSize == 48) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_48_96;
			u4SecSpareSize = 48;
		}else if (u4SpareSize == 49) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_49_98;
			u4SecSpareSize = 49;
		}else if (u4SpareSize == 50) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_50_100;
			u4SecSpareSize = 50;
		}else if (u4SpareSize == 51) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_51_102;
			u4SecSpareSize = 51;
		}else if ((u4SpareSize >=52) && (u4SpareSize <62)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_52_104;
			u4SecSpareSize = 52;
		}else if (u4SpareSize == 62) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_62_124;
			u4SecSpareSize = 62;
		}else if (u4SpareSize == 63) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_63_126;
			u4SecSpareSize = 63;
		}else if (u4SpareSize >= 64) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_64_128;
			u4SecSpareSize = 64;
		}else { // < 16
		}
	}
	else // sector size is 1024 Bytes
	{
		switch (u4PageSize) {
		//case 1024:  u4NFI_PageFmt |= PAGEFMT_512_2K;	 break; // The virtual Nand set for debug
		case 2048:	u4NFI_PageFmt |= PAGEFMT_512_2K;   break;
		case 4096:	u4NFI_PageFmt |= PAGEFMT_2K_4K;    break;
		case 8192:	u4NFI_PageFmt |= PAGEFMT_4K_8K;    break;
		case 16384: u4NFI_PageFmt |= PAGEFMT_RESV_16K; break;
		default:;
		}

		if ((u4SpareSize >=32) && (u4SpareSize <52)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_16_32;
			u4SecSpareSize = 32;
		}else if ((u4SpareSize >=52) && (u4SpareSize <54)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_26_52;
			u4SecSpareSize = 52;
		}else if ((u4SpareSize >=54) && (u4SpareSize <56)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_27_54;
			u4SecSpareSize = 54;
		}else if ((u4SpareSize >=56) && (u4SpareSize <64)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_28_56;
			u4SecSpareSize = 56;
		}else if ((u4SpareSize >=64) && (u4SpareSize <72)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_32_64;
			u4SecSpareSize = 64;
		}else if ((u4SpareSize >=72) && (u4SpareSize <80)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_36_72;
			u4SecSpareSize = 72;
		}else if ((u4SpareSize >=80) && (u4SpareSize <88)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_40_80;
			u4SecSpareSize = 80;
		}else if ((u4SpareSize >=88) && (u4SpareSize <96)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_44_88;
			u4SecSpareSize = 88;
		}else if ((u4SpareSize >=96) && (u4SpareSize <98)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_48_96;
			u4SecSpareSize = 96;
		}else if ((u4SpareSize >=98) && (u4SpareSize <100)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_49_98;
			u4SecSpareSize = 98;
		}else if ((u4SpareSize >=100) && (u4SpareSize <102)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_50_100;
			u4SecSpareSize = 100;
		}else if ((u4SpareSize >=102) && (u4SpareSize <104)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_51_102;
			u4SecSpareSize = 102;
		}else if ((u4SpareSize >=104) && (u4SpareSize <124)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_52_104;
			u4SecSpareSize = 104;
		}else if ((u4SpareSize >=124) && (u4SpareSize <126)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_62_124;
			u4SecSpareSize = 124;
		}else if ((u4SpareSize >=126) && (u4SpareSize <128)) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_63_126;
			u4SecSpareSize = 126;
		}else if (u4SpareSize >= 128) {
			u4NFI_PageFmt |= PAGEFMT_SPARE_64_128;
			u4SecSpareSize = 128;
		}else { // < 32
		}
	}// end sector 1024

	// set NFI_PAGEFMT
	*NFI_PAGEFMT = u4NFI_PageFmt;

	// Configure NFIECC registers.
	//ECC Correct bis count = (Sector spare - FDM_ECC)*8 / Parity .  (sector space bits number / parity factor)
	u1ECC_CorrectBits = (UINT8)(((u4SecSpareSize-FDM_ECC_BYTES)<<3)/ECC_PARITY_NUM);

	if ((u1ECC_CorrectBits >=4) && (u1ECC_CorrectBits <=24)) {
		u4ECCBits = (ECC_LEVEL_E)((u1ECC_CorrectBits >>1) -2); // x/2 -2
		u1ECC_CorrectBits = (u1ECC_CorrectBits & 0xFE);
	}else if ((u1ECC_CorrectBits >24) && (u1ECC_CorrectBits <=60)) {
		u4ECCBits = (ECC_LEVEL_E)((u1ECC_CorrectBits >>2) +4); // x/4 +4
		u1ECC_CorrectBits = (u1ECC_CorrectBits & 0xFC);
	}else if (u1ECC_CorrectBits >60) {
		u4ECCBits = ECC_60_BITS;
		u1ECC_CorrectBits = 60;
	}else {  // (u1ECC_CorrectBits <4)
	}

	// set NFIECC_ENCCNFG
	u4TmpValue = u4SectorSize + FDM_ECC_BYTES;
	*NFIECC_ENCCNFG = (ENC_TNUM(u4ECCBits) | ENC_MODE_NFI | ENC_MS(u4TmpValue));

	// set NFIECC_DECCNFG. // DEC_CON = 2 is ECC_DEC_LOCATE
	u4TmpValue = ((u4SectorSize + FDM_ECC_BYTES) <<3) + (u1ECC_CorrectBits*ECC_PARITY_NUM);
	if(_fgECCSWCorrect)
		*NFIECC_DECCNFG = (DEC_TNUM(u4ECCBits) | DEC_MODE_NFI | DEC_CON(ECC_DEC_LOCATE) | DEC_CS(u4TmpValue) | DEC_EMPTY_EN);
	else //DEC_CON = 3 is ECC_DEC_CORRECT
		*NFIECC_DECCNFG = (DEC_TNUM(u4ECCBits) | DEC_MODE_NFI | DEC_CON(ECC_DEC_CORRECT) | DEC_CS(u4TmpValue) | DEC_EMPTY_EN);
}

//------------------------------------------------------------------------------
// Reset Device Callback Function
//------------------------------------------------------------------------------
STATUS_E  NAND_COMMON_Reset(
        const struct mtd_info *mtd
        ,const u32  c_timeout
)
{
    u32    timeout = c_timeout;
	STATUS_E  ret=S_UNKNOWN_ERR;
	#ifdef _XOS_ISR_ENABLE_
    s32    i4Val;
    EV_GRP_EVENT_T rRecvEvt;
	#endif //_XOS_ISR_ENABLE_

	// reset the NFI core state machine, data FIFO and flushing FIFO
	*NFI_CON = (CON_NFI_RST | CON_FIFO_FLUSH);

	*NFI_CNFG = CNFG_RAN_SEL;  // Recover to default value
	*NFI_CNFG |= CNFG_OP_RESET;

	// enable interrupt
	*NFI_INTR_EN = INTR_RST_DONE_EN;

	// reset cmd
	*NFI_CMD = NAND_COMMON_RESET_CMD;

	*NFI_CNRNB = CNRNB_STROBE_TIMEOUT;

	// wait til CMD is completely issued
	while ((*NFI_STA & STATUS_CMD) != 0)
    {
        timeout--;
    }
	if( 0 == timeout ) {
        ret = S_TIMEOUT;
        goto end;
    }

	#ifdef _XOS_ISR_ENABLE_
    if (_bUsingISR)
    {
        i4Val = x_ev_group_wait_event_timeout(_hNFIEvent,
                                          RESET_DONE,
                                          &rRecvEvt,
                                          X_EV_OP_OR_CONSUME,
                                          timeout);
        ASSERT( (i4Val == OSR_OK) || (OSR_TIMEOUT == i4Val));
        if (OSR_TIMEOUT == i4Val)
        {
            ret = S_TIMEOUT;
            goto end;
        }
    }
    else
	#endif //_XOS_ISR_ENABLE_
	#ifdef LINUX_ISR_ENABLE
    if (_fgUsingISR)
    {
            wait_for_completion((struct completion*)(&mt85xx->reset_completion));
    }
    else
	#endif
	{
		timeout = MTD_NAND_DEFAULT_TIMEOUT;
		while ((*NFI_INTR_STA & INTR_RST_DONE) == 0)
	    {
	        timeout--;
	    }
		if( 0 == timeout ) {
            ret = S_TIMEOUT;
            goto end;
        }
	}

end:
	// disable interrupt
    *NFI_INTR_EN = 0;

	return ret;
}

#define SUPPORT_MICRON_29F2G08ABAEA 0
#if SUPPORT_MICRON_29F2G08ABAEA
STATUS_E  NAND_COMMON_SetFeature(
        const struct mtd_info *mtd
        ,const u8  u1FeatuerAddr
        ,u32 u4FeatureValue  // P1 is [7:0], P2 is [15:8], P3 is [23:16], P4 is [31:24]
)
{
	STATUS_E eRet = S_UNKNOWN_ERR;
    UINT32   u4Timeout = MTD_NAND_DEFAULT_TIMEOUT;
    UINT8    u1Value =0;
    UINT8    i =0;

    // reset the NFI core state machine, data FIFO and flushing FIFO
    *NFI_CON = (CON_NFI_RST | CON_FIFO_FLUSH);

    // Disable randomizer encoder and decoder
    *NFI_RANDOM_CNFG = 0;

    // Single Byte read.
    *NFI_CNFG = CNFG_RAN_SEL;  // Recover to default value
    //customer process and read disable(write)
    *NFI_CNFG |= (CNFG_OP_CUST | CNFG_BYTE_RW);

    // Issue Get Feature cmd
    *NFI_CMD = NAND_COMMON_SET_FEATURES_ACMD;
    // wait til CMD is completely issued
    NFI_Wait((*NFI_STA & STATUS_CMD), u4Timeout);

    // issue addr
    *NFI_COLADDR = (u1FeatuerAddr & 0xFF);
    *NFI_ROWADDR = 0;
    *NFI_ADDRNOB = 1;
    // wait til ADDR is completely issued
    u4Timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_STA & STATUS_ADDR), u4Timeout);

    // set one sector and burst write
    *NFI_CON = CON_NFI_SEC_NUM(1) | CON_NFI_BWR;
    // Customer operation mode ,this signal triggers the data trasfer for read or write
    *NFI_STRDATA = STR_DATA;

    if (0 == (*NFI_NAND_TYPE_CNFG & NAND_INFT_TYPE_MASK))  // Async mode
    {
        for (i=0; i<4; i++)
        {
            // Check the PIO bit is ready or not.
            u4Timeout = MTD_NAND_DEFAULT_TIMEOUT;
            NFI_Wait(!GET_PIO_RDY, u4Timeout);
            if( 0 == u4Timeout) {
                eRet = S_TIMEOUT;
                goto ERR_END;
            }

            // P1 is [7:0], P2 is [15:8], P3 is [23:16], P4 is [31:24]
            u1Value = (u4FeatureValue >>(i<<3)) & 0xFF; // i*8
            // single write
            *NFI_DATAW = u1Value;
        }
    }else {    // Sync mode or Toggle mode
        for (i=0; i<8; i++)
        {
            // Check the PIO bit is ready or not.
            u4Timeout = MTD_NAND_DEFAULT_TIMEOUT;
            NFI_Wait(!GET_PIO_RDY, u4Timeout);
            if( 0 == u4Timeout) {
                eRet = S_TIMEOUT;
                goto ERR_END;
            }

            if (0 == (i&0x1)) {
                // P1 is [7:0], P2 is [15:8], P3 is [23:16], P4 is [31:24]
                u1Value = (u4FeatureValue <<(i<<2)) & 0xFF; // i/2*8
            }

            // single write
            *NFI_DATAW = u1Value;
        }
    }
    eRet = S_DONE;

ERR_END:

    return eRet;
}


STATUS_E  NAND_COMMON_GetFeature(
        const struct mtd_info *mtd
        ,const u8  u1FeatuerAddr
        ,u32 *p4FeatureValue  // P1 is [7:0], P2 is [15:8], P3 is [23:16], P4 is [31:24]
)
{
	STATUS_E eRet = S_UNKNOWN_ERR;
    UINT32   u4Timeout = MTD_NAND_DEFAULT_TIMEOUT;
    UINT8    u1Value =0;
    UINT8    i =0;

    // reset the NFI core state machine, data FIFO and flushing FIFO
    *NFI_CON = (CON_NFI_RST | CON_FIFO_FLUSH);

    // Disable randomizer encoder and decoder
    *NFI_RANDOM_CNFG = 0;

    // Single Byte read.
    *NFI_CNFG = CNFG_RAN_SEL;  // Recover to default value
    //customer process and Read enable
    *NFI_CNFG |= (CNFG_OP_CUST | CNFG_READ_EN | CNFG_BYTE_RW);

    // Issue Get Feature cmd
    *NFI_CMD = NAND_COMMON_GET_FEATURES_ACMD;
    // wait til CMD is completely issued
    NFI_Wait((*NFI_STA & STATUS_CMD), u4Timeout);

    // issue addr
    *NFI_COLADDR = (u1FeatuerAddr &0xFF);
    *NFI_ROWADDR = 0;
    *NFI_ADDRNOB = 1;
    // wait til ADDR is completely issued
    u4Timeout = MTD_NAND_DEFAULT_TIMEOUT
    NFI_Wait((*NFI_STA & STATUS_ADDR), u4Timeout);

    // set single read 4bytes
    *NFI_CON = (CON_NFI_SRD | CON_NOB_DWORD);
    // Customer operation mode ,this signal triggers the data trasfer for read or write
    *NFI_STRDATA = STR_DATA;
    // wait til DATA_READ is completely issued
    u4Timeout = MTD_NAND_DEFAULT_TIMEOUT
    NFI_Wait((*NFI_STA & STATUS_DATAR), u4Timeout);

    *p4FeatureValue = 0;
    if (0 == (*NFI_NAND_TYPE_CNFG & NAND_INFT_TYPE_MASK))  // Async mode
    {
        for (i=0; i<4; i++)
        {
            // Check the PIO bit is ready or not.
            u4Timeout = MTD_NAND_DEFAULT_TIMEOUT
            NFI_Wait(!GET_PIO_RDY, u4Timeout);
            if( 0 == u4Timeout) {
                eRet = S_TIMEOUT;
                goto ERR_END;
            }

            // single read doesn't need to polling FIFO
            u1Value = *NFI_DATAR;

            // P1 is [7:0], P2 is [15:8], P3 is [23:16], P4 is [31:24]
            *p4FeatureValue |= ((UINT32)u1Value <<(i<<3));  // i*8
            printk("P%d:%#02x\n", i, u1Value);
        }
    }else {    // Sync mode or Toggle mode
        for (i=0; i<8; i++)
        {
            // Check the PIO bit is ready or not.
            u4Timeout = MTD_NAND_DEFAULT_TIMEOUT
            NFI_Wait(!GET_PIO_RDY, u4Timeout);
            if( 0 == u4Timeout) {
                eRet = S_TIMEOUT;
                goto ERR_END;
            }

            // single read doesn't need to polling FIFO
            u1Value = *NFI_DATAR;

            if (0 == (i&0x1)) {
                // P1 is [7:0], P2 is [15:8], P3 is [23:16], P4 is [31:24]
                *p4FeatureValue |= ((UINT32)u1Value <<(i<<2));  // i/2*8
                printk("P%d:%#02x\n", i, u1Value);
            }
        }
    }

    eRet = S_DONE;

ERR_END:

    return eRet;
}
#endif

static void  NAND_COMMON_ReadID(struct mtd_info *mtd, unsigned command)
{
    register struct nand_chip *this = mtd->priv;
	u8* p1Data;
	int i;
	int IDNum = 4;
	u32 timeout = MTD_NAND_DEFAULT_TIMEOUT;

	#if CONFIG_READID6BYTE
	IDNum = 6;
	#endif

    // reset the NFI core state machine, data FIFO and flushing FIFO
    *NFI_CON = (CON_NFI_RST | CON_FIFO_FLUSH);

    // Disable randomizer encoder and decoder
    *NFI_RANDOM_CNFG = 0;

    // Single Byte read.
    *NFI_CNFG = CNFG_RAN_SEL;  // Recover to default value
    *NFI_CNFG |= (CNFG_OP_SRD | CNFG_READ_EN | CNFG_BYTE_RW);

    // always use 8bits I/O interface to read device id
    *NFI_PAGEFMT &= (~PAGEFMT_16BITS_IO_EN);

    // read id cmd
    *NFI_CMD = NAND_COMMON_READID_CMD;
    // wait til CMD is completely issued
    timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_STA & STATUS_CMD), timeout);

    // issue addr
    *NFI_COLADDR = 0;
    *NFI_ROWADDR = 0;
    *NFI_ADDRNOB = 1;
    // wait til ADDR is completely issued
    timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_STA & STATUS_ADDR), timeout);

    // set single read
    *NFI_CON = CON_NFI_SRD;

	p1Data = (u8*)this->buffers->databuf;
    for (i = 0; i < IDNum; i++)
    {
    	timeout = MTD_NAND_DEFAULT_TIMEOUT;
        NFI_Wait(!GET_PIO_RDY, timeout);
		p1Data[i] = *NFI_DATAR;
		//printk("[nand] id[%d]: 0x%x\n",i,p1Data[i]);
    }
}

//------------------------------------------------------------------------------
// ECC Error Software Correct
//------------------------------------------------------------------------------
STATUS_E NAND_COMMON_ECCErrCorrect(
        const u32  c_timeout
        ,const u32 u4SectIdx
        ,const u32 *p_data32 /* MUST be 32bits alignment addr */
)
{
	STATUS_E eRet = S_UNKNOWN_ERR;
	UINT32	 u4Timeout = c_timeout;
	UINT8	 u1ErrNum =0;
	UINT32	 u4TmpValue =0;
	UINT32	 i =0;
	UINT16	 u2ErrLoc =0;
	UINT16	 u2ErrByteLoc =0;
	UINT8	 u1ErrBitLoc =0;
	UINT32	 u4ErrDRAMLoc =0;

	u4TmpValue = (1 << u4SectIdx);
	NFI_Wait(!(*NFIECC_DECDONE & u4TmpValue), u4Timeout); // wait for current sector decode done
	if( 0 == u4Timeout ) {
		eRet = S_TIMEOUT;
		goto ERR_END;
	}

	u1ErrNum = GET_DECENUM_SEC(u4SectIdx); // error number
	if (0x3F == u1ErrNum)
	{
		eRet = S_ECC_UNCORRECT_ERR;
		goto ERR_END;
	}
	else if (0x0 == u1ErrNum)
	{
		eRet = S_DONE;
		goto ERR_END;
	}
	else if (u1ErrNum & 0x3F)
	{
		for (i=0; i<u1ErrNum; i++)
		{
			u2ErrLoc = GET_DECEL(i);
			u2ErrByteLoc = u2ErrLoc >>3;  // x/8
			u1ErrBitLoc = (UINT8)(u2ErrLoc & 0x07);

			{
				// Auto_FMT is disbale, p_data32 and p_spare32 must is continuous.
				// Data sequence is 1st sector(Sector main data + FDM + Parity data) + 2nd sector ...
				u4ErrDRAMLoc = (UINT32)p_data32 + u4SectIdx*(u4SectorSize + u4SecSpareSize) + u2ErrByteLoc;
				*(UINT8*)u4ErrDRAMLoc ^= (0x01 <<u1ErrBitLoc);
			}
		}// end for (i=0; i<u1ErrNum; i++)

		eRet = S_ECC_CORRECTABLE_ERR;
	}// end   else if (u1ErrNum & 0x3F)

ERR_END:

	  return eRet;
}

u32 _u4ErrBits=0; //record err bit number
//------------------------------------------------------------------------------
// ECC Error Detect
//------------------------------------------------------------------------------
STATUS_E NAND_COMMON_ECCErrDetect(void)
{
    STATUS_E eRet = S_UNKNOWN_ERR;
    UINT32   u4Timeout = MTD_NAND_DEFAULT_TIMEOUT;
    UINT8    u1ErrNum =0;
    UINT32   u4TmpValue =0;
    UINT32   i =0;

    for (i=0; i< u4SectorNum; i++) {
        u4TmpValue |= (1 << i);
    }
    NFI_Wait((*NFIECC_DECDONE != u4TmpValue), u4Timeout); // wait for all sector decode done
    if( 0 == u4Timeout ) {
        eRet = S_TIMEOUT;
        goto ERR_END;
    }

    if (!(*NFIECC_DECFER))
    {
        eRet = S_DONE;
        goto ERR_END;
    }
    else // ECC error detect
    {
        for (i=0; i<u4SectorNum; i++)
        {
            u1ErrNum = GET_DECENUM_SEC(i); // error number
            if (0x3F == u1ErrNum)
            {
                eRet = S_ECC_UNCORRECT_ERR;
                goto ERR_END;
            }
            else if (u1ErrNum & 0x3F)
            {
                eRet = S_ECC_CORRECTABLE_ERR;
            }
			else //no error
			{
				eRet = S_DONE;
			}
        }
    }

ERR_END:

      return eRet;
}


STATUS_E _NFI_HW_ECCErrSWCorrect(
    const UINT8 u1SectIdx
    ,UINT8 *p_data32
    ,UINT8 *p_spare32
) {
    STATUS_E eRet = S_UNKNOWN_ERR;
    UINT32   u4Timeout = MTD_NAND_DEFAULT_TIMEOUT;
    UINT8    u1ErrNum =0;
    UINT32   u4TmpValue =0;
    UINT32   i =0;
    UINT16   u2ErrLoc =0;
    UINT16   u2ErrByteLoc =0;
    UINT8    u1ErrBitLoc =0;
    UINT32   u4ErrDRAMLoc =0;

    u4TmpValue = (1 << u1SectIdx);
    NFI_Wait(!(*NFIECC_DECDONE & u4TmpValue), u4Timeout); // wait for current sector decode done
    if( 0 == u4Timeout ) {
        eRet = S_TIMEOUT;
        goto ERR_END;
    }

    u1ErrNum = GET_DECENUM_SEC(u1SectIdx); // error number
    if (0x3F == u1ErrNum)
    {
        eRet = S_ECC_UNCORRECT_ERR;
        goto ERR_END;
    }
    else if (0x0 == u1ErrNum)
    {
        eRet = S_DONE;
        goto ERR_END;
    }
    else if (u1ErrNum & 0x3F)
    {
        for (i=0; i<u1ErrNum; i++)
        {
            u2ErrLoc = GET_DECEL(i);
            u2ErrByteLoc = u2ErrLoc >>3;  // x/8
            u1ErrBitLoc = (UINT8)(u2ErrLoc & 0x07);

            if (_fgAUTO_FMT) // Normal use,
            {
                if (u2ErrByteLoc < u4SectorSize)
                {
                    u4ErrDRAMLoc = (UINT32)p_data32 + u1SectIdx*u4SectorSize+ u2ErrByteLoc;
                    *(UINT8*)u4ErrDRAMLoc ^= (0x01 <<u1ErrBitLoc);
                }
                else if (u2ErrByteLoc < (UINT16)(u4SectorSize+FDM_ECC_BYTES))
                {
                    u2ErrByteLoc -= u4SectorSize;
                    u4ErrDRAMLoc = (UINT32)p_spare32 + u1SectIdx*FDM_ECC_BYTES + u2ErrByteLoc;
                    *(UINT8*)u4ErrDRAMLoc ^= (0x01 <<u1ErrBitLoc);
                }
                else
                {
                    // Error in parity area.
                }
            }
            else  // Auto_FMT is disable. for CopyBack data correct.
            {
                // Auto_FMT is disbale, p_data32 and p_spare32 must is continuous.
                // Data sequence is 1st sector(Sector main data + FDM + Parity data) + 2nd sector ...
                u4ErrDRAMLoc = (UINT32)p_data32 + u1SectIdx*(u4SectorSize+u4SecSpareSize) + u2ErrByteLoc;
                *(UINT8*)u4ErrDRAMLoc ^= (0x01 <<u1ErrBitLoc);
            }
        }// end for (i=0; i<u1ErrNum; i++)

        eRet = S_ECC_CORRECTABLE_ERR;
    }// end   else if (u1ErrNum & 0x3F)

ERR_END:

      return eRet;
}

//------------------------------------------------------------------------------
// Read From NFI FIFO
//------------------------------------------------------------------------------
STATUS_E  NAND_COMMON_FIFO_Read(
        struct mtd_info *mtd
        ,const u32  c_timeout
        ,const BOOL bUsingDMA
        ,u8*p_data32 /* MUST be 32bits alignment addr */
        ,u32 page_addr
) {
	register struct nand_chip *mt85xx = mtd->priv;
    u32  timeout = c_timeout;
    u32  i, j;
	u32   u4FDMData_L =0xFFFFFFFF, u4FDMData_M =0xFFFFFFFF;
	u8   *p1TmpPage, *p1TmpSpare;
	STATUS_E eRet = S_DONE;
	u32 u4PageDataLength;
#ifdef _XOS_ISR_ENABLE_
    EV_GRP_EVENT_T rRecvEvt;
    s32 i4Val;
#endif //_XOS_ISR_ENABLE_

	p1TmpPage  = p_data32;
    p1TmpSpare = mt85xx->oob_poi;

    if(bUsingDMA)
    {
        // read page data with DMA
        timeout = c_timeout;
        // wait for DMA transmission complete
#ifdef _XOS_ISR_ENABLE_
        if (_bUsingISR)
        {
            i4Val = x_ev_group_wait_event_timeout(_hNFIEvent,
                                              AHB_DONE,
                                              &rRecvEvt,
                                              X_EV_OP_OR_CONSUME,
                                              timeout);

            if(i4Val == OSR_TIMEOUT) {
                return S_TIMEOUT;
            }
        }
        else
#endif //_XOS_ISR_ENABLE_
#ifdef LINUX_ISR_ENABLE
        if (_fgUsingISR)
        {
        	    wait_for_completion((struct completion*)(&mt85xx->dma_completion));
        }
        else
#endif
		{    // Polling Interrupt status.
             // Trigger DMA action
            NFI_Wait(!(*NFI_INTR_STA & INTR_AHB_DONE), timeout);
            *NFI_INTR_EN &= (~INTR_AHB_DONE_EN); // disable INT first
            if( 0 == timeout) {
                return S_TIMEOUT;
            }
        }
    }
    else
    {  // PIO mode. read page data
        if (_fgAUTO_FMT)  // FDM from FDM registesr.
        {
            for (i=0; i < u4SectorNum; i++)
            {
                // Toshiba randomizer seed config for each sector.
                if (_fgRandSeed_by_sec && _fgRand_on) {
                    _NFI_HW_RandomizerConfig(mtd, (page_addr+i), FALSE);  // randomizer decoder
                }

                for(j=0; j<u4SectorSize; j+=4)
                {
                    // wait for data ready
                    // when RD_EMPTY_MASK flag is poll-down, it means data is ready in FIFO at least 4 bytes.
                    //NFI_WAIT((*NFI_FIFOSTA & FIFO_RD_EMPTY), u4Timeout);

                    // Check the PIO bit is ready or not.
                    NFI_Wait(!GET_PIO_RDY, timeout);
                    if( 0 == timeout) {
                        return S_TIMEOUT;
                    }

                    *(u32*)p1TmpPage = *NFI_DATAR;
                    p1TmpPage += 4;
                }  // end for(j=0; j<pSWCfg->u4SelSectorSize;

               // Check current Sector is done, FDM data input to FDM registers.
                NFI_Wait((GET_SEC_CNTR!=(i+1)), timeout);
                if( 0 == timeout) {
                    return S_TIMEOUT;
                }

                u4FDMData_L = *NFI_FDM_L(i);
                if (FDM_BYTES <=4) {
                    for(j=0; j<FDM_BYTES; j++) {
                        *p1TmpSpare = (UINT8)((u4FDMData_L>>(j<<3)) &0xFF);
                        p1TmpSpare++;
                    }
                }else {
                    for(j=0; j<4; j++) {
                        *p1TmpSpare = (UINT8)((u4FDMData_L>>(j<<3)) &0xFF);
                        p1TmpSpare++;
                    }

                    u4FDMData_M = *NFI_FDM_M(i);
                    for(j=0; j<(FDM_BYTES-4); j++) {
                        *p1TmpSpare = (UINT8)((u4FDMData_M>>(j<<3)) &0xFF);
                        p1TmpSpare++;
                    }
                }

                if (_fgECCSWCorrect) {
                    eRet = _NFI_HW_ECCErrSWCorrect(i, p_data32, mt85xx->oob_poi);
                    if (S_ECC_UNCORRECT_ERR == eRet)
                    {
                        return eRet;
                    }
                }
            }// end for (i=0; i<pDevCfg->m_SectorNum
        }// end if (pSWCfg->fgAuto_FMT)  // FDM from FDM registesr.
        else {  // PIO mode. read page data, else if (pSWCfg->fgAuto_FMT)
            // All data from PIO.
            // Auto_FMT is disbale, p_data32 and p_spare32 must is continuous.
            // Data sequence is 1st sector(Sector main data + FDM + Parity data) + 2nd sector ...
            u4PageDataLength = (0xFFFFFFFC & (u4SectorNum*(u4SectorSize+u4SecSpareSize)));
            for (i=0; i<u4PageDataLength; i+=4)
            {
                // Check the PIO bit is ready or not.
                timeout = c_timeout;
                NFI_Wait(!GET_PIO_RDY, timeout);
                if( 0 == timeout) {
                    return S_TIMEOUT;
                }

                *(UINT32*)p1TmpPage = *NFI_DATAR;
                p1TmpPage += 4;
            }  // end for (i=0; i<u4PageDataLength; i+=4)

            u4PageDataLength = (0x03 & (u4SectorNum*(u4SectorSize+u4SecSpareSize)));
            if (0 != u4PageDataLength)
            {
                *NFI_CNFG |= CNFG_BYTE_RW;
                for(j=0; j<u4PageDataLength; j++)
                {
                    // Check the PIO bit is ready or not.
                    timeout = c_timeout;
                    NFI_Wait(!GET_PIO_RDY, timeout);
                    if( 0 == timeout ) {
                        return S_TIMEOUT;
                    }
                    *(UINT8*)p1TmpPage = *NFI_DATAR;
                    p1TmpPage ++;
                }  // end for(j=0; j<pDevCfg->m_SecSpareSize;
                *NFI_CNFG &= (~CNFG_BYTE_RW);
            }
        }
    }
    return S_DONE;

}

static void  NAND_COMMON_PageRead(struct mtd_info *mtd, unsigned command, int column, int page_addr)
{
    register struct nand_chip *mt85xx = mtd->priv;
    u32    page_size = mtd->writesize;
	u32 timeout = MTD_NAND_DEFAULT_TIMEOUT;

	page_addr_for_read = page_addr;

	// reset the NFI core state machine, data FIFO and flushing FIFO
    *NFI_CON = CON_NFI_RST | CON_FIFO_FLUSH;
    // Set Sector number
    *NFI_CON = CON_NFI_SEC_NUM(u4SectorNum);

    *NFI_CNFG = CNFG_RAN_SEL;  // Recover to default value
    *NFI_CNFG |= (CNFG_OP_READ | CNFG_READ_EN);
    *NFI_CNFG  |= CNFG_HW_ECC_EN;
    if (_fgAUTO_FMT) {
        *NFI_CNFG |= CNFG_AUTO_FMT_EN;
    }
    if (_fgUsingDMA) {
        *NFI_CNFG |= (_fgDMABurstEn) ? (CNFG_DMA | CNFG_DMA_BURST_EN) : (CNFG_DMA);
    }

    // Randomizer config
    if (_fgRand_on) {
        if (!_fgRandSeed_by_sec) {  // FALSE is randomizer by page.
            _NFI_HW_RandomizerConfig(mtd, page_addr, FALSE);  // randomizer decoder
        }
    }else {
        *NFI_RANDOM_CNFG = 0;  // Disable randomizer decoded.
    }

    *NFIECC_ENCCON = 0; // make sure encoder is closed.
    *NFIECC_DECCON = DEC_EN;

    // read cmd
    *NFI_CMD = NAND_COMMON_PAGE_READ_1ST_CMD;
    // wait til CMD is completely issued
    timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_STA & STATUS_CMD), timeout);

    // fill 3~5 cycle addr
    //UTIL_Printf("ReadPage Address = 0x%08x.\n", u4RowAddr);
    *NFI_COLADDR = 0;
    *NFI_ROWADDR = page_addr;
}


//------------------------------------------------------------------------------
// Read Status Callback Function
//------------------------------------------------------------------------------
STATUS_E  NAND_COMMON_ReadStatus(
        const struct mtd_info *mtd
        ,const u32  c_timeout
) {
    register struct nand_chip *mt85xx = mtd->priv;
//    struct mt85xx_priv *mt85xx = mtd->priv;
    u32* p4Data = (u32*)mt85xx->buffers->databuf;
	UINT32    u4Timeout = c_timeout;
	STATUS_E  eRet = S_UNKNOWN_ERR;

    // reset the NFI core state machine, data FIFO and flushing FIFO
    *NFI_CON = CON_NFI_RST | CON_FIFO_FLUSH;

    // Disable randomizer encoder and decoder
    *NFI_RANDOM_CNFG = 0;

    // Single Byte read.
    *NFI_CNFG = CNFG_RAN_SEL;  // Recover to default value
    *NFI_CNFG |= (CNFG_OP_SRD | CNFG_BYTE_RW | CNFG_READ_EN);

    // read status cmd
    *NFI_CMD = NAND_COMMON_READ_STATUS_CMD;
    // wait til CMD is completely issued
    NFI_Wait((*NFI_STA & STATUS_CMD), u4Timeout);

    // set single read 1 byte
    *NFI_CON = (CON_NFI_SRD | CON_NOB_BYTE);
    // wait til DATA_READ is completely issued
    u4Timeout = c_timeout;
    NFI_Wait((*NFI_STA & STATUS_DATAR), u4Timeout);

    // Check the PIO bit is ready or not.
    u4Timeout = c_timeout;
    NFI_Wait(!GET_PIO_RDY, u4Timeout);

    // single read doesn't need to polling FIFO
    p4Data[0] = *NFI_DATAR;

	// check READY/BUSY status first
    if(!(NAND_STATUS_READY & p4Data[0])) {
        eRet = S_IN_PROGRESS;
    }else if(NAND_STATUS_FAIL & p4Data[0] ) {
        // flash is ready now, check status code
        if(!(NAND_STATUS_WP & p4Data[0])) {
          eRet = S_BLOCK_LOCKED_ERR;
        }
        else {
          eRet = S_UNKNOWN_ERR;
        }
    }
    else {
        eRet = S_DONE;
    }

    return eRet;
}

//------------------------------------------------------------------------------
// Block Erase Related Callback Function
//------------------------------------------------------------------------------
STATUS_E  NAND_COMMON_BlockErase(
        const struct mtd_info *mtd
        ,const u32  page_addr
) {
//  uint32  page_size;
//  uint32  spare_size;
    u32    page_size = mtd->writesize;
    s32 i4Val;
#ifdef _XOS_ISR_ENABLE_
    EV_GRP_EVENT_T rRecvEvt;
#endif
    u32 timeout;
    u32  addr_cycle, column_addr_bytes,row_addr_bytes;
    STATUS_E ret = S_UNKNOWN_ERR;

    if (page_size > 512)
    {
        addr_cycle = 5;
        column_addr_bytes = 2;
    }
    else if (mtd->writesize == 512)
    {
        addr_cycle = 4;
        column_addr_bytes = 1;
    }
	else
    {
        addr_cycle = 3;
        column_addr_bytes = 1;
    }
    row_addr_bytes = addr_cycle - column_addr_bytes;

	// reset the NFI core state machine, data FIFO and flushing FIFO
    *NFI_CON = CON_NFI_RST | CON_FIFO_FLUSH;

    *NFI_CNFG = CNFG_RAN_SEL;  // Recover to default value
    *NFI_CNFG |= CNFG_OP_ERASE ;

    *NFI_INTR_EN = INTR_ERASE_DONE_EN;

    // block erase cmd
    *NFI_CMD = NAND_COMMON_BLOCK_ERASE_1ST_CMD;
    // wait til CMD is completely issued
    timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_STA & STATUS_CMD), timeout);

    // fill 1~3 cycle addr, erase command only fill row address, so column bits shift is unnecessary
    *NFI_COLADDR = 0;
    *NFI_ROWADDR = page_addr;
    // no. of addr cycle
    *NFI_ADDRNOB = ROW_ADDR_NUM(row_addr_bytes);
    // wait til ADDR is completely issued
    timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_STA & STATUS_ADDR), timeout);

    // block erase confirm
    *NFI_CMD = NAND_COMMON_BLOCK_ERASE_2ND_CMD;
    // wait til CMD is completely issued
    timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_STA & STATUS_CMD), timeout);

    timeout = MTD_NAND_DEFAULT_TIMEOUT;
#ifdef _XOS_ISR_ENABLE_
    if (_bUsingISR)
    {
        i4Val = x_ev_group_wait_event_timeout(_hNFIEvent,
                                      ERASE_DONE,
                                      &rRecvEvt,
                                      X_EV_OP_OR_CONSUME,
                                      timeout);
        if( i4Val == OSR_TIMEOUT )
        {
            ret = S_TIMEOUT;
            goto end;
        }
    }
    else
#endif //_XOS_ISR_ENABLE_
#ifdef LINUX_ISR_ENABLE
    if (_fgUsingISR)
    {
        struct mt85xx_priv *mt85xx = mtd->priv;
        wait_for_completion((struct completion*)&mt85xx->erase_completion);
    }
    else
#endif
    {
        NFI_Wait(!(*NFI_INTR_STA & INTR_ERASE_DONE), timeout); //clear INT status
        *NFI_INTR_EN &= (~INTR_ERASE_DONE_EN); // disable INT first
        if( 0 == timeout ) {
            ret = S_TIMEOUT;
            goto ERR_END;
        }
    }

	timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((S_IN_PROGRESS==(ret=NAND_COMMON_ReadStatus(mtd,MTD_NAND_DEFAULT_TIMEOUT))), timeout);
    if( 0 == timeout ) {
        ret = S_TIMEOUT;
        goto ERR_END;
    }
    else if( S_DONE != ret ) {
        goto ERR_END;
    }

    ret = S_DONE;

ERR_END:
    *NFI_CON = 0;
    *NFIECC_ENCCON = 0x0;

    return ret;
}

//------------------------------------------------------------------------------
// Write To NFI FIFO
//------------------------------------------------------------------------------
STATUS_E  NAND_COMMON_FIFO_Write(
        const struct mtd_info *mtd
        ,const u32  c_timeout
        ,const BOOL bUsingDMA
        ,const u32 *p_data32 /* MUST be 32bits alignment addr */
        ,const u32 *p_spare32
        ,const u32 page
) {
    u32  timeout = c_timeout;
    u32  i,j;
	u8* p1TmpPage, p1TmpSpare;

	p1TmpPage  = (u8*)p_data32;
	p1TmpSpare = (u8*)p_spare32;
#ifdef _XOS_ISR_ENABLE_
    int32 i4Val;
    EV_GRP_EVENT_T rRecvEvt;
#endif //_XOS_ISR_ENABLE_

    if(bUsingDMA)
    {
        // program page data with DMA

        // wait for DMA transmission complete
        timeout = c_timeout;
#ifdef _XOS_ISR_ENABLE_
        if (_bUsingISR)
        {
            i4Val = x_ev_group_wait_event_timeout(_hNFIEvent,
                                              AHB_DONE,
                                              &rRecvEvt,
                                              X_EV_OP_OR_CONSUME,
                                              timeout);

            if( i4Val == OSR_TIMEOUT )
                return S_TIMEOUT;
        }
        else
#endif //_XOS_ISR_ENABLE_
#ifdef LINUX_ISR_ENABLE
        if (_fgUsingISR)
        {
            wait_for_completion((struct completion*)&mt85xx->dma_completion);
        }
        else
#endif
		{    // Polling Interrupt status.
             // Trigger DMA action
            NFI_Wait(!(*NFI_INTR_STA & INTR_AHB_DONE), timeout);
            *NFI_INTR_EN &= (~INTR_AHB_DONE_EN); // disable INT first
            if( 0 == timeout) {
                return S_TIMEOUT;
            }
        }
    }
    else
	{ // PIO mode.  program page data
        for (i=0; i<u4SectorNum; i++)
        {
            // Toshiba randomizer seed config for each sector.
            if (_fgRandSeed_by_sec && _fgRand_on) {
                _NFI_HW_RandomizerConfig(mtd, (page+i), TRUE);  // randomizer encoder
            }

            for(j=0; j<u4SectorSize; j+=4)
            {
                // wait for FIFO has space to enqueue
                // when WR_FULL_MASK flag is poll-down, it means there are at least 4 bytes free space in FIFO.
                //NFI_WAIT((*NFI_FIFOSTA & FIFO_WR_FULL), u4Timeout);

                // Check the PIO bit is ready or not.
                timeout = c_timeout;
                NFI_Wait(!GET_PIO_RDY, timeout);
                if( 0 == timeout ) {
                    return S_TIMEOUT;
                }
                *NFI_DATAW = *(UINT32*)p1TmpPage;
                p1TmpPage += 4;
            }

            // Check current Sector is done, FDM and parity data by HW program.
            timeout = c_timeout;
            NFI_Wait((GET_SEC_CNTR!=(i+1)), timeout);
            if( 0 == timeout) {
                return S_TIMEOUT;
            }

			if (!_fgAUTO_FMT)
			{
				*NFI_CNFG |= CNFG_BYTE_RW;
				for(j=0; j<u4SecSpareSize; j++)
				{
					// Check the PIO bit is ready or not.
					timeout = c_timeout;
                    NFI_Wait(!GET_PIO_RDY, timeout);
                    if( 0 == timeout ) {
                        return S_TIMEOUT;
                    }
                    *NFI_DATAW = *(UINT8*)p1TmpSpare;
                    p1TmpSpare ++;
				}
				*NFI_CNFG &= (~CNFG_BYTE_RW);
			}
        }  // end for (i=0; i<pDevCfg->m_SectorNum; i++)
    }// end if (pSWCfg->fgAuto_FMT)  // FDM Data and Partiy data from HW to Nand
    return S_DONE;
}

//------------------------------------------------------------------------------
// Page Program Callback Function
//------------------------------------------------------------------------------
STATUS_E  NAND_COMMON_SEQIN(
        const struct mtd_info *mtd
        ,const u32  c_timeout
        ,const u32  page_addr
        //,const u32 *p_data32 /* MUST be 32bits alignment addr */
        //,const u32 *p_spare32
        //,uint32  ecc_parity_from_reg[4]
) {
    register struct nand_chip *mt85xx = mtd->priv;
    u32    page_size, spare_size;
    u32    column_addr_bytes,row_addr_bytes;
    u32    addr_cycle;
	u32 timeout = c_timeout;
//    u32     timeout;
    //  uint32    io_interface;
//    STATUS_E  ret=S_UNKNOWN_ERR;
//    u32 i;
    u32* p_data32 = (u32*)mt85xx->buffers->databuf;
    u32 u4TmpValue;
#ifdef _XOS_ISR_ENABLE_
    s32 i4Val;
    EV_GRP_EVENT_T rRecvEvt;
#endif //_XOS_ISR_ENABLE_

    //ASSERT(!((UINT32)p_data32%0x4)); //assert

    //if (_fgDataInvert && _bAUTO_FMT)  //  _bAUTO_FMT to avoid the recursive loop
    //    return NAND_PATCH_PageProgram(nand_info,c_timeout, row_addr, p_data32, p_spare32);

    page_size = mtd->writesize;
    spare_size = mtd->oobsize;

    if (page_size > 512)
    {
        addr_cycle = 5;
        column_addr_bytes = 2;
    }
    else
    {
        addr_cycle = 3;
        column_addr_bytes = 1;
    }
    row_addr_bytes = addr_cycle - column_addr_bytes;

    // reset the NFI core state machine, data FIFO and flushing FIFO
    *NFI_CON = CON_NFI_RST | CON_FIFO_FLUSH;
    // Set Sector number
    *NFI_CON = CON_NFI_SEC_NUM(u4SectorNum);

    *NFI_CNFG = CNFG_RAN_SEL;  // Recover to default value
    *NFI_CNFG |= CNFG_OP_PRGM ;

    if (_fgUsingDMA) {
        *NFI_CNFG |= (_fgDMABurstEn) ? (CNFG_DMA | CNFG_DMA_BURST_EN) : (CNFG_DMA);
        *NFI_STRADDR = (u32)p_data32 | NAND_HW_OFFSET;
    }

    // Randomizer config
    if (_fgRand_on) {
        if (!_fgRandSeed_by_sec) {  // FALSE is randomizer by page.
            _NFI_HW_RandomizerConfig(mtd, page_addr, TRUE);  // randomizer encoder
        }
    }else {
        *NFI_RANDOM_CNFG = 0;  // Disable randomizer encoded.
    }

    *NFIECC_DECCON = 0;  // Make sure decoder is closed.
    *NFIECC_ENCCON = ENC_EN;

    // in most 512 page size NAND flash, you have to setup destination pointer to 1st half area
    if(page_size == 512) {
        *NFI_CMD = NAND_COMMON_PAGE_READ_1ST_CMD;
        // wait til CMD is completely issued
        timeout = c_timeout;
        NFI_Wait((*NFI_STA & STATUS_CMD), timeout);
    }

    // program cmd
    *NFI_CMD = NAND_COMMON_PAGE_PROGRAM_CMD;
    // wait til CMD is completely issued
    timeout = c_timeout;
    NFI_Wait((*NFI_STA & STATUS_CMD), timeout);

    // fill 3~5 cycle addr
    //UTIL_Printf("WritePage Address = 0x%08x.\n", u4RowAddr);
    *NFI_COLADDR = 0;
    *NFI_ROWADDR = page_addr;
    *NFI_ADDRNOB = (COL_ADDR_NUM(column_addr_bytes) | ROW_ADDR_NUM(row_addr_bytes));
    // wait til ADDR is completely issued
    timeout = c_timeout;
    NFI_Wait((*NFI_STA & STATUS_ADDR), timeout);

    return S_DONE;
}


//------------------------------------------------------------------------------
// Page Program Callback Function
//------------------------------------------------------------------------------
STATUS_E  NAND_COMMON_PageProgram(
        const struct mtd_info *mtd
        ,const u32  c_timeout
        ,const u32  page_addr
        //,const u32 *p_data32 /* MUST be 32bits alignment addr */
        //,const u32 *p_spare32
        //,uint32  ecc_parity_from_reg[4]
) {
    register struct nand_chip *mt85xx = mtd->priv;
    u32    page_size, spare_size;
    u32    column_addr_bytes,row_addr_bytes;
    u32    addr_cycle;
    u32     timeout;
    //  uint32    io_interface;
    STATUS_E  ret=S_UNKNOWN_ERR;
    u32 i,j;
    u32* p_data32 = (u32*)mt85xx->buffers->databuf;
	u32 u4FDMData_L =0xFFFFFFFF, u4FDMData_M =0xFFFFFFFF;
	u8  *p1TmpSpare;
	u32 u4TmpValue;
#ifdef _XOS_ISR_ENABLE_
    s32 i4Val;
    EV_GRP_EVENT_T rRecvEvt;
#endif //_XOS_ISR_ENABLE_

    //ASSERT(!((UINT32)p_data32%0x4)); //assert

    //if (_fgDataInvert && _bAUTO_FMT)  //  _bAUTO_FMT to avoid the recursive loop
    //    return NAND_PATCH_PageProgram(nand_info,c_timeout, row_addr, p_data32, p_spare32);

    page_size =  mtd->writesize;
    spare_size = mtd->oobsize;

    if (page_size > 512)
    {
        addr_cycle = 5;
        column_addr_bytes = 2;
    }
    else if (mtd->writesize == 512)
    {
        addr_cycle = 4;
        column_addr_bytes = 1;
    }
	else
    {
        addr_cycle = 3;
        column_addr_bytes = 1;
    }
    row_addr_bytes = addr_cycle - column_addr_bytes;

	p1TmpSpare = (u8*)(mt85xx->buffers->databuf+ page_size);
	if (_fgAUTO_FMT) {
        *NFI_CNFG |= (CNFG_AUTO_FMT_EN | CNFG_HW_ECC_EN);
        // prepare FDM data
        // FDM Data from DRAM to FDM registesr.
        for (i=0; i<u4SectorNum; i++) {
            if (FDM_BYTES <=4) {
                for(j=0; j<FDM_BYTES; j++) {
                    u4FDMData_L &= (~(UINT32)(0x000000FF<<(j<<3)));
                    u4FDMData_L |= ((UINT32)(*p1TmpSpare) <<(j<<3));
                    p1TmpSpare++;
                }
                *NFI_FDM_L(i) = u4FDMData_L;
            }else {
                for(j=0; j<4; j++) {
                    u4FDMData_L &= (~(UINT32)(0x000000FF<<(j<<3)));
                    u4FDMData_L |= ((UINT32)(*p1TmpSpare) <<(j<<3));
                    p1TmpSpare++;
                }
                *NFI_FDM_L(i) = u4FDMData_L;

                for(j=0; j<(FDM_BYTES-4); j++) {
                    u4FDMData_M &= (~(UINT32)(0x000000FF<<(j<<3)));
                    u4FDMData_M |= ((UINT32)(*p1TmpSpare) <<(j<<3));
                    p1TmpSpare++;
                }
                *NFI_FDM_M(i) = u4FDMData_M;
            }
        }// end for (i=0; i<pDevCfg->m_SectorNum; i++)
    }

    if (_fgUsingDMA) {
        *NFI_INTR_EN |= (INTR_AHB_DONE_EN);
        u4TmpValue = *NFI_INTR_STA;  // clean interrupt status
    }

	// set burst program by DWORD
    *NFI_CON |= (CON_NFI_BWR | CON_NOB_DWORD);

    // program page data
	timeout = c_timeout;
    ret = NAND_COMMON_FIFO_Write(mtd, timeout, _fgUsingDMA, p_data32, (u32*)p1TmpSpare, page_addr);
    if (S_DONE != ret) {
        goto ERR_END;
    }

    // <<<<  WARNING!! >>>>
    // 1. You MUST read parity registers before issue program confirm (0x10) command.
    //    Since the parity registers will be clean by NFI after issue program confirm.
    // 2. You MUST wait until the NFI FIFO is empty!
    //    It means all data in the FIFO had been written to NAND flash, and then you can
    //    start to read ECC parity registers.
    //while(!(*NFI_FIFOSTA & FIFO_WR_EMPTY));
    //while ( *NFI_ADDRCNTR & 0x3FF);
    timeout = c_timeout;
    NFI_Wait((GET_SEC_CNTR != u4SectorNum), timeout); // write sector Number.
    if( 0 == timeout ) {
        ret = S_TIMEOUT;
        goto ERR_END;
    }

    *NFI_INTR_EN |= INTR_WR_DONE_EN;
    u4TmpValue = *NFI_INTR_STA;  // clean interrupt status
    // program confirm
    *NFI_CMD = NAND_COMMON_PAGE_PROG_CNF_CMD;
    // wait til CMD is completely issued
    timeout = c_timeout;
    NFI_Wait((*NFI_STA & STATUS_CMD), timeout);

    *NFI_CNRNB = CNRNB_STROBE_TIMEOUT;

    timeout = c_timeout;
#ifdef _XOS_ISR_ENABLE_
    if (_bUsingISR)
    {
        i4Val = x_ev_group_wait_event_timeout(_hNFIEvent,
                                      WR_DONE,
                                      &rRecvEvt,
                                      X_EV_OP_OR_CONSUME,
                                      timeout);
        if( i4Val == OSR_TIMEOUT )
        {
            ret = S_TIMEOUT;
            goto ERR_END;
        }
    }
    else
#endif //_XOS_ISR_ENABLE_
	{
		timeout = c_timeout;
        NFI_Wait(!(*NFI_INTR_STA & INTR_WR_DONE), timeout);
        *NFI_INTR_EN &= (~INTR_WR_DONE_EN); // disable INT first
        if( 0 == timeout) {
            ret = S_TIMEOUT;
            goto ERR_END;
        }
    }

	//read NFI master status ,must ensure that there is MAS in the address phase of AHB protocol,
    //MAS_ isn't in the read_data phase or write data phase  of read data delay phase
    timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_MASTER_STA & 0xFFF), timeout);

	timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((S_IN_PROGRESS==(ret=NAND_COMMON_ReadStatus(mtd,MTD_NAND_DEFAULT_TIMEOUT))), timeout);
    if( 0 == timeout ) {
        ret = S_TIMEOUT;
        goto ERR_END;
    }
    else if( S_DONE != ret ) {
        goto ERR_END;
    }

    ret = S_DONE;

ERR_END:

	//read NFI master status ,must ensure that there is MAS in the address phase of AHB protocol,
    //MAS_ isn't in the read_data phase or write data phase  of read data delay phase
    timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_MASTER_STA & 0xFFF), timeout);

    *NFI_RANDOM_CNFG = 0;  // Disable randomizer encoded.
    // disable burst read
    *NFI_CON = 0x0;
    *NFIECC_ENCCON = 0x0;

    if ( ret != S_DONE)
        printk("\nwrite page error, error code = 0x%x\n", ret);

//    printk("\nPageProgram completed , FDM = 0x%x, ret = 0x%x\n", nfi_readl(NFI_FDM0L), ret);

    return ret;
}

static void mt85xx_nand_cmdfunc(struct mtd_info *mtd, unsigned command,
			      int column, int page_addr)
{
//    register struct nand_chip *this = mtd->priv;
    unsigned long    page_size = mtd->writesize;
    _data_buf_index = 0;

    if (command == NAND_CMD_ERASE2 ) {
        /* Second half of a command we already calculated */
        goto do_command;
    }
#if 1
    /* Emulate NAND_CMD_READOOB on large-page chips */
    if (page_size >= 512 &&  command == NAND_CMD_READOOB) {
        //column += mtd->writesize;
        command = NAND_CMD_READ0;
    }
#endif
    switch (command)
    {
        case NAND_CMD_READID:
            NAND_COMMON_ReadID(mtd, command);
             break;

        case NAND_CMD_READ0:
            NAND_COMMON_PageRead(mtd, command, column, page_addr);
            break;

        case NAND_CMD_STATUS:
            NAND_COMMON_ReadStatus(mtd,MTD_NAND_DEFAULT_TIMEOUT);
            break;

        case NAND_CMD_ERASE1:
            NAND_COMMON_BlockErase(mtd, page_addr);
            break;

        case NAND_CMD_SEQIN:
            NAND_COMMON_SEQIN(mtd,MTD_NAND_DEFAULT_TIMEOUT,page_addr);
            break;

        case NAND_CMD_PAGEPROG:
            NAND_COMMON_PageProgram(mtd,MTD_NAND_DEFAULT_TIMEOUT,page_addr);
            break;

        case NAND_CMD_RESET:
            NAND_COMMON_Reset(mtd, MTD_NAND_DEFAULT_TIMEOUT);
            break;
#if SUPPORT_MICRON_29F2G08ABAEA
        case NAND_CMD_SET_FEATURE:
			u8 u1FeatuerAddr = 0x80;
			u32 u4FeatureValue = 0x00030003;
            NAND_COMMON_SetFeature(mtd, u1FeatuerAddr, u4FeatureValue);
            break;

        case NAND_CMD_GET_FEATURE:
			u8 u1FeatuerAddr = 0x80;
			u8 u1FeatureValue[4] = 0x00000000;
            NAND_COMMON_GetFeature(mtd, u1FeatuerAddr, u1FeatureValue);
            break;
#endif
        default:
            printk("\ncommand code = 0x%x\n", command);
            BUG();

    }


do_command:
    return;
}

static void mt85xx_nand_select_chip(struct mtd_info *mtd, int chipnr)
{
	if(chipnr > 1)
		chipnr = 1;

    *NFI_CSEL = chipnr;
}

static int mt85xx_nand_read_oob(struct mtd_info *mtd, struct nand_chip *chip,
			      int page, int sndcmd)
{
    register struct nand_chip *mt85xx = mtd->priv;
    u32    page_size = mtd->writesize;
    u32     spare_size = mtd->oobsize;
    u32    column_addr_bytes, row_addr_bytes, addr_cycle;
    //u32* p_data32 = (u32*)chip->buffers->databuf;
    //u32* p_data32 = (u32*)buf;
    u32 i, j;
    u32 c_timeout = MTD_NAND_DEFAULT_TIMEOUT;
    STATUS_E ret = S_UNKNOWN_ERR;
    u8* p1TmpSpare = (chip->oob_poi);
    u32 p4TmpBuf[128];
	u32 u4TmpValue;
	u32 u4FDMData_L =0xFFFFFFFF, u4FDMData_M =0xFFFFFFFF;
	u8* buf = (u8*)mt85xx->buffers->databuf;

    if (mtd->writesize > 512)
    {
        addr_cycle = 5;
        column_addr_bytes = 2;
    }
    else if (mtd->writesize == 512)
    {
        addr_cycle = 4;
        column_addr_bytes = 1;
    }
	else
    {
        addr_cycle = 3;
        column_addr_bytes = 1;
    }
    row_addr_bytes = addr_cycle - column_addr_bytes;

	// reset the NFI core state machine, data FIFO and flushing FIFO
    *NFI_CON = CON_NFI_RST | CON_FIFO_FLUSH;
    // Set Sector number
    *NFI_CON = CON_NFI_SEC_NUM(u4SectorNum);

    *NFI_CNFG = CNFG_RAN_SEL;  // Recover to default value
    *NFI_CNFG |= (CNFG_OP_READ | CNFG_READ_EN);
    *NFI_CNFG  |= CNFG_HW_ECC_EN;
    if (_fgAUTO_FMT) {
        *NFI_CNFG |= CNFG_AUTO_FMT_EN;
    }
    if (_fgUsingDMA) {
        *NFI_CNFG |= (_fgDMABurstEn) ? (CNFG_DMA | CNFG_DMA_BURST_EN) : (CNFG_DMA);
    }

    // Randomizer config
    if (_fgRand_on) {
        if (!_fgRandSeed_by_sec) {  // FALSE is randomizer by page.
            _NFI_HW_RandomizerConfig(mtd, page, FALSE);  // randomizer decoder
        }
    }else {
        *NFI_RANDOM_CNFG = 0;  // Disable randomizer decoded.
    }

    *NFIECC_ENCCON = 0; // make sure encoder is closed.
    *NFIECC_DECCON = DEC_EN;

    // read cmd
    *NFI_CMD = NAND_COMMON_PAGE_READ_1ST_CMD;
    // wait til CMD is completely issued
    c_timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_STA & STATUS_CMD), c_timeout);

    // fill 3~5 cycle addr
    //UTIL_Printf("ReadPage Address = 0x%08x.\n", u4RowAddr);
    *NFI_COLADDR = 0;
    *NFI_ROWADDR = page;

    if (_fgUsingDMA)
    {
        *NFI_STRADDR = (u32)buf|NAND_HW_OFFSET; // the AHB start address must be sat before the *NFI_ADDRNOB for auto ecc correct address
    }

    // no. of addr cycle
    *NFI_ADDRNOB = (COL_ADDR_NUM(column_addr_bytes) | ROW_ADDR_NUM(row_addr_bytes));
    // wait til ADDR is completely issued
    c_timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_STA & STATUS_ADDR), c_timeout);

    // read confirm
    if(mtd->writesize > 512) {
        *NFI_CMD = NAND_COMMON_RAGE_READ_2ND_CMD;
		c_timeout = MTD_NAND_DEFAULT_TIMEOUT;
        NFI_Wait((*NFI_STA & STATUS_CMD), c_timeout);
    }

	*NFI_CNRNB = CNRNB_STROBE_TIMEOUT;

	while(!(*NFI_STA  & STATUS_BUSY2READY)); //wait busy return

	if (_fgUsingDMA)
	{
		u4TmpValue = *NFI_INTR_STA;  // clean interrupt status
    	*NFI_INTR_EN |= INTR_AHB_DONE_EN;
	}

	// set burst read by DWORD
    *NFI_CON |= (CON_NFI_BRD | CON_NOB_DWORD);

    ret = NAND_COMMON_FIFO_Read(mtd, c_timeout, _fgUsingDMA, buf, page);
    if (_fgUsingDMA) {
        ret = NAND_COMMON_ECCErrDetect();
    }

    if (S_ECC_UNCORRECT_ERR == ret)
    {
        if (STATUS_READ_EMPTY_PAGE == (*NFI_STA & STATUS_READ_EMPTY_PAGE)) // Read is empty page
        {
        	memset(buf, 0xFF, page_size);
			memset(p1TmpSpare, 0xFF, u4SectorNum * FDM_BYTES);
            ret = S_DONE;
			goto end;
        }
    }

    //read NFI master status ,must ensure that there is MAS in the address phase of AHB protocol,
    //MAS_ isn't in the read_data phase or write data phase  of read data delay phase
    c_timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_MASTER_STA & 0xFFF), c_timeout);

    if (_fgUsingDMA) {
        // DMA and Auto FMT read FDM data from FDM register to Memory.
        if ((_fgAUTO_FMT) && ((S_DONE==ret) || (S_ECC_CORRECTABLE_ERR==ret)))
        {
            // FDM from FDM registesr.
            for (i=0; i<u4SectorNum; i++)
            {
                u4FDMData_L = *NFI_FDM_L(i);
                if (FDM_BYTES <=4) {
                    for(j=0; j<FDM_BYTES; j++) {
                        *p1TmpSpare = (u8)((u4FDMData_L>>(j<<3)) &0xFF);
                        p1TmpSpare++;
                    }
                }else {
                    for(j=0; j<4; j++) {
                        *p1TmpSpare = (u8)((u4FDMData_L>>(j<<3)) &0xFF);
                        p1TmpSpare++;
                    }

                    u4FDMData_M = *NFI_FDM_M(i);
                    for(j=0; j<(FDM_BYTES-4); j++) {
                        *p1TmpSpare = (u8)((u4FDMData_M>>(j<<3)) &0xFF);
                        p1TmpSpare++;
                    }
                }
            }
        }
    }// end if (pSWCfg->fgUsingDMA)

end:
    *NFI_RANDOM_CNFG = 0;  // Disable randomizer decoded.
    // disable burst read
    *NFI_CON = 0x0;
    *NFIECC_DECCON = 0x0;

	return ret;
}


static int mt85xx_nand_read_page(struct mtd_info *mtd, struct nand_chip *chip,
			       uint8_t *buf)
{
//    register struct nand_chip *mt85xx = mtd->priv;
    u32    page_size = mtd->writesize;
    u32     spare_size = mtd->oobsize;
    u32    column_addr_bytes, row_addr_bytes, addr_cycle;
    //u32* p_data32 = (u32*)chip->buffers->databuf;
    //u32* p_data32 = (u32*)buf;
    u32 i, j;
    u32 c_timeout = MTD_NAND_DEFAULT_TIMEOUT;
    STATUS_E ret = S_UNKNOWN_ERR;
    u8* p1TmpSpare = (chip->oob_poi);
    u32 p4TmpBuf[128];
	u32 u4TmpValue;
	u32 u4FDMData_L =0xFFFFFFFF, u4FDMData_M =0xFFFFFFFF;
//     volatile u32* p4TmpSpare2 = p4TmpBuf;

    if (mtd->writesize > 512)
    {
        addr_cycle = 5;
        column_addr_bytes = 2;
    }
    else if (mtd->writesize == 512)
    {
        addr_cycle = 4;
        column_addr_bytes = 1;
    }
	else
    {
        addr_cycle = 3;
        column_addr_bytes = 1;
    }
    row_addr_bytes = addr_cycle - column_addr_bytes;

    if (_fgUsingDMA)
    {
        *NFI_STRADDR = (u32)buf|NAND_HW_OFFSET; // the AHB start address must be sat before the *NFI_ADDRNOB for auto ecc correct address
    }

    // no. of addr cycle
    *NFI_ADDRNOB = (COL_ADDR_NUM(column_addr_bytes) | ROW_ADDR_NUM(row_addr_bytes));
    // wait til ADDR is completely issued
    c_timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_STA & STATUS_ADDR), c_timeout);

    // read confirm
    if(mtd->writesize > 512) {
        *NFI_CMD = NAND_COMMON_RAGE_READ_2ND_CMD;
		c_timeout = MTD_NAND_DEFAULT_TIMEOUT;
        NFI_Wait((*NFI_STA & STATUS_CMD), c_timeout);
    }

	*NFI_CNRNB = CNRNB_STROBE_TIMEOUT;

	while(!(*NFI_STA  & STATUS_BUSY2READY)); //wait busy return

	if (_fgUsingDMA)
	{
		u4TmpValue = *NFI_INTR_STA;  // clean interrupt status
    	*NFI_INTR_EN |= INTR_AHB_DONE_EN;
	}

	// set burst read by DWORD
    *NFI_CON |= (CON_NFI_BRD | CON_NOB_DWORD);

    ret = NAND_COMMON_FIFO_Read(mtd, c_timeout, _fgUsingDMA, buf, page_addr_for_read);
    if (_fgUsingDMA) {
        ret = NAND_COMMON_ECCErrDetect();
    }

    if (S_ECC_UNCORRECT_ERR == ret)
    {
        if (STATUS_READ_EMPTY_PAGE == (*NFI_STA & STATUS_READ_EMPTY_PAGE)) // Read is empty page
        {
        	memset(buf, 0xFF, page_size);
			memset(p1TmpSpare, 0xFF, u4SectorNum * FDM_BYTES);
            ret = S_DONE;
			goto end;
        }
    }

    //read NFI master status ,must ensure that there is MAS in the address phase of AHB protocol,
    //MAS_ isn't in the read_data phase or write data phase  of read data delay phase
    c_timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_MASTER_STA & 0xFFF), c_timeout);

    if (_fgUsingDMA) {
        // DMA and Auto FMT read FDM data from FDM register to Memory.
        if ((_fgAUTO_FMT) && ((S_DONE==ret) || (S_ECC_CORRECTABLE_ERR==ret)))
        {
            // FDM from FDM registesr.
            for (i=0; i<u4SectorNum; i++)
            {
                u4FDMData_L = *NFI_FDM_L(i);
                if (FDM_BYTES <=4) {
                    for(j=0; j<FDM_BYTES; j++) {
                        *p1TmpSpare = (u8)((u4FDMData_L>>(j<<3)) &0xFF);
                        p1TmpSpare++;
                    }
                }else {
                    for(j=0; j<4; j++) {
                        *p1TmpSpare = (u8)((u4FDMData_L>>(j<<3)) &0xFF);
                        p1TmpSpare++;
                    }

                    u4FDMData_M = *NFI_FDM_M(i);
                    for(j=0; j<(FDM_BYTES-4); j++) {
                        *p1TmpSpare = (u8)((u4FDMData_M>>(j<<3)) &0xFF);
                        p1TmpSpare++;
                    }
                }
            }
		ret = S_DONE;
        }
    }// end if (pSWCfg->fgUsingDMA)

end:
    *NFI_RANDOM_CNFG = 0;  // Disable randomizer decoded.
    // disable burst read
    *NFI_CON = 0x0;
    *NFIECC_DECCON = 0x0;

	return ret;
}

#define MTK_MARK_BAD_BLOCK 1
#if MTK_MARK_BAD_BLOCK
uint32_t _setbadblk = 0;
#endif
int mt85xx_nand_write_page(struct mtd_info *mtd, struct nand_chip *chip,
				const uint8_t *buf, int page, int cached, int raw)
{
    register struct nand_chip *mt85xx = mtd->priv;
    u32    page_size, spare_size;
    u32    column_addr_bytes,row_addr_bytes;
    u32    addr_cycle;
    u32     timeout;
    u32 c_timeout = MTD_NAND_DEFAULT_TIMEOUT;
    //  uint32    io_interface;
    STATUS_E  ret=S_UNKNOWN_ERR;
    u32 i, j;
	u32 u4FDMData_L =0xFFFFFFFF, u4FDMData_M =0xFFFFFFFF;
	u8  *p1TmpSpare;
	u32  u4TmpValue =0;
    //u32* p_data32 = (u32*)buf;
    //u32* p_data32 = (u32*)mt85xx->buffers->databuf;
#ifdef _XOS_ISR_ENABLE_
    s32 i4Val;
    EV_GRP_EVENT_T rRecvEvt;
#endif //_XOS_ISR_ENABLE_
#if MTK_MARK_BAD_BLOCK
    BOOL temp = _fgAUTO_FMT;
    if (_setbadblk)
    {
      _fgAUTO_FMT = FALSE;
    }
#endif

    page_size =  mtd->writesize;
    spare_size = mtd->oobsize;
    p1TmpSpare = (u8*)(mt85xx->buffers->databuf+ page_size);

    if (mtd->writesize > 512)
    {
        addr_cycle = 5;
        column_addr_bytes = 2;
    }
    else if (mtd->writesize == 512)
    {
        addr_cycle = 4;
        column_addr_bytes = 1;
    }
	else
    {
        addr_cycle = 3;
        column_addr_bytes = 1;
    }
    row_addr_bytes = addr_cycle - column_addr_bytes;

	// reset the NFI core state machine, data FIFO and flushing FIFO
    *NFI_CON = CON_NFI_RST | CON_FIFO_FLUSH;
    // Set Sector number
    *NFI_CON = CON_NFI_SEC_NUM(u4SectorNum);

    *NFI_CNFG = CNFG_RAN_SEL;  // Recover to default value
    *NFI_CNFG |= CNFG_OP_PRGM ;

    if (_fgAUTO_FMT) {
        *NFI_CNFG |= (CNFG_AUTO_FMT_EN | CNFG_HW_ECC_EN);
        // prepare FDM data
        // FDM Data from DRAM to FDM registesr.
        for (i=0; i<u4SectorNum; i++) {
            if (FDM_BYTES <=4) {
                for(j=0; j<FDM_BYTES; j++) {
                    u4FDMData_L &= (~(UINT32)(0x000000FF<<(j<<3)));
                    u4FDMData_L |= ((UINT32)(*p1TmpSpare) <<(j<<3));
                    p1TmpSpare++;
                }
                *NFI_FDM_L(i) = u4FDMData_L;
            }else {
                for(j=0; j<4; j++) {
                    u4FDMData_L &= (~(UINT32)(0x000000FF<<(j<<3)));
                    u4FDMData_L |= ((UINT32)(*p1TmpSpare) <<(j<<3));
                    p1TmpSpare++;
                }
                *NFI_FDM_L(i) = u4FDMData_L;

                for(j=0; j<(FDM_BYTES-4); j++) {
                    u4FDMData_M &= (~(UINT32)(0x000000FF<<(j<<3)));
                    u4FDMData_M |= ((UINT32)(*p1TmpSpare) <<(j<<3));
                    p1TmpSpare++;
                }
                *NFI_FDM_M(i) = u4FDMData_M;
            }
        }// end for (i=0; i<pDevCfg->m_SectorNum; i++)
    }
    if (_fgUsingDMA) {
        *NFI_CNFG |= (_fgDMABurstEn) ? (CNFG_DMA | CNFG_DMA_BURST_EN) : (CNFG_DMA);
        *NFI_STRADDR = ((u32)buf)|NAND_HW_OFFSET;
        *NFI_INTR_EN |= (INTR_AHB_DONE_EN);
        u4TmpValue = *NFI_INTR_STA;  // clean interrupt status
    }

    // Randomizer config
    if (_fgRand_on) {
        if (!_fgRandSeed_by_sec) {  // FALSE is randomizer by page.
            _NFI_HW_RandomizerConfig(mtd, page, TRUE);  // randomizer encoder
        }
    }else {
        *NFI_RANDOM_CNFG = 0;  // Disable randomizer encoded.
    }

    *NFIECC_DECCON = 0;  // Make sure decoder is closed.
    *NFIECC_ENCCON = ENC_EN;

    // in most 512 page size NAND flash, you have to setup destination pointer to 1st half area
    if(page_size == 512) {
        *NFI_CMD = NAND_COMMON_PAGE_READ_1ST_CMD;
        // wait til CMD is completely issued
        c_timeout = MTD_NAND_DEFAULT_TIMEOUT;
        NFI_Wait((*NFI_STA & STATUS_CMD), c_timeout);
    }

    // program cmd
    *NFI_CMD = NAND_COMMON_PAGE_PROGRAM_CMD;
    // wait til CMD is completely issued
    c_timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_STA & STATUS_CMD), c_timeout);

    // fill 3~5 cycle addr
    //UTIL_Printf("WritePage Address = 0x%08x.\n", u4RowAddr);
    *NFI_COLADDR = 0;
    *NFI_ROWADDR = page;
    *NFI_ADDRNOB = (COL_ADDR_NUM(column_addr_bytes) | ROW_ADDR_NUM(row_addr_bytes));
    // wait til ADDR is completely issued
    c_timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_STA & STATUS_ADDR), c_timeout);

    // set burst program by DWORD
    *NFI_CON |= (CON_NFI_BWR | CON_NOB_DWORD);

	// program page data
	c_timeout = MTD_NAND_DEFAULT_TIMEOUT;
    ret = NAND_COMMON_FIFO_Write(mtd, c_timeout, _fgUsingDMA, (u32*)buf, (u32*)(buf + mtd->writesize), page);
    if (S_DONE != ret) {
        goto ERR_END;
    }

    // <<<<  WARNING!! >>>>
    // 1. You MUST read parity registers before issue program confirm (0x10) command.
    //    Since the parity registers will be clean by NFI after issue program confirm.
    // 2. You MUST wait until the NFI FIFO is empty!
    //    It means all data in the FIFO had been written to NAND flash, and then you can
    //    start to read ECC parity registers.
    //while(!(*NFI_FIFOSTA & FIFO_WR_EMPTY));
    //while ( *NFI_ADDRCNTR & 0x3FF);
    c_timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((GET_SEC_CNTR != u4SectorNum), c_timeout); // write sector Number.
    if( 0 == c_timeout ) {
        ret = S_TIMEOUT;
        goto ERR_END;
    }

    *NFI_INTR_EN |= INTR_WR_DONE_EN;
    u4TmpValue = *NFI_INTR_STA;  // clean interrupt status
    // program confirm
    *NFI_CMD = NAND_COMMON_PAGE_PROG_CNF_CMD;
    // wait til CMD is completely issued
    c_timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_STA & STATUS_CMD), c_timeout);

    *NFI_CNRNB = CNRNB_STROBE_TIMEOUT;

    timeout = MTD_NAND_DEFAULT_TIMEOUT;
#ifdef _XOS_ISR_ENABLE_
    if (_bUsingISR)
    {
        i4Val = x_ev_group_wait_event_timeout(_hNFIEvent,
                                      WR_DONE,
                                      &rRecvEvt,
                                      X_EV_OP_OR_CONSUME,
                                      timeout);
        if( i4Val == OSR_TIMEOUT )
        {
            ret = S_TIMEOUT;
            goto ERR_END;
        }
    }
    else
#endif //_XOS_ISR_ENABLE_
#ifdef LINUX_ISR_ENABLE
    if (_fgUsingISR)
    {
        struct mt85xx_priv *mt85xx = mtd->priv;
        wait_for_completion((struct completion*)&mt85xx->program_completion);
    }
    else
#endif
	{
		timeout = MTD_NAND_DEFAULT_TIMEOUT;
        NFI_Wait(!(*NFI_INTR_STA & INTR_WR_DONE), timeout);
        *NFI_INTR_EN &= (~INTR_WR_DONE_EN); // disable INT first
        if( 0 == timeout) {
            ret = S_TIMEOUT;
            goto ERR_END;
        }
    }

	//read NFI master status ,must ensure that there is MAS in the address phase of AHB protocol,
    //MAS_ isn't in the read_data phase or write data phase  of read data delay phase
    timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_MASTER_STA & 0xFFF), timeout);

	timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((S_IN_PROGRESS==(ret=NAND_COMMON_ReadStatus(mtd,MTD_NAND_DEFAULT_TIMEOUT))), timeout);
    if( 0 == timeout ) {
        ret = S_TIMEOUT;
        goto ERR_END;
    }
    else if( S_DONE != ret ) {
        goto ERR_END;
    }

    ret = S_DONE;
ERR_END:

	//read NFI master status ,must ensure that there is MAS in the address phase of AHB protocol,
    //MAS_ isn't in the read_data phase or write data phase  of read data delay phase
    timeout = MTD_NAND_DEFAULT_TIMEOUT;
    NFI_Wait((*NFI_MASTER_STA & 0xFFF), timeout);

#if MTK_MARK_BAD_BLOCK
		_fgAUTO_FMT = temp;
#endif
    *NFI_RANDOM_CNFG = 0;  // Disable randomizer encoded.
    // disable burst read
    *NFI_CON = 0x0;
    *NFIECC_ENCCON = 0x0;

    if ( ret != S_DONE)
    {
        printk("\nmt85xx_nand_write_page, error code = 0x%x\n", ret);
        return -1;
    }

    /////////
    //  test function
    ///

#ifdef CONFIG_MTD_NAND_VERIFY_WRITE
			if (((page * page_size) >= 0x9A00000) && ((page * page_size) <= 0xA600000))
			{
				NAND_COMMON_PageRead(mtd, NAND_CMD_READ0, 0 , page);
				if (-1==mt85xx_nand_read_page(mtd, chip,mt85xx->buffers->databuf))
				{
					printk("read verify read failed at page 0x%x\n", page);
					BUG();
					return -EIO;
				}
				if (memcmp(mt85xx->buffers->databuf, buf, page_size)!=0)
				{
					printk("read verify compare failed at page 0x%x\n", page);
					BUG();
					return -EIO;
				}
			}
#endif

    return 0;
}


static void mt85xx_nand_read_buf(struct mtd_info *mtd, unsigned char *buf, int len)
{
    register struct nand_chip *this = mtd->priv;

//    printk("\nread buffer, offset= 0x%x\n", mt85xx->datalen);

        memcpy(buf, this->buffers->databuf+_data_buf_index, len);

	_data_buf_index+= len;

}

static void mt85xx_nand_write_buf(struct mtd_info *mtd, const uint8_t *buf, int len)
{
    register struct nand_chip *this = mtd->priv;

//        printk("\nwrite buffer, offset= 0x%x\n", mt85xx->datalen);
    memcpy(this->buffers->databuf + _data_buf_index, buf, len);

    _data_buf_index += len;

}


static u_char mt85xx_nand_read_byte(struct mtd_info *mtd)
{
	unsigned char d;

	mt85xx_nand_read_buf(mtd, &d, 1);

	return d;
}

static u16 mt85xx_nand_read_word(struct mtd_info *mtd)
{
	u16 d;

	mt85xx_nand_read_buf(mtd, (u_char*)&d, 2);

	return d;
}

int mt85xx_nand_calculate_ecc(struct mtd_info *mtd, const u_char *dat, u_char *ecc_code)
{
    return 0;
}

int mt85xx_nand_correct_data(struct mtd_info *mtd, u_char *dat, u_char *read_ecc, u_char *calc_ecc)
{
    return 0;
}

static void mt85xx_nand_enable_hwecc(struct mtd_info *mtd, int mode)
{
}


int board_nand_init(struct nand_chip *nand)
{
	nand->buffers = kmalloc(sizeof(struct nand_buffers ), GFP_KERNEL);
	if(!nand->buffers)
	{
      printk("nand init allocate mem fail\n");
	  return -1;
	}
    nand->options |= NAND_USE_FLASH_BBT| NAND_NO_SUBPAGE_WRITE;

    NFI_Init();

    /* Set address of NAND IO lines (Using Linear Data Access Region) */
    nand->IO_ADDR_R = (void __iomem *) CONFIG_SYS_NAND_BASE;
    nand->IO_ADDR_W = (void __iomem *) CONFIG_SYS_NAND_BASE;
    /* Reference hardware control function */
    //nand->hwcontrol  = sc3_nand_hwcontrol;
    nand->dev_ready  = mt85xx_nand_device_ready;
    nand->select_chip = mt85xx_nand_select_chip;

    //nand->write_byte = nand_write_byte16 : nand_write_byte;
    nand->read_byte = mt85xx_nand_read_byte;
    //nand->write_word = nand_write_word;
    nand->read_word = mt85xx_nand_read_word;
    //nand->write_buf = nand_write_buf16 : nand_write_buf;
    nand->read_buf = mt85xx_nand_read_buf;
    nand->write_buf = mt85xx_nand_write_buf;

    nand->cmdfunc = mt85xx_nand_cmdfunc;
    nand->ecc.mode = NAND_ECC_HW;

    nand->write_page = mt85xx_nand_write_page;

    nand->ecc.calculate = mt85xx_nand_calculate_ecc;
    nand->ecc.correct = mt85xx_nand_correct_data;
    nand->ecc.hwctl = mt85xx_nand_enable_hwecc;
    nand->ecc.read_page = mt85xx_nand_read_page;
    nand->ecc.read_page_raw = mt85xx_nand_read_page;
    nand->ecc.read_oob = mt85xx_nand_read_oob;

#if 0
    nand->ecc.bytes = 12;
    nand->ecc.size = 1024;
#endif
	//    nand->bbt_td		= &mt85xx_bbt_main_descr;
	//    nand->bbt_md		= &mt85xx_bbt_mirror_descr;


	return 0;
    }

int mt85xx_nand_init(struct mtd_info *mtd)
{
	//    unsigned long    page_size = mtd->writesize;
    struct nand_chip *this = mtd->priv;

	NFI_Config(mtd);

    /* If no default placement scheme is given, select an
     * appropriate one */
    //if (!mtd->autooob) {
        /* Select the appropriate default oob placement scheme for
         * placement agnostic filesystems */
    switch (mtd->oobsize) {
        case 16:
            this->ecc.layout = &mt85xx_oobinfo_512;
            break;
        case 64:
            this->ecc.layout = &mt85xx_oobinfo_2048;
            break;
        case 128:
            this->ecc.layout = &mt85xx_oobinfo_4096;
            break;
        default:
            if (mtd->oobsize > 128)
                this->ecc.layout = &mt85xx_oobinfo_4096;
            else
                printk (KERN_WARNING "No oob scheme defined for oobsize %d\n",
                        mtd->oobsize);
            /*          BUG(); */
    }
    this->badblock_pattern = (mtd->writesize > 512) ?
			&mt85xx_largepage_bbt_descr : &mt85xx_smallpage_bbt_descr;
	#if 1
	if (mtd->writesize > 512)
	{
	  this->ecc.bytes = 12;
      this->ecc.size = 1024;
	}
	else
	{
      this->ecc.bytes = 4;
      this->ecc.size = 512;
	}
	#endif

    return 0;
}

