/*
* u-boot-2009.08/drivers/i2c/ext_mcu.c
*
* external mcu i2c operation
*
* Copyright (c) 2010-2012 Mediatek Inc.
* $Author: bdbm01 $
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
* more details.
* http://www.gnu.org/licenses/old-licenses/gpl-2.0.0html
*
*/
//#include "general.h"
//#include "hw_rs232.h"

#include <common.h>
#include <asm/io.h>
#include <asm/arch/mt85xx.h>
#include <asm/arch/sys_proto.h>
#include <asm/arch/sys_info.h>
#include <asm/mach-types.h>
#include <asm/arch/x_typedef.h>
#include <asm/arch/x_ckgen.h>
#include <mt85xx_gpio.h>
#include <asm/arch/drv_vfd_8530.h>
#include <asm/arch/mt85xx_vfd_cus.h>

#if CONFIG_MT85XX_VFD
#include <asm/arch/x_pdwnc.h>
#if CONFIG_EXTERNAL_MCU
#include <ext_mcu.h>
#endif

/******************************************************************************
* Local variable
******************************************************************************/
#define REG_RW_RESRV1 0x164
#define STDBY_CHECK_VALUE 0x24000164


/****************************************************************************
** Function prototypes
****************************************************************************/

/****************************************************************************
** static Function
****************************************************************************/
static void vStbyVfdDirectWrite(BYTE bAddr, BYTE bData);
static void vStbyVfdUpdate(UINT8 bTmp);
static void vStbyVfdSegDisplay(BYTE SegPos, BYTE bNum, BOOL fgDirect);
static void vStbyDirectSequenceUpd(BYTE bSize,BYTE *pbData, UINT32 wSegInfo);
static void vStbyLEDDirectWrite(BYTE bAddr, BYTE bData);
static void vStbyLEDUpdate(UINT8 bTmp);
static void vStbyLEDSegDisplay(BYTE SegPos, BYTE bNum, BOOL fgDirect);



const UINT8 _pbStbyVfdKeyScan[] = {
	IR_NONE,
};



const BYTE _pbStbyVfdSeg0[8][28] = 
{
  {0x00, 0x01, 0x00, 0x08, 0x01, 0x01, 0x01, 0x20, 0x01, 0x02, 0x00, 0x10, 0x00, 0x80, 0x00, 0x04, 0x00, 0x40, 0x01, 0x08, 0x01, 0x40, 0x01, 0x04, 0x00, 0x02, 0x01, 0x10, },
  {0x02, 0x01, 0x02, 0x08, 0x03, 0x01, 0x03, 0x20, 0x03, 0x02, 0x02, 0x10, 0x02, 0x80, 0x02, 0x04, 0x02, 0x40, 0x03, 0x08, 0x03, 0x40, 0x03, 0x04, 0x02, 0x02, 0x03, 0x10, },
  {0x04, 0x01, 0x04, 0x08, 0x05, 0x01, 0x05, 0x20, 0x05, 0x02, 0x04, 0x10, 0x04, 0x80, 0x04, 0x04, 0x04, 0x40, 0x05, 0x08, 0x05, 0x40, 0x05, 0x04, 0x04, 0x02, 0x05, 0x10, },
  {0x06, 0x01, 0x06, 0x08, 0x07, 0x01, 0x07, 0x20, 0x07, 0x02, 0x06, 0x10, 0x06, 0x80, 0x06, 0x04, 0x06, 0x40, 0x07, 0x08, 0x07, 0x40, 0x07, 0x04, 0x06, 0x02, 0x07, 0x10, },
  {0x08, 0x01, 0x08, 0x08, 0x09, 0x01, 0x09, 0x20, 0x09, 0x02, 0x08, 0x10, 0x08, 0x80, 0x08, 0x04, 0x08, 0x40, 0x09, 0x08, 0x09, 0x40, 0x09, 0x04, 0x08, 0x02, 0x09, 0x10, },
  {0x0a, 0x01, 0x0a, 0x08, 0x0b, 0x01, 0x0b, 0x20, 0x0b, 0x02, 0x0a, 0x10, 0x0a, 0x80, 0x0a, 0x04, 0x0a, 0x40, 0x0b, 0x08, 0x0b, 0x40, 0x0b, 0x04, 0x0a, 0x02, 0x0b, 0x10, },
  {0x0c, 0x01, 0x0c, 0x08, 0x0d, 0x01, 0x0d, 0x20, 0x0d, 0x02, 0x0c, 0x10, 0x0c, 0x80, 0x0c, 0x04, 0x0c, 0x40, 0x0d, 0x08, 0x0d, 0x40, 0x0d, 0x04, 0x0c, 0x02, 0x0d, 0x10, },
  {0x0e, 0x01, 0x0e, 0x08, 0x0f, 0x01, 0x0f, 0x20, 0x0f, 0x02, 0x0e, 0x10, 0x0e, 0x80, 0x0e, 0x04, 0x0e, 0x40, 0x0f, 0x08, 0x0f, 0x40, 0x0f, 0x04, 0x0e, 0x02, 0x0f, 0x10, },
};
// The following data is for Font Segment table.
static const BYTE _pbStbyLEDSeg0[5][28] = 
{
  {0x00, 0x01, 0x02, 0x01, 0x04, 0x01, 0x06, 0x01, 0x0c, 0x01, 0x0a, 0x01, 0x08, 0x01, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, },
  {0x00, 0x02, 0x02, 0x02, 0x04, 0x02, 0x06, 0x02, 0x0c, 0x02, 0x0a, 0x02, 0x08, 0x02, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, },
  {0x00, 0x04, 0x02, 0x04, 0x04, 0x04, 0x06, 0x04, 0x0c, 0x04, 0x0a, 0x04, 0x08, 0x04, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, },
  {0x00, 0x08, 0x02, 0x08, 0x04, 0x08, 0x06, 0x08, 0x0c, 0x08, 0x0a, 0x08, 0x08, 0x08, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, },
  {0x00, 0x10, 0x02, 0x10, 0x04, 0x10, 0x06, 0x10, 0x0c, 0x10, 0x0a, 0x10, 0x08, 0x10, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, },
};

const BYTE _pbStbyVfdMsg[][MAX_SEG_SEQ_NUM] = {
        {VFD_CHAR_P, VFD_CHAR_H, VFD_CHAR_I,VFD_CHAR_L, VFD_CHAR_I, VFD_CHAR_P,VFD_CHAR_S,CLR} // PHILIPS
       ,{VFD_CHAR_P, VFD_CHAR_L, VFD_CHAR_S, CLR, VFD_CHAR_W, VFD_CHAR_A, VFD_CHAR_I, VFD_CHAR_T}   // PLS WAIT
    //  ,{CLR, CLR, CLR, CLR, CLR, CLR, CLR, CLR},  // NULL
};

const BYTE _pbStbyLEDMsg[][MAX_SEG_SEQ_NUM_LED] = {
        {VFD_CHAR_H, VFD_CHAR_E, VFD_CHAR_L,VFD_CHAR_L, VFD_CHAR_O} // HELLO
       ,{VFD_CHAR_H, VFD_CHAR_E, VFD_CHAR_L,VFD_CHAR_L, VFD_CHAR_O}   // HELLO
    //  ,{CLR, CLR, CLR, CLR, CLR},  // NULL
};

// The following data is for Character table.
const UINT16 _pwStbySegUnit[] = 
{
    0x003f, 0x0006, 0x015b, 0x014f, 0x0166, 
    0x016d, 0x017d, 0x0007, 0x017f, 0x016f, 
    0x0000, 0x0177, 0x0177, 0x070f, 0x017c, 
    0x0039, 0x0158, 0x060f, 0x015e, 0x0179, 
    0x017b, 0x0171, 0x0171, 0x013d, 0x016f, 
    0x0176, 0x0174, 0x0609, 0x0609, 0x001f, 
    0x001e, 0x1870, 0x1870, 0x0038, 0x0038, 
    0x10b6, 0x0554, 0x08b6, 0x0037, 0x003f, 
    0x015c, 0x0173, 0x0173, 0x083f, 0x0067, 
    0x0973, 0x0050, 0x016d, 0x016d, 0x0601, 
    0x0178, 0x003e, 0x001c, 0x0886, 0x3030, 
    0x063e, 0x041c, 0x3880, 0x3880, 0x016e, 
    0x016e, 0x3009, 0x3009, 0x0000, 0x0000, 
    0x0008, 0x0140, 0x0600, 0x3000, 0x0880, 
    0x0039, 0x000f, 0xffff, 0x0503, 0x0740,
};

// The following data is for Character table.
//For the 8-segment VFD panel, u and v are the same font, w as u , and x as ||. 
const UINT16 _pwStbyLEDSegUnit[] = 
{
  0x003f, 0x0006, 0x015b, 0x014f, 0x0166, //0,1,2,3,4
  0x016d, 0x017d, 0x0007, 0x017f, 0x016f, //5,6,7,8,9
  0x0000, 0x0177, 0x0177, 0x007c, 0x007c, //null,A,a,B,b
  0x0039, 0x0058, 0x015e, 0x015e, 0x0179, //C,c,D,d,E
  0x017b, 0x0171, 0x0171, 0x016f, 0x016f, //e,F,f,G,g
  0x0176, 0x0174, 0x0006, 0x0006, 0x000e, //H,h,I,i,J
  0x000e, 0x0070, 0x0070, 0x0038, 0x0038, //j,K,k,L,l
  0x0137, 0x0137, 0x0037, 0x0054, 0x003f, //M,m,N,n,O
  0x015c, 0x0173, 0x0173, 0x0167, 0x0167, //o,P,p,Q,q

  0x0031,  0x0050, 0x016d, 0x016d, 0x0178, //R,r,S,s,T
  0x0178, 0x003e, 0x001c, 0x001c, 0x001c, //t,U,u,V,v
  0x003e, 0x001c, 0x0170, 0x0170, 0x006e,//0x0007, //W,w,X,x,Y
  0x006e, 0x000f, 0x014c, 0x0000, 0x0000, //y,Z,z
  0x0008, 0x0140, 0x0600, 0x0152, 0x0880, 
  0x0039, 0x000f, 0x017f, 
};


static void vStbyVfdUpdate(UINT8 bTmp) 
{
  UINT8 i;

  for (i = 0; i < 8; i++)
  {
    vStbyVfdClk(FALSE);

    vStbyVfdCombineBit(bTmp);
    bTmp >>= 1;

    vStbyVfdClk(TRUE);
  }

  vStbyVfdData(TRUE);
}

/****************************************************************************
** public  Function
****************************************************************************/
void vVFDInit(void) 
{ 
#if	CONFIG_PHILIPS_CUSTOMER_VFD    
	   return;
#endif
#if SUPPORT_T8032_VFD
	if(STDBY_CHECK_VALUE != PDWNC_READ32(REG_RW_RESRV1)) // only init while ac on
#endif
	{
		GPIO_Config(PIN_VFD_STB, OUTPUT, LOW);
		GPIO_Config(PIN_VFD_CLK, OUTPUT, LOW);
		GPIO_Config(PIN_VFD_DATA, OUTPUT, LOW);
		
		vStbyVfdStrobe(FALSE);		   // begin to write the VFD command
		vStbyVfdUpdate(MODESET);
		vStbyVfdStrobe(TRUE);		   // strobe pull high to release the mode set

		vStbyVfdStrobe(FALSE);		  
		vStbyVfdUpdate(DISPMODE_ON);
		vStbyVfdStrobe(TRUE);		  

		vStbyVfdDirectClrAll();

    #if (!SUPPORT_T8032_VFD)
		if(STDBY_CHECK_VALUE == PDWNC_READ32(REG_RW_RESRV1))
		{
			vStbyVfdShowMsg(SEG_MSG_PHILIPS, TRUE); 	   
			printf("[uboot_vfd] Showing 'PHILIPS'.\n");
		}
		else
    #endif
		{
			{
				vStbyVfdShowMsg(SEG_MSG_PHILIPS, TRUE); 	   
				printf("[uboot_vfd] Showing 'PHILIPS'.\n");
			}

		}
   }
}
void vLEDInit(void) 
{ 

#if	CONFIG_PHILIPS_CUSTOMER_VFD    
	   return;
#endif
#if SUPPORT_T8032_VFD
	if(STDBY_CHECK_VALUE != PDWNC_READ32(REG_RW_RESRV1)) // only init while ac on
#endif
	{
		GPIO_Config(PIN_VFD_STB, OUTPUT, LOW);
		GPIO_Config(PIN_VFD_CLK, OUTPUT, LOW);
		GPIO_Config(PIN_VFD_DATA, OUTPUT, LOW);
		
		vStbyVfdStrobe(FALSE);		   // begin to write the VFD command
		vStbyVfdUpdate(MODESET_LED);
		vStbyVfdStrobe(TRUE);		   // strobe pull high to release the mode set

		vStbyVfdStrobe(FALSE);		  
		vStbyVfdUpdate(DISPMODE_ON);
		vStbyVfdStrobe(TRUE);		  

		vStbyLEDDirectClrAll();

    #if (!SUPPORT_T8032_VFD)
		if(STDBY_CHECK_VALUE == PDWNC_READ32(REG_RW_RESRV1))
		{
			vStbyLEDShowMsg(SEG_MSG_PHILIPS, TRUE); 	   
			printf("[uboot_LED] Showing 'PHILIPS'.\n");
		}
		else
    #endif
		{
			{
				vStbyLEDShowMsg(SEG_MSG_PHILIPS, TRUE); 	   
				printf("[uboot_LED] Showing 'PHILIPS'.\n");
			}

		}
   }
}


void vStbyVFDInit(void) 
{ 
#if ((CONFIG_CHIP_VER_CURR != CONFIG_CHIP_VER_MT8561)&& (CONFIG_CHIP_VER_CURR != CONFIG_CHIP_VER_MT8563))
  GPIO_Config(PIN_NFCEN2, INPUT, LOW);//support both LED and VFD
  if(GPIO_Input(PIN_NFCEN2))//VFD
#else
  if(1)
#endif
  {
    vVFDInit();
  }
  else //LED
  {
    vLEDInit();
  }
}

void vStbyVfdDirectClrAll(void) 
{
  BYTE i;

  for(i = 0; i < MAX_VFD_ADDR; i++)
  {
    vStbyVfdDirectWrite(i, 0);
  }

}

void vStbyLEDDirectClrAll(void) 
{
  BYTE i;

  for(i = 0; i < MAX_VFD_ADDR_LED; i++)
  {
    vStbyVfdDirectWrite(i, 0);
  }

}


void vStbyVfdDirectSetAll(void) 
{
  BYTE i;

  for(i = 0; i < MAX_VFD_ADDR; i++)
  {
    vStbyVfdDirectWrite(i, 0xff);
  }

}
void vStbyLEDDirectSetAll(void) 
{
  BYTE i;

  for(i = 0; i < MAX_VFD_ADDR_LED; i++)
  {
    vStbyVfdDirectWrite(i, 0xff);
  }

}


void vStbyVfdShowMsg(BYTE bMsgIdIdx, BOOL fgDirect) 
{
  BYTE i = 0,j = 0;
  BYTE pbAddr[MAX_VFD_ADDR];
  BYTE pbData[MAX_VFD_ADDR];
  UINT16 wSegInfo = 0x00;
  UINT16 wUpdateData;
  
  for(i = 0; i < MAX_VFD_ADDR; i++)
  {
	pbAddr[i] = 0xff;
	pbData[i] = 0x00;
  }
	
  for(i = 0; i < MAX_SEG_SEQ_NUM; i++)
{

    wSegInfo = _pwStbySegUnit[_pbStbyVfdMsg[bMsgIdIdx][i]];

	 for(j = 0; j < SEG_SIZE; j ++)
	 {
		wUpdateData = wSegInfo >>j;
		if(0x01 & wUpdateData)
		{
			if(0xff == pbAddr[_pbStbyVfdSeg0[i][2*j]])
			{
				pbAddr[_pbStbyVfdSeg0[i][2*j]] = _pbStbyVfdSeg0[i][2*j];
			}
			pbData[_pbStbyVfdSeg0[i][2*j]] = pbData[_pbStbyVfdSeg0[i][2*j]] | (_pbStbyVfdSeg0[i][2*j+1]);
		}
		else
		{
			
		}
	 }
  }
  for(i = 0; i < MAX_VFD_ADDR; i++)
  {
  	if(0xff != pbAddr[i])
  	{
	  	vStbyVfdDirectWrite(pbAddr[i], pbData[i]);
  	}
  }
  	
}
void vStbyLEDShowMsg(BYTE bMsgIdIdx, BOOL fgDirect) 
{
  BYTE i = 0,j = 0;
  BYTE pbAddr[MAX_VFD_ADDR_LED];
  BYTE pbData[MAX_VFD_ADDR_LED];
  UINT16 wSegInfo = 0x00;
  UINT16 wUpdateData;
  
  for(i = 0; i < MAX_VFD_ADDR_LED; i++)
  {
	pbAddr[i] = 0xff;
	pbData[i] = 0x00;
  }
	
  for(i = 0; i < MAX_SEG_SEQ_NUM_LED; i++)
{

    wSegInfo = _pwStbyLEDSegUnit[_pbStbyLEDMsg[bMsgIdIdx][i]];

	 for(j = 0; j < SEG_SIZE; j ++)
	 {
		wUpdateData = wSegInfo >>j;
		if(0x01 & wUpdateData)
		{
			if(0xff == pbAddr[_pbStbyLEDSeg0[i][2*j]])
			{
				pbAddr[_pbStbyLEDSeg0[i][2*j]] = _pbStbyLEDSeg0[i][2*j];
			}
			pbData[_pbStbyLEDSeg0[i][2*j]] = pbData[_pbStbyLEDSeg0[i][2*j]] | (_pbStbyLEDSeg0[i][2*j+1]);
		}
		else
		{
			
		}
	 }
  }
  for(i = 0; i < MAX_VFD_ADDR_LED; i++)
  {
  	if(0xff != pbAddr[i])
  	{
	  	vStbyVfdDirectWrite(pbAddr[i], pbData[i]);
  	}
  }
  	
}

static void vStbyVfdDirectWrite(BYTE bAddr, BYTE bData) 
{
  if(bAddr == 0xff)
  {
    return;
  }
  vStbyVfdStrobe(FALSE);
  vStbyVfdUpdate(DATA_SET_INC) ;  // issue the write data command
  vStbyVfdStrobe(TRUE);

  
  vStbyVfdStrobe(FALSE);
  vStbyVfdUpdate(ADDR_SET | bAddr);  // after setting the address, the strobe need not to pull high
  vStbyVfdStrobe(FALSE);

  vStbyVfdUpdate(bData);   // the final data is written , strobe pull high
  vStbyVfdStrobe(TRUE);
  
}

void vStbyDelay(UINT32 wTicks)
{
    UINT32 i;
	for(i = 0; i < wTicks; i++)
	{
		udelay(1000);
	}
}

#elif  CONFIG_MT85XX_VFD_JSN
#define SUPPORT_VFD_FV950       1
/******************************************************************************
* Local variable
******************************************************************************/


/****************************************************************************
** Function prototypes
****************************************************************************/

/****************************************************************************
** static Function
****************************************************************************/
static void vStbyVfdDirectWrite(BYTE bAddr, BYTE bData);
static void vStbyVfdUpdate(UINT8 bTmp);

const UINT8 _pbStbyVfdKeyScan[] = {
	IR_POWER, IR_EJECT, IR_PLAY, IR_STOP,
	IR_PLAY, IR_FR, IR_CH_DOWN, IR_SW14,
	IR_STOP, IR_NEXT, IR_POWER, IR_SW15,
	IR_PAUSE, IR_PREV, IR_EJECT, IR_SW16
};

//DRPNECG,      SMFBHJKA
const BYTE _pbStbyVfdSeg0[MAX_SEG_SEQ_NUM][SEG_SIZE*2] = 
{
 {                                                                                                                                                                                                                                                     
    0x0c,0x02, 0x0c,0x04, 0x0c,0x08, 0x0c,0x10, 0x0c,0x20, 0x0c,0x40, 0x0c,0x80, 0x0d,0x01, 0x0d,0x02, 0x0d,0x04, 0x0d,0x08, 0x0d,0x10, 0x0d,0x20, 0x0d,0x40, 0x0d,0x80,
 },                                                                                                                                                          
 {                                                                                                                                                                     
    0x09,0x02, 0x09,0x04, 0x09,0x08, 0x09,0x10, 0x09,0x20, 0x09,0x40, 0x09,0x80, 0x0a,0x01, 0x0a,0x02, 0x0a,0x04, 0x0a,0x08, 0x0a,0x10, 0x0a,0x20, 0x0a,0x40, 0x0a,0x80,
 },                                                                                                                                                          
 {                                                                                                                                                                     
    0x06,0x02, 0x06,0x04, 0x06,0x08, 0x06,0x10, 0x06,0x20, 0x06,0x40, 0x06,0x80, 0x07,0x01, 0x07,0x02, 0x07,0x04, 0x07,0x08, 0x07,0x10, 0x07,0x20, 0x07,0x40, 0x07,0x80,
 },                                                                                                                                                          
 {                                                                                                                                                           
    0x03,0x02, 0x03,0x04, 0x03,0x08, 0x03,0x10, 0x03,0x20, 0x03,0x40, 0x03,0x80, 0x04,0x01, 0x04,0x02, 0x04,0x04, 0x04,0x08, 0x04,0x10, 0x04,0x20, 0x04,0x40, 0x04,0x80,
 },                                                                                                                                                          
 {                                                                                                                                                            
    0x00,0x02, 0x00,0x04, 0x00,0x08, 0x00,0x10, 0x00,0x20, 0x00,0x40, 0x00,0x80, 0x01,0x01, 0x01,0x02, 0x01,0x04, 0x01,0x08, 0x01,0x10, 0x01,0x20, 0x01,0x40, 0x01,0x80,
 },   
};

const BYTE _pbStbyVfdMsg[][MAX_SEG_SEQ_NUM] = {
     {VFD_CHAR_S, VFD_CHAR_MINU, VFD_CHAR_U,VFD_CHAR_P, VFD_CHAR_G} // S_UPG
    ,{VFD_CHAR_S, VFD_CHAR_MINU, VFD_CHAR_S,VFD_CHAR_T, VFD_CHAR_R}   // S_STR
    ,{VFD_CHAR_S, VFD_CHAR_MINU, VFD_CHAR_E,VFD_CHAR_N, VFD_CHAR_D}   // S_END
    ,{VFD_CHAR_S, VFD_CHAR_MINU, VFD_CHAR_N,VFD_CHAR_G, VFD_SPACE}   // S_NG
    ,{VFD_CHAR_U, VFD_CHAR_B,    VFD_CHAR_O,VFD_CHAR_O, VFD_CHAR_T}   //UBOOT
    ,{VFD_SPACE,  VFD_SPACE,     VFD_SPACE, VFD_SPACE, VFD_SPACE}   //Clean
};


// The following data is for Character table.
#define  DOT_D       (0x00)
#define  DOT_R       (0x01)
#define  DOT_P       (0x02)
#define  DOT_N       (0x03)
#define  DOT_E       (0x04)
#define  DOT_C       (0x05)
#define  DOT_G       (0x06)
#define  DOT_S       (0x07)
#define  DOT_M       (0x08)
#define  DOT_F       (0x09)
#define  DOT_B       (0x0A)
#define  DOT_H       (0x0B)
#define  DOT_J       (0x0C)
#define  DOT_K       (0x0D)
#define  DOT_A       (0x0E)

const UINT16 _pwStbySegUnit[] = 
{
    /*0-9*/
   (1<<DOT_A)|(1<<DOT_B)|(1<<DOT_C)|(1<<DOT_D)|(1<<DOT_E)|(1<<DOT_F),   
   (1<<DOT_B)|(1<<DOT_C) ,
   (1<<DOT_A)|(1<<DOT_B)|(1<<DOT_D)|(1<<DOT_E)|(1<<DOT_G)|(1<<DOT_M)|(1<<DOT_S),
   (1<<DOT_A)|(1<<DOT_B)|(1<<DOT_C)|(1<<DOT_D)|(1<<DOT_G)|(1<<DOT_M)|(1<<DOT_S),
   (1<<DOT_B)|(1<<DOT_C)|(1<<DOT_F)|(1<<DOT_G)|(1<<DOT_M)|(1<<DOT_S),
   (1<<DOT_A)|(1<<DOT_C)|(1<<DOT_D)|(1<<DOT_F)|(1<<DOT_G)|(1<<DOT_M)|(1<<DOT_S),
   (1<<DOT_A)|(1<<DOT_C)|(1<<DOT_D)|(1<<DOT_E)|(1<<DOT_F)|(1<<DOT_G)|(1<<DOT_M)|(1<<DOT_S),
   (1<<DOT_A)|(1<<DOT_B)|(1<<DOT_C),
   (1<<DOT_A)|(1<<DOT_B)|(1<<DOT_C)|(1<<DOT_D)|(1<<DOT_E)|(1<<DOT_F)|(1<<DOT_G)|(1<<DOT_M)|(1<<DOT_S),
   (1<<DOT_A)|(1<<DOT_B)|(1<<DOT_C)|(1<<DOT_D)|(1<<DOT_F)|(1<<DOT_G)|(1<<DOT_M)|(1<<DOT_S),
   (1<<DOT_G)|(1<<DOT_S)|(1<<DOT_M), /*VFD_CHAR_, VFD_CHAR_NUL*/     //10
   
    /*A--Z*/
   (1<<DOT_A)|(1<<DOT_B)|(1<<DOT_C)|(1<<DOT_E)|(1<<DOT_F)|(1<<DOT_G)|(1<<DOT_M)|(1<<DOT_S),
   (1<<DOT_A)|(1<<DOT_B)|(1<<DOT_C)|(1<<DOT_D)|(1<<DOT_G)|(1<<DOT_M)|(1<<DOT_S)|(1<<DOT_J)|(1<<DOT_P),
   (1<<DOT_A)|(1<<DOT_D)|(1<<DOT_E)|(1<<DOT_F),
   (1<<DOT_A)|(1<<DOT_B)|(1<<DOT_C)|(1<<DOT_D)|(1<<DOT_S)|(1<<DOT_J)|(1<<DOT_P),
   (1<<DOT_A)|(1<<DOT_D)|(1<<DOT_E)|(1<<DOT_F)|(1<<DOT_G)|(1<<DOT_M)|(1<<DOT_S),
   (1<<DOT_A)|(1<<DOT_E)|(1<<DOT_F)|(1<<DOT_G)|(1<<DOT_M)|(1<<DOT_S),
   (1<<DOT_A)|(1<<DOT_C)|(1<<DOT_D)|(1<<DOT_E)|(1<<DOT_F)|(1<<DOT_M),
   (1<<DOT_B)|(1<<DOT_C)|(1<<DOT_E)|(1<<DOT_F)|(1<<DOT_G)|(1<<DOT_M)|(1<<DOT_S),
   (1<<DOT_A)|(1<<DOT_D)|(1<<DOT_S)|(1<<DOT_J)|(1<<DOT_P),
   (1<<DOT_B)|(1<<DOT_C)|(1<<DOT_D)|(1<<DOT_E),
   (1<<DOT_E)|(1<<DOT_F)|(1<<DOT_G)|(1<<DOT_S)|(1<<DOT_N)|(1<<DOT_K),
   (1<<DOT_D)|(1<<DOT_E)|(1<<DOT_F),
   (1<<DOT_B)|(1<<DOT_C)|(1<<DOT_E)|(1<<DOT_F)|(1<<DOT_H)|(1<<DOT_S)|(1<<DOT_K),
   (1<<DOT_B)|(1<<DOT_C)|(1<<DOT_E)|(1<<DOT_F)|(1<<DOT_H)|(1<<DOT_S)|(1<<DOT_N),
   (1<<DOT_A)|(1<<DOT_B)|(1<<DOT_C)|(1<<DOT_D)|(1<<DOT_E)|(1<<DOT_F),
   (1<<DOT_A)|(1<<DOT_B)|(1<<DOT_E)|(1<<DOT_F)|(1<<DOT_G)|(1<<DOT_M)|(1<<DOT_S),
   (1<<DOT_A)|(1<<DOT_B)|(1<<DOT_C)|(1<<DOT_D)|(1<<DOT_E)|(1<<DOT_F)|(1<<DOT_N),
   (1<<DOT_A)|(1<<DOT_B)|(1<<DOT_E)|(1<<DOT_F)|(1<<DOT_G)|(1<<DOT_M)|(1<<DOT_S)|(1<<DOT_N),
   (1<<DOT_A)|(1<<DOT_C)|(1<<DOT_D)|(1<<DOT_F)|(1<<DOT_G)|(1<<DOT_M)|(1<<DOT_S),
   (1<<DOT_A)|(1<<DOT_S)|(1<<DOT_J)|(1<<DOT_P),
   (1<<DOT_B)|(1<<DOT_C)|(1<<DOT_D)|(1<<DOT_E)|(1<<DOT_F),
   (1<<DOT_E)|(1<<DOT_F)|(1<<DOT_S)|(1<<DOT_K)|(1<<DOT_R),
   (1<<DOT_B)|(1<<DOT_C)|(1<<DOT_E)|(1<<DOT_F)|(1<<DOT_S)|(1<<DOT_N)|(1<<DOT_R),  //SEG_CHAR_W  33
   (1<<DOT_H)|(1<<DOT_S)|(1<<DOT_N)|(1<<DOT_K)|(1<<DOT_R),
   (1<<DOT_B)|(1<<DOT_C)|(1<<DOT_D)|(1<<DOT_F)|(1<<DOT_G)|(1<<DOT_M)|(1<<DOT_S),
   (1<<DOT_A)|(1<<DOT_D)|(1<<DOT_S)|(1<<DOT_K)|(1<<DOT_R),
   (0x00),    //   37
   (1<<DOT_D),
};


void vStbyVfdKeyScan(void) 
{
  UINT8 i;
  UINT8 bData;
  UINT8  _bStbyVfdKeyScan = IR_NONE;
  vStbyVfdStrobe(FALSE);       
  vStbyVfdUpdate(KEY_SCAN);
  vStbyVfdClk(TRUE);         

  vStbyVfdData(TRUE);        
  vStbyVfdClk(TRUE);

  for(i = 0; i < KEY_SCAN_SIZE; i++)     
  {
    vStbyVfdClk(FALSE);
    vStbyVfdClk(FALSE);  
    vStbyVfdClk(TRUE);

    // get the bits from the datout pin
    vStbyVfdDataIn(bData);
    if(bData)
    {
        _bStbyVfdKeyScan = _pbStbyVfdKeyScan[i];
    }
    vStbyVfdClk(TRUE);   
  }

  vStbyVfdStrobe(TRUE);     

  if(IR_NONE != _bStbyVfdKeyScan)
  {
  	printf("_bStbyVfdKeyScan = %d !\n", _bStbyVfdKeyScan);
  }
  
}

static void vStbyVfdUpdate(UINT8 bTmp) 
{
  UINT8 i;

  for (i = 0; i < 8; i++)
  {
    vStbyVfdClk(FALSE);

    vStbyVfdCombineBit(bTmp);
    bTmp >>= 1;

    vStbyVfdClk(TRUE);
  }

  vStbyVfdData(TRUE);
}

/****************************************************************************
** public  Function
****************************************************************************/

void vStbyVFDInit(void) 
{

  
  //#if (CONFIG_CHIP_VER_CURR != CONFIG_CHIP_VER_MT8580)
      puts("[MT85XX Boot] 53 63 80 do not need vfd return\n");

      #if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8580)
        GPIO_Config(PIN_GPIO27, OUTPUT, HIGH);
      #endif
      #if CONFIG_MT8580_AUDIO_JSN
      GPIO_Config(PIN_VFD_CLK, INPUT, LOW);
      #else      
  //    return; 
      #endif
      
  //#endif
  puts("[MT85XX Boot] Uboot vStbyVFDInit\n");
      #if (CONFIG_CHIP_VER_CURR != CONFIG_CHIP_VER_MT8581)
      GPIO_Config(PIN_VFD_STB, OUTPUT, LOW);
      GPIO_Config(PIN_VFD_CLK, OUTPUT, LOW);
      GPIO_Config(PIN_VFD_DATA ,INPUT, HIGH);  //data in
      GPIO_Config(PIN_LCDRD , OUTPUT, LOW);     //data out

  vStbyVfdDirectClrAll();

  vStbyVfdStrobe(FALSE);         // begin to write the VFD command
  vStbyVfdUpdate(MODESET);
  vStbyVfdStrobe(TRUE);          // strobe pull high to release the mode set
  #if SUPPORT_VFD_FV950
  /*command and data must be within strobe*/  //command2 follow 0x41(charge pump)
  vStbyVfdStrobe(FALSE);   
  vStbyVfdUpdate(0x41);    
  vStbyVfdUpdate(0x80);   
  vStbyVfdStrobe(TRUE);         
  #endif
  vStbyVfdStrobe(FALSE);        
  vStbyVfdUpdate(DISPMODE_ON);
  vStbyVfdStrobe(TRUE);         
#endif
#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8581)// 

if(!b_model_is_cx())//
{
	GPIO_Config(PIN_RX_PWR5V, OUTPUT, LOW);//RESET
	vStbyDelay(100);
	GPIO_Config(PIN_RX_PWR5V, OUTPUT, HIGH);//ACTIVE
 GPIO_Config(PIN_VSTB, OUTPUT, LOW);
 GPIO_Config(PIN_VCLK, OUTPUT, LOW);
 GPIO_Config(PIN_VDATA ,OUTPUT, LOW);  //data in
}

 #endif 
  
  printf("***uboot vfd init***!, %d\n", GPIO_Input(PIN_GPIO5));  
}

void vStbyVfdDirectClrAll(void) 
{
  BYTE i;

  for(i = 0; i < MAX_VFD_ADDR; i++)
  {
    vStbyVfdDirectWrite(i, 0);
  }
}

static void vUpgVfdDirectWrite2Byte(BYTE cmd, BYTE bData) 
{


  vStbyVfdStrobe(FALSE);

  vStbyVfdUpdate(cmd);  // after setting the address, the strobe need not to pull high command3  
  
  vStbyVfdUpdate(bData);   // the final data is written , strobe pull high
  
  vStbyVfdStrobe(TRUE);

}
//vStbyVfdDirectWrite1Byte
//cmd:E8: light on ,EA :light off
//cmd :A2: charge pump on,A0 :charge pump off
//cmd :EC standby normal, ED: grid low level sw high level.

static void vUpgVfdDirectWrite1Byte(BYTE cmd) 
{

  vStbyVfdStrobe(FALSE);
  
  //vStbyDelay(1);
  vStbyVfdUpdate(cmd);  
  
  vStbyVfdStrobe(TRUE);
  
  //vStbyDelay(1);
}
/*PT6315 spec: 
   Command2:data setting command(DATA_SET_INC 0x40)->Command3:Address setting command(ADDR_SET 0xc0)->
   Command1:Display mode command(MODESET 0x01)->Command4:Display control command(DISPMODE_ON 0x8f)
*/
void vStbyVfdShowMsg(BYTE bMsgIdIdx, BOOL fgDirect) 
{
//#if (CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8581) 

fgDirect=TRUE;


if(b_model_is_cx())// CI support VFD
{

  return;
}

//puts("[MT85XX Boot]  vStbyVfdShowMsg\n");

 if(bMsgIdIdx == SEG_MSG_S_UPG )
 {
 
// puts("[MT85XX Boot]  SHOW UPG \n");
//step2 number of digit set command 
vUpgVfdDirectWrite2Byte(0xE0,0x98); //cmd :E0 ,DATA:98
//step3 URAM Control Set
vUpgVfdDirectWrite2Byte(0x80,0x06); //cmd :E0 ,DATA:98
vUpgVfdDirectWrite2Byte(0x81,0x18); //cmd :E0 ,DATA:98

//step4 charge pump mode
vUpgVfdDirectWrite1Byte(0xA3);//charge pump on  A3 //A0
//step5 dimming set
vUpgVfdDirectWrite2Byte(0xE4,0xFF); //cmd :E4 ,DATA:F0 full luminance  //E4 FF
//step6 display data 
// off
vUpgVfdDirectWrite2Byte(0x20,0x20); //cmd :20 for G1 ,DATA:4F:display nothing
vUpgVfdDirectWrite2Byte(0x21,0x20); //cmd :21 for G2 ,DATA:4C:display nothing
vUpgVfdDirectWrite2Byte(0x22,0x20); //cmd :22 for G3 ,DATA:4C:display nothing
vUpgVfdDirectWrite2Byte(0x23,0x20); //cmd :23 for G4 ,DATA:45:display nothing
vUpgVfdDirectWrite2Byte(0x24,0x20); //cmd :24 for G5 ,DATA:48:display nothing
vUpgVfdDirectWrite2Byte(0x25,0x20); //cmd :24 for G6 ,DATA:48:display nothing


//set 
vUpgVfdDirectWrite2Byte(0x22,0x55); //cmd :23 for G4 ,DATA:45:display 'U' 
vUpgVfdDirectWrite2Byte(0x23,0x50); //cmd :22 for G3 ,DATA:4C:display 'P' 
vUpgVfdDirectWrite2Byte(0x24,0x47); //cmd :21 for G2 ,DATA:4C:display 'G' 

//step7 release all display off
//vStbyVfdDirectWrite1Byte(0xEA);//light off
//step 8 light on
vUpgVfdDirectWrite1Byte(0xE8);//light on

 }
else if(bMsgIdIdx == SEG_MSG_S_STR )//STR
{
	//puts("[MT85XX Boot]  SHOW STR \n");

//step2 number of digit set command 
vUpgVfdDirectWrite2Byte(0xE0,0x98); //cmd :E0 ,DATA:98
//step3 URAM Control Set
vUpgVfdDirectWrite2Byte(0x80,0x06); //cmd :E0 ,DATA:98
vUpgVfdDirectWrite2Byte(0x81,0x18); //cmd :E0 ,DATA:98

//step4 charge pump mode
vUpgVfdDirectWrite1Byte(0xA3);//charge pump on  A3 //A0
//step5 dimming set
vUpgVfdDirectWrite2Byte(0xE4,0xFF); //cmd :E4 ,DATA:F0 full luminance	//E4 FF
//step6 display data 
// off
vUpgVfdDirectWrite2Byte(0x20,0x20); //cmd :20 for G1 ,DATA:4F:display nothing
vUpgVfdDirectWrite2Byte(0x21,0x20); //cmd :21 for G2 ,DATA:4C:display nothing
vUpgVfdDirectWrite2Byte(0x22,0x20); //cmd :22 for G3 ,DATA:4C:display nothing
vUpgVfdDirectWrite2Byte(0x23,0x20); //cmd :23 for G4 ,DATA:45:display nothing
vUpgVfdDirectWrite2Byte(0x24,0x20); //cmd :24 for G5 ,DATA:48:display nothing
vUpgVfdDirectWrite2Byte(0x25,0x20); //cmd :24 for G6 ,DATA:48:display nothing


//set 
vUpgVfdDirectWrite2Byte(0x22,0x53); //cmd :23 for G4 ,DATA:45:display 'S' 
vUpgVfdDirectWrite2Byte(0x23,0x54); //cmd :22 for G3 ,DATA:4C:display 'T' 
vUpgVfdDirectWrite2Byte(0x24,0x52); //cmd :21 for G2 ,DATA:4C:display 'R' 

//step7 release all display off
//vStbyVfdDirectWrite1Byte(0xEA);//light off
//step 8 light on
vUpgVfdDirectWrite1Byte(0xE8);//light on

 }
else if(bMsgIdIdx == SEG_MSG_S_END )//END
{
//	puts("[MT85XX Boot]  SHOW END \n");

//step2 number of digit set command 
vUpgVfdDirectWrite2Byte(0xE0,0x98); //cmd :E0 ,DATA:98
//step3 URAM Control Set
vUpgVfdDirectWrite2Byte(0x80,0x06); //cmd :E0 ,DATA:98
vUpgVfdDirectWrite2Byte(0x81,0x18); //cmd :E0 ,DATA:98

//step4 charge pump mode
vUpgVfdDirectWrite1Byte(0xA3);//charge pump on  A3 //A0
//step5 dimming set
vUpgVfdDirectWrite2Byte(0xE4,0xFF); //cmd :E4 ,DATA:F0 full luminance	//E4 FF
//step6 display data 
// off
vUpgVfdDirectWrite2Byte(0x20,0x20); //cmd :20 for G1 ,DATA:4F:display nothing
vUpgVfdDirectWrite2Byte(0x21,0x20); //cmd :21 for G2 ,DATA:4C:display nothing
vUpgVfdDirectWrite2Byte(0x22,0x20); //cmd :22 for G3 ,DATA:4C:display nothing
vUpgVfdDirectWrite2Byte(0x23,0x20); //cmd :23 for G4 ,DATA:45:display nothing
vUpgVfdDirectWrite2Byte(0x24,0x20); //cmd :24 for G5 ,DATA:48:display nothing
vUpgVfdDirectWrite2Byte(0x25,0x20); //cmd :24 for G6 ,DATA:48:display nothing


//set 
vUpgVfdDirectWrite2Byte(0x22,0x45); //cmd :23 for G4 ,DATA:45:display 'E' 
vUpgVfdDirectWrite2Byte(0x23,0x4E); //cmd :22 for G3 ,DATA:4C:display 'N' 
vUpgVfdDirectWrite2Byte(0x24,0x44); //cmd :21 for G2 ,DATA:4C:display 'D' 

//step7 release all display off
//vStbyVfdDirectWrite1Byte(0xEA);//light off
//step 8 light on
vUpgVfdDirectWrite1Byte(0xE8);//light on

 }
else if(bMsgIdIdx == SEG_MSG_CLEAN )
{
//	puts("[MT85XX Boot]   CLEAN \n");

//step2 number of digit set command 
vUpgVfdDirectWrite2Byte(0xE0,0x98); //cmd :E0 ,DATA:98
//step3 URAM Control Set
vUpgVfdDirectWrite2Byte(0x80,0x06); //cmd :E0 ,DATA:98
vUpgVfdDirectWrite2Byte(0x81,0x18); //cmd :E0 ,DATA:98

//step4 charge pump mode
vUpgVfdDirectWrite1Byte(0xA3);//charge pump on  A3 //A0
//step5 dimming set
vUpgVfdDirectWrite2Byte(0xE4,0xFF); //cmd :E4 ,DATA:F0 full luminance	//E4 FF
//step6 display data 
// off
vUpgVfdDirectWrite2Byte(0x20,0x20); //cmd :20 for G1 ,DATA:4F:display nothing
vUpgVfdDirectWrite2Byte(0x21,0x20); //cmd :21 for G2 ,DATA:4C:display nothing
vUpgVfdDirectWrite2Byte(0x22,0x20); //cmd :22 for G3 ,DATA:4C:display nothing
vUpgVfdDirectWrite2Byte(0x23,0x20); //cmd :23 for G4 ,DATA:45:display nothing
vUpgVfdDirectWrite2Byte(0x24,0x20); //cmd :24 for G5 ,DATA:48:display nothing
vUpgVfdDirectWrite2Byte(0x25,0x20); //cmd :24 for G6 ,DATA:48:display nothing


//set 
//vStbyVfdDirectWrite2Byte(0x21,0x47); //cmd :21 for G2 ,DATA:4C:display 'G' 
//vStbyVfdDirectWrite2Byte(0x22,0x50); //cmd :22 for G3 ,DATA:4C:display 'P' 
//vStbyVfdDirectWrite2Byte(0x23,0x55); //cmd :23 for G4 ,DATA:45:display 'U' 
//step7 release all display off
vUpgVfdDirectWrite1Byte(0xEA);//light off
//step 8 light on
//vStbyVfdDirectWrite1Byte(0xE8);//light on

 }
else if(bMsgIdIdx == SEG_MSG_S_NG )//NG
{
	//puts("[MT85XX Boot]  SHOW NG \n");

//step2 number of digit set command 
vUpgVfdDirectWrite2Byte(0xE0,0x98); //cmd :E0 ,DATA:98
//step3 URAM Control Set
vUpgVfdDirectWrite2Byte(0x80,0x06); //cmd :E0 ,DATA:98
vUpgVfdDirectWrite2Byte(0x81,0x18); //cmd :E0 ,DATA:98

//step4 charge pump mode
vUpgVfdDirectWrite1Byte(0xA3);//charge pump on  A3 //A0
//step5 dimming set
vUpgVfdDirectWrite2Byte(0xE4,0xFF); //cmd :E4 ,DATA:F0 full luminance	//E4 FF
//step6 display data 
// off
vUpgVfdDirectWrite2Byte(0x20,0x20); //cmd :20 for G1 ,DATA:4F:display nothing
vUpgVfdDirectWrite2Byte(0x21,0x20); //cmd :21 for G2 ,DATA:4C:display nothing
vUpgVfdDirectWrite2Byte(0x22,0x20); //cmd :22 for G3 ,DATA:4C:display nothing
vUpgVfdDirectWrite2Byte(0x23,0x20); //cmd :23 for G4 ,DATA:45:display nothing
vUpgVfdDirectWrite2Byte(0x24,0x20); //cmd :24 for G5 ,DATA:48:display nothing
vUpgVfdDirectWrite2Byte(0x25,0x20); //cmd :24 for G6 ,DATA:48:display nothing


//set 
vUpgVfdDirectWrite2Byte(0x22,0x4E); //cmd :23 for G3 ,DATA:4C:display 'N' 
vUpgVfdDirectWrite2Byte(0x23,0x47); //cmd :22 for G4 ,DATA:45:display 'G' 

//step7 release all display off
//vStbyVfdDirectWrite1Byte(0xEA);//light off
//step 8 light on
vUpgVfdDirectWrite1Byte(0xE8);//light on

 }
else if(bMsgIdIdx == SEG_MSG_UBOOT )//UBOOT
{
//	puts("[MT85XX Boot]  SHOW UBOOT \n");

//step2 number of digit set command 
vUpgVfdDirectWrite2Byte(0xE0,0x98); //cmd :E0 ,DATA:98
//step3 URAM Control Set
vUpgVfdDirectWrite2Byte(0x80,0x06); //cmd :E0 ,DATA:98
vUpgVfdDirectWrite2Byte(0x81,0x18); //cmd :E0 ,DATA:98

//step4 charge pump mode
vUpgVfdDirectWrite1Byte(0xA3);//charge pump on  A3 //A0
//step5 dimming set
vUpgVfdDirectWrite2Byte(0xE4,0xFF); //cmd :E4 ,DATA:F0 full luminance	//E4 FF
//step6 display data 
// off
vUpgVfdDirectWrite2Byte(0x20,0x20); //cmd :20 for G1 ,DATA:4F:display nothing
vUpgVfdDirectWrite2Byte(0x21,0x20); //cmd :21 for G2 ,DATA:4C:display nothing
vUpgVfdDirectWrite2Byte(0x22,0x20); //cmd :22 for G3 ,DATA:4C:display nothing
vUpgVfdDirectWrite2Byte(0x23,0x20); //cmd :23 for G4 ,DATA:45:display nothing
vUpgVfdDirectWrite2Byte(0x24,0x20); //cmd :24 for G5 ,DATA:48:display nothing
vUpgVfdDirectWrite2Byte(0x25,0x20); //cmd :24 for G6 ,DATA:48:display nothing


//set 
vUpgVfdDirectWrite2Byte(0x20,0x55); //cmd :24 for G5 ,DATA:48:display 'U'
vUpgVfdDirectWrite2Byte(0x21,0x42); //cmd :23 for G4 ,DATA:45:display 'B' 
vUpgVfdDirectWrite2Byte(0x22,0x4F); //cmd :22 for G3 ,DATA:4C:display 'O' 
vUpgVfdDirectWrite2Byte(0x23,0x4F); //cmd :21 for G2 ,DATA:4C:display 'O' 
vUpgVfdDirectWrite2Byte(0x24,0x54); //cmd :20 for G1 ,DATA:4F:display 'T'

//step7 release all display off
//vStbyVfdDirectWrite1Byte(0xEA);//light off
//step 8 light on
vUpgVfdDirectWrite1Byte(0xE8);//light on

 }


}


static void vStbyVfdDirectWrite(BYTE bAddr, BYTE bData) 
{
  if(bAddr == 0xff)
  {
    return;
  }

  #if SUPPORT_VFD_FV950
  vStbyVfdStrobe(FALSE);
  vStbyVfdUpdate(ADDR_SET | bAddr);  // after setting the address, the strobe need not to pull high command3  
  vStbyVfdStrobe(TRUE);
  
  vStbyVfdStrobe(FALSE);
  vStbyVfdUpdate(DATA_SET_INC) ;  // issue the write data command 2   
  vStbyVfdUpdate(bData);   // the final data is written , strobe pull high
  vStbyVfdStrobe(TRUE);
  #else
  vStbyVfdStrobe(FALSE);
  vStbyVfdUpdate(DATA_SET_INC) ;  // issue the write data command2
  vStbyVfdStrobe(TRUE);

  
  vStbyVfdStrobe(FALSE);
  vStbyVfdUpdate(ADDR_SET | bAddr);  // after setting the address, the strobe need not to pull high command3
  vStbyVfdUpdate(bData);   // the final data is written , strobe pull high
  vStbyVfdStrobe(TRUE);
  #endif
}

void vStbyDelay(UINT32 wTicks)
{
    UINT32 i;
	for(i = 0; i < wTicks; i++)
	{
		udelay(1000);
	}
}

#else


/******************************************************************************
* Local variable
******************************************************************************/


/****************************************************************************
** Function prototypes
****************************************************************************/

/****************************************************************************
** static Function
****************************************************************************/
static void vStbyVfdDirectWrite(BYTE bAddr, BYTE bData);
static void vStbyVfdUpdate(UINT8 bTmp);

#if YA_VFD_DRV_PT6315_DIG8_SEG20
const UINT8 _pbStbyVfdKeyScan[] = {
	IR_NONE
};
#else
const UINT8 _pbStbyVfdKeyScan[] = {
	IR_RECORD, IR_FF, IR_CH_UP, IR_SW13,
	IR_PLAY, IR_FR, IR_CH_DOWN, IR_SW14,
	IR_STOP, IR_NEXT, IR_POWER, IR_SW15,
	IR_PAUSE, IR_PREV, IR_EJECT, IR_SW16
};
#endif

#if YA_VFD_DRV_PT6315_DIG8_SEG20
const BYTE _pbStbyVfdSeg0[8][30] = 
{
   /*a               b               c                d                 e                f                 g                h                 m                 j                 p                  n                 k                  r                 s */
  {0x00, 0x02, 0x00, 0x40, 0x01, 0x40, 0x01, 0x80, 0x01, 0x04, 0x00, 0x04, 0x00, 0x80, 0x00, 0x08, 0x01, 0x02, 0x00, 0x10,  0x01, 0x10,  0x01, 0x20,   0x00, 0x20,  0x01, 0x08, 0x01, 0x01,},
  {0x03, 0x02, 0x03, 0x40, 0x04, 0x40, 0x04, 0x80, 0x04, 0x04, 0x03, 0x04, 0x03, 0x80, 0x03, 0x08, 0x04, 0x02, 0x03, 0x10,  0x04, 0x10,  0x04, 0x20,   0x03, 0x20,  0x04, 0x08, 0x04, 0x01,},
  {0x06, 0x02, 0x06, 0x40, 0x07, 0x40, 0x07, 0x80, 0x07, 0x04, 0x06, 0x04, 0x06, 0x80, 0x06, 0x08, 0x07, 0x02, 0x06, 0x10,  0x07, 0x10,  0x07, 0x20,   0x06, 0x20,  0x07, 0x08, 0x07, 0x01,},
  {0x09, 0x02, 0x09, 0x40, 0x0a, 0x40, 0x0a, 0x80, 0x0a, 0x04, 0x09, 0x04, 0x09, 0x80, 0x09, 0x08, 0x0a, 0x02, 0x09, 0x10,  0x0a, 0x10,  0x0a, 0x20,   0x09, 0x20,  0x0a, 0x08, 0x0a, 0x01,},
  {0x0c, 0x02, 0x0c, 0x40, 0x0d, 0x40, 0x0d, 0x80, 0x0d, 0x04, 0x0c, 0x04, 0x0c, 0x80, 0x0c, 0x08, 0x0d, 0x02, 0x0c, 0x10,  0x0d, 0x10,  0x0d, 0x20,   0x0c, 0x20,  0x0d, 0x08, 0x0d, 0x01,},
  {0x0f, 0x02, 0x0f, 0x40, 0x10, 0x40, 0x10, 0x80, 0x10, 0x04, 0x0f, 0x04, 0x0f, 0x80, 0x0f, 0x08, 0x10, 0x02, 0x0f, 0x10,  0x10, 0x10,  0x10, 0x20,   0x0f, 0x20,  0x10, 0x08, 0x10, 0x01,},
  {0x12, 0x02, 0x12, 0x40, 0x13, 0x40, 0x13, 0x80, 0x13, 0x04, 0x12, 0x04, 0x12, 0x80, 0x12, 0x08, 0x13, 0x02, 0x12, 0x10,  0x13, 0x10,  0x13, 0x20,   0x12, 0x20,  0x13, 0x08, 0x13, 0x01,},
  {0x15, 0x02, 0x15, 0x40, 0x16, 0x40, 0x16, 0x80, 0x16, 0x04, 0x15, 0x04, 0x15, 0x80, 0x15, 0x08, 0x16, 0x02, 0x15, 0x10,  0x16, 0x10,  0x16, 0x20,   0x15, 0x20,  0x16, 0x08, 0x16, 0x01,},
};
const BYTE _pbStbyVfdMsg[][MAX_SEG_SEQ_NUM] = {
	{CLR, VFD_CHAR_H, VFD_CHAR_E, VFD_CHAR_L, VFD_CHAR_L, VFD_CHAR_O, CLR, CLR, CLR, CLR, CLR},	// hello 
	{CLR, CLR, CLR, CLR, CLR, CLR, CLR, CLR, CLR, CLR, CLR},	// hello 
};

#else
const BYTE _pbStbyVfdSeg0[11][28] = 
{
  {0x0a, 0x20, 0x0a, 0x10, 0x09, 0x20, 0x09, 0x04, 0x09, 0x02, 0x0a, 0x01, 0x09, 0x40, 0x0a, 0x04, 0x09, 0x80, 0x0a, 0x02, 0x0a, 0x02, 0x09, 0x10, 0x0a, 0x08, 0x09, 0x01, },
  {0x07, 0x20, 0x07, 0x10, 0x06, 0x20, 0x06, 0x04, 0x06, 0x02, 0x07, 0x01, 0x06, 0x40, 0x07, 0x04, 0x06, 0x80, 0x07, 0x02, 0x07, 0x02, 0x06, 0x10, 0x07, 0x08, 0x06, 0x01, },
  {0x0d, 0x20, 0x0d, 0x10, 0x0c, 0x20, 0x0c, 0x04, 0x0c, 0x02, 0x0d, 0x01, 0x0c, 0x40, 0x0d, 0x04, 0x0c, 0x80, 0x0d, 0x02, 0x0d, 0x02, 0x0c, 0x10, 0x0d, 0x08, 0x0c, 0x01, },
  {0x10, 0x20, 0x10, 0x10, 0x0f, 0x20, 0x0f, 0x04, 0x0f, 0x02, 0x10, 0x01, 0x0f, 0x40, 0x10, 0x04, 0x0f, 0x80, 0x10, 0x02, 0x10, 0x02, 0x0f, 0x10, 0x10, 0x08, 0x0f, 0x01, },
  {0x13, 0x20, 0x13, 0x10, 0x12, 0x20, 0x12, 0x04, 0x12, 0x02, 0x13, 0x01, 0x12, 0x40, 0x13, 0x04, 0x12, 0x80, 0x13, 0x02, 0x13, 0x02, 0x12, 0x10, 0x13, 0x08, 0x12, 0x01, },
  {0x16, 0x20, 0x16, 0x10, 0x15, 0x20, 0x15, 0x04, 0x15, 0x02, 0x16, 0x01, 0x15, 0x40, 0x16, 0x04, 0x15, 0x80, 0x16, 0x02, 0x16, 0x02, 0x15, 0x10, 0x16, 0x08, 0x15, 0x01, },
  {0x19, 0x20, 0x19, 0x10, 0x18, 0x20, 0x18, 0x04, 0x18, 0x02, 0x19, 0x01, 0x18, 0x40, 0x19, 0x04, 0x18, 0x80, 0x19, 0x02, 0x19, 0x02, 0x18, 0x10, 0x19, 0x08, 0x18, 0x01, },
  {0x1c, 0x20, 0x1c, 0x10, 0x1b, 0x20, 0x1b, 0x04, 0x1b, 0x02, 0x1c, 0x01, 0x1b, 0x40, 0x1c, 0x04, 0x1b, 0x80, 0x1c, 0x02, 0x1c, 0x02, 0x1b, 0x10, 0x1c, 0x08, 0x1b, 0x01, },
  {0x1f, 0x20, 0x1f, 0x10, 0x1e, 0x20, 0x1e, 0x04, 0x1e, 0x02, 0x1f, 0x01, 0x1e, 0x40, 0x1f, 0x04, 0x1e, 0x80, 0x1f, 0x02, 0x1f, 0x02, 0x1e, 0x10, 0x1f, 0x08, 0x1e, 0x01, },
  {0x22, 0x20, 0x22, 0x10, 0x21, 0x20, 0x21, 0x04, 0x21, 0x02, 0x22, 0x01, 0x21, 0x40, 0x22, 0x04, 0x21, 0x80, 0x22, 0x02, 0x22, 0x02, 0x21, 0x10, 0x22, 0x08, 0x21, 0x01, },
  {0x25, 0x20, 0x25, 0x10, 0x24, 0x20, 0x24, 0x04, 0x24, 0x02, 0x25, 0x01, 0x24, 0x40, 0x25, 0x04, 0x24, 0x80, 0x25, 0x02, 0x25, 0x02, 0x24, 0x10, 0x25, 0x08, 0x24, 0x01, },
};

const BYTE _pbStbyVfdMsg[][MAX_SEG_SEQ_NUM] = {
	{VFD_CHAR_H, VFD_CHAR_E, VFD_CHAR_L, VFD_CHAR_L, VFD_CHAR_O, CLR, CLR, CLR, CLR, CLR, CLR},	// hello 
	{VFD_CHAR_S, VFD_CHAR_T, VFD_CHAR_O,VFD_CHAR_P, CLR, CLR, CLR, CLR, CLR, CLR,CLR},		   // stop 
	{VFD_CHAR_O, VFD_CHAR_P, VFD_CHAR_E, VFD_CHAR_N,CLR, CLR, CLR, CLR, CLR, CLR, CLR},				// open, 
	{VFD_CHAR_C, VFD_CHAR_L, VFD_CHAR_O, VFD_CHAR_S, VFD_CHAR_E, CLR, CLR, CLR, CLR, CLR, CLR},	// close, 
	{VFD_CHAR_O, VFD_CHAR_F, VFD_CHAR_F, CLR, CLR, CLR, CLR, CLR, CLR, CLR, CLR},							// OFF, 
	{VFD_CHAR_E, VFD_CHAR_R, VFD_CHAR_R, CLR, CLR, CLR, CLR, CLR, CLR, CLR, CLR},								// err, 
	{VFD_CHAR_H, VFD_CHAR_E, VFD_CHAR_L,VFD_CHAR_L, VFD_CHAR_O, CLR,VFD_CHAR_W,VFD_CHAR_O,VFD_CHAR_R, VFD_CHAR_L,VFD_CHAR_D},// HELLO WORLD, 
	{VFD_CHAR_U, CLR,VFD_CHAR_B, VFD_CHAR_O, VFD_CHAR_O, VFD_CHAR_T,CLR,CLR,CLR,CLR,CLR},	// U_BOOT, 
	{CLR, CLR, CLR, CLR, CLR, CLR, CLR, CLR, CLR, CLR, CLR} // clean all, 10: start 93
};
#endif

// The following data is for Character table.
#if YA_VFD_DRV_PT6315_DIG8_SEG20
const UINT16 _pwStbySegUnit[] = 
{
	0x003f,  0x0006,  0x415b,  0x414f,  0x4166, //01234 
	0x416d,  0x417d,  0x0027,  0x417f,  0x416f, //56789 
	0x0000,  0x4177,  0x4177,  0x470f,  0x417c,//clean all, AaBb  
	0x0039,  0x0158,  0x460f,  0x415e,  0x4179, //CcDdE 
	0x417b,  0x4171,  0x4171,  0x013d,  0x416f, //eFfGg 
	0x4176,  0x4174,  0x4609,  0x4609,  0x001e, //HhIiJ 
	0x001e,  0x5870,  0x5870,  0x0038,  0x0038, //jKkLl 
	0x50b6,  0x4554,  0x08b6,  0x0037,  0x003f, //MmNnO 
	0x415c,  0x4173,  0x4173,  0x083f,  0x083f, //oPpQq 
	0x4973,  0x4973,  0x416d,  0x416d,  0x4601, //RrSsT 
	0x4178,  0x003e,  0x001c,  0x7030,  0x7030, //tUuVv 
	0x6836,  0x441c,  0x7880,  0x7880,  0x0562, //WwXxY 
	0x7080,  0x3009,  0x7009,  0x0000,  0x0000,//yZz  
	0x0008,  0x4140,  0x4600,  0x7000,  0x4880,  
	0x0039,  0x000f,  0x7fff,  
};
#else
const UINT16 _pwStbySegUnit[] = 
{
  0x003f, 0x0006, 0x015b, 0x014f, 0x0166, 
  0x016d, 0x017d, 0x0007, 0x017f, 0x016f, 
  0x0000, 0x0177, 0x0177, 0x070f, 0x017c, 
  0x0039, 0x0158, 0x060f, 0x015e, 0x0179, 
  0x017b, 0x0171, 0x0171, 0x013d, 0x016f, 
  0x0176, 0x0174, 0x0609, 0x0609, 0x001f, 
  0x001e, 0x1870, 0x1870, 0x0038, 0x0038, 
  0x10b6, 0x0554, 0x08b6, 0x0037, 0x003f, 
  0x015c, 0x0173, 0x0173, 0x083f, 0x0067, 
  0x0973, 0x0050, 0x016d, 0x016d, 0x0601, 
  0x0178, 0x003e, 0x001c, 0x0886, 0x3030, 
  0x063e, 0x041c, 0x3880, 0x3880, 0x016e, 
  0x016e, 0x3009, 0x3009, 0x0000, 0x0000, 
  0x0008, 0x0140, 0x0600, 0x3000, 0x0880, 
  0x0039, 0x000f, 0x3fff, 0x0503, 
};
#endif
void vStbyVfdKeyScan(void) 
{
  UINT8 i;
  UINT8 bData;
  UINT8  _bStbyVfdKeyScan = IR_NONE;
  
  vStbyVfdStrobe(FALSE);       
  vStbyVfdUpdate(KEY_SCAN);
  vStbyVfdClk(TRUE);         

  vStbyVfdData(TRUE);        
  vStbyVfdClk(TRUE);

  for(i = 0; i < KEY_SCAN_SIZE; i++)     
  {
    vStbyVfdClk(FALSE);
    vStbyVfdClk(FALSE);  
    vStbyVfdClk(TRUE);

    // get the bits from the datout pin
    vStbyVfdDataIn(bData);
    if(bData)
    {
        _bStbyVfdKeyScan = _pbStbyVfdKeyScan[i];
    }
    vStbyVfdClk(TRUE);   
  }

  vStbyVfdStrobe(TRUE);     

  if(IR_NONE != _bStbyVfdKeyScan)
  {
  	printf("_bStbyVfdKeyScan = %d !\n", _bStbyVfdKeyScan);
  }
  
}

static void vStbyVfdUpdate(UINT8 bTmp) 
{
  UINT8 i;

  for (i = 0; i < 8; i++)
  {
    vStbyVfdClk(FALSE);

    vStbyVfdCombineBit(bTmp);
    bTmp >>= 1;

    vStbyVfdClk(TRUE);
  }

  vStbyVfdData(TRUE);
}

/****************************************************************************
** public  Function
****************************************************************************/

void vStbyVFDInit(void) 
{ 
  #if SUPPORT_SAMSUNG_VFD
   return;
  #endif
  
#if	CONFIG_PHILIPS_CUSTOMER_VFD    
	   return;
#endif
  GPIO_Config(PIN_VFD_STB, OUTPUT, LOW);
  GPIO_Config(PIN_VFD_CLK, OUTPUT, LOW);
  GPIO_Config(PIN_VFD_DATA, OUTPUT, LOW);
#if YA_VFD_DRV_PT6315_DIG8_SEG20  
  GPIO_Config(PIN_GPIO2, OUTPUT, HIGH);
#endif  
  vStbyVfdStrobe(FALSE);         // begin to write the VFD command
  vStbyVfdUpdate(MODESET);
  vStbyVfdStrobe(TRUE);          // strobe pull high to release the mode set

  vStbyVfdDirectClrAll();
  vStbyVfdStrobe(FALSE);        
  vStbyVfdUpdate(DISPMODE_ON);
  vStbyVfdStrobe(TRUE);         
  
  printf("SEG_MSG_UBOOT !\n");
  vStbyVfdDirectClrAll();
#if YA_VFD_DRV_PT6315_DIG8_SEG20  
  vStbyVfdShowMsg(0, TRUE);   //martin 20120314 SHOW HELLO
#else
  vStbyVfdShowMsg(SEG_MSG_UBOOT, TRUE);
#endif

}


void vStbyVfdDirectClrAll(void) 
{
  BYTE i;

  for(i = 0; i < MAX_VFD_ADDR; i++)
  {
    vStbyVfdDirectWrite(i, 0);
  }

}

void vStbyVfdDirectSetAll(void) 
{
  BYTE i;

  for(i = 0; i < MAX_VFD_ADDR; i++)
  {
    vStbyVfdDirectWrite(i, 0xff);
  }

}

void vStbyVfdShowMsg(BYTE bMsgIdIdx, BOOL fgDirect) 
{
  BYTE i = 0,j = 0;
  BYTE pbAddr[MAX_VFD_ADDR];
  BYTE pbData[MAX_VFD_ADDR];
  UINT16 wSegInfo = 0x00;
  UINT16 wUpdateData;
  
  for(i = 0; i < MAX_VFD_ADDR; i++)
  {
	pbAddr[i] = 0xff;
	pbData[i] = 0x00;
  }
	
  for(i = 0; i < MAX_SEG_SEQ_NUM; i++)
{

    wSegInfo = _pwStbySegUnit[_pbStbyVfdMsg[bMsgIdIdx][i]];

	 for(j = 0; j < SEG_SIZE; j ++)
	 {
		wUpdateData = wSegInfo >>j;
		if(0x01 & wUpdateData)
		{
			if(0xff == pbAddr[_pbStbyVfdSeg0[i][2*j]])
			{
				pbAddr[_pbStbyVfdSeg0[i][2*j]] = _pbStbyVfdSeg0[i][2*j];
			}
			pbData[_pbStbyVfdSeg0[i][2*j]] = pbData[_pbStbyVfdSeg0[i][2*j]] | (_pbStbyVfdSeg0[i][2*j+1]);
		}
		else
		{
			
		}
	 }
  }
  for(i = 0; i < MAX_VFD_ADDR; i++)
  {
  	if(0xff != pbAddr[i])
  	{
	  	vStbyVfdDirectWrite(pbAddr[i], pbData[i]);
  	}
  }
  	
}

static void vStbyVfdDirectWrite(BYTE bAddr, BYTE bData) 
{
  if(bAddr == 0xff)
  {
    return;
  }
  vStbyVfdStrobe(FALSE);
  vStbyVfdUpdate(DATA_SET_INC) ;  // issue the write data command
  vStbyVfdStrobe(TRUE);

  
  vStbyVfdStrobe(FALSE);
  vStbyVfdUpdate(ADDR_SET | bAddr);  // after setting the address, the strobe need not to pull high
  vStbyVfdStrobe(FALSE);

  vStbyVfdUpdate(bData);   // the final data is written , strobe pull high
  vStbyVfdStrobe(TRUE);
  
}

void vStbyDelay(UINT32 wTicks)
{
    UINT32 i;
	for(i = 0; i < wTicks; i++)
	{
		udelay(1000);
	}
}

#endif


