/********************************************************************************************
 *     LEGAL DISCLAIMER 
 *
 *     (Header of MediaTek Software/Firmware Release or Documentation)
 *
 *     BY OPENING OR USING THIS FILE, BUYER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES 
 *     THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE") RECEIVED 
 *     FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO BUYER ON AN "AS-IS" BASIS 
 *     ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES, EXPRESS OR IMPLIED, 
 *     INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR 
 *     A PARTICULAR PURPOSE OR NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY 
 *     WHATSOEVER WITH RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY, 
 *     INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND BUYER AGREES TO LOOK 
 *     ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO. MEDIATEK SHALL ALSO
 *     NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE RELEASES MADE TO BUYER'S SPECIFICATION 
 *     OR TO CONFORM TO A PARTICULAR STANDARD OR OPEN FORUM.
 *     
 *     BUYER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S ENTIRE AND CUMULATIVE LIABILITY WITH 
 *     RESPECT TO THE MEDIATEK SOFTWARE RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, 
 *     TO REVISE OR REPLACE THE MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE 
 *     FEES OR SERVICE CHARGE PAID BY BUYER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE. 
 *     
 *     THE TRANSACTION CONTEMPLATED HEREUNDER SHALL BE CONSTRUED IN ACCORDANCE WITH THE LAWS 
 *     OF THE STATE OF CALIFORNIA, USA, EXCLUDING ITS CONFLICT OF LAWS PRINCIPLES.  
 ************************************************************************************************/
/******************************************************************************
* [File]			NFIECC.h
* [Version]			v1.0
* [Revision Date]	2007-12-27
* [Author]			Meng-Chang Liu, mengchang.Liu@mediatek.com, 26615, 2007-12-27
* [Description]
*	MT8520 Download Agent NFI ECC include file
* [Copyright]
*	Copyright (C) 2007 MediaTek Incorporation. All Rights Reserved.
******************************************************************************/
#ifndef _NFIECC3_H
#define _NFIECC3_H

//#include "x_typedef.h"

/*******************************************************************************
 * NFI ECC registers definition
 *******************************************************************************/
#define NFIECC_BASE     (0xF001E000) 

/******************************************************************************/
// ECC Encoder Registers
#define NFIECC_ENCCON    ((volatile UINT16 *)(NFIECC_BASE+0x0000))
    #define ENC_EN           (0x0001)

#define NFIECC_ENCCNFG   ((volatile UINT32 *)(NFIECC_BASE+0x0004))
    #define ENC_TNUM(x)      (((UINT32)(x) &0x1F) <<0)
    #define ENC_MODE_DMA     (0x0000)  // Source data from Bus. 
    #define ENC_MODE_NFI     (0x0020)  // Source data from NFI module.
    #define ENC_MODE_PIO     (0x0040)  // Source data is written by MCU.
    #define ENC_BURST_EN     (0x0100)  // 0: DMA mode uses single read; 1: DMA mode uses burst read.
    #define ENC_MS(x)        (((UINT32)(x) &0x07FF) <<19)  // unit byte

#define NFIECC_ENCDIADDR    ((volatile UINT32 *)(NFIECC_BASE+0x0008))  // 4-bytes align

#define NFIECC_ENCIDLE      ((volatile UINT16 *)(NFIECC_BASE+0x000C))
    #define ENC_IDLE            (0x0001)

// NFI ECC Parity x(0~26) register. The registers range is 0x0010~0x0078
#define NFIECC_ENCPAR(x)    ((volatile UINT32 *)(NFIECC_BASE+0x0010+(((x)&0x1F) <<2))) // x *4

#define NFIECC_ENCSTA       ((volatile UINT32 *)(NFIECC_BASE+0x007C))

#define NFIECC_ENCIRQ_EN     ((volatile UINT32 *)(NFIECC_BASE+0x0080))
    #define ENC_IRQ_EN           (0x0001)
#define NFIECC_ENCIRQ_STA    ((volatile UINT32 *)(NFIECC_BASE+0x0084))
    #define ENC_IRQ_STA          (0x0001)
    
#define NFIECC_PIO_DIRDY     ((volatile UINT32 *)(NFIECC_BASE+0x0090))
    #define ECC_PIO_DIRDY        (0x0001)  // The PIO mode is ready for input data. 0: Busy; 1: Ready.
#define NFIECC_PIO_DI        ((volatile UINT32 *)(NFIECC_BASE+0x0094))  // PIO mdoe data input by MCU.


/******************************************************************************/
// ECC Decoder Registers
#define NFIECC_DECCON        ((volatile UINT16 *)(NFIECC_BASE+0x0100))
    #define DEC_EN               (0x0001)

#define NFIECC_DECCNFG       ((volatile UINT32 *)(NFIECC_BASE+0x0104))
    #define DEC_TNUM(x)          (((UINT32)(x) &0x1F) <<0)
    #define DEC_MODE_DMA         (0x0000)  // Source data from Bus. 
    #define DEC_MODE_NFI         (0x0020)  // Source data from NFI module.
    #define DEC_MODE_PIO         (0x0040)  // Source data is written by MCU.
    #define DEC_BURST_EN         (0x0100)  // 0: DMA mode uses single read; 1: DMA mode uses burst read.
    #define DEC_CON(x)           (((UINT32)(x) &0x03) << 12) 
    #define DEC_CS(x)            (((UINT32)(x) &0x3FFF) <<16)
    #define DEC_EMPTY_EN         (0x80000000)

#define NFIECC_DECDIADDR     ((volatile UINT32 *)(NFIECC_BASE+0x0108))  // 4-bytes align

#define NFIECC_DECIDLE       ((volatile UINT32 *)(NFIECC_BASE+0x010C))
    #define DEC_IDLE             (0x0001)

#define NFIECC_DECFER        ((volatile UINT16 *)(NFIECC_BASE+0x0110))

#define NFIECC_DECENUM0      ((volatile UINT32 *)(NFIECC_BASE+0x0114))  // sector 0~3
#define NFIECC_DECENUM1      ((volatile UINT32 *)(NFIECC_BASE+0x0118))  // sector 4~7
#define NFIECC_DECENUM2      ((volatile UINT32 *)(NFIECC_BASE+0x011C))  // sector 8~11
#define NFIECC_DECENUM3      ((volatile UINT32 *)(NFIECC_BASE+0x0120))  // sector 12~15
#define NFIECC_DECENUM(x)     ((volatile UINT32 *)(NFIECC_BASE+0x0114+((x)&0x0C)))  
    #define GET_DECENUM_SEC(x)      ((UINT8)(((*NFIECC_DECENUM(x) >>(((x)&0x3)<<3))) &0x3F))  // (x%4)*8  

#define NFIECC_DECDONE       ((volatile UINT32 *)(NFIECC_BASE+0x0124))

// NFIECC decoder Error location x(0~59) registers. The registers range is 0x0128~0x019C.
#define NFIECC_DECEL(x)      ((volatile UINT32 *)(NFIECC_BASE+0x0128+(((x)&0x3E)<<1)))  // (x&0x3E)*2
    #define GET_DECEL(x)      ((UINT16)((*NFIECC_DECEL(x) >>(((x)&0x1)<<4)) &0x3FFF))

#define NFIECC_DECIRQ_EN     ((volatile UINT16 *)(NFIECC_BASE+0x0200))
    #define DEC_IRQ_EN           (0x0001)
#define NFIECC_DECIRQ_STA    ((volatile UINT16 *)(NFIECC_BASE+0x0204))
    #define DEC_IRQ_STA          (0x0001)

#define NFIECC_DEC_FSM       ((volatile UINT32 *)(NFIECC_BASE+0x0208))

#define NFIECC_BYPASS        ((volatile UINT32 *)(NFIECC_BASE+0x020C))
    #define ECC_BYPASS           (0x0001)

typedef enum {
   ECC_DEC_NONE,
   ECC_DEC_DETECT,
   ECC_DEC_LOCATE,
   ECC_DEC_CORRECT
} ECC_Decode_Type_t;

#define ECC_PARITY_NUM   (14)

typedef enum {
    ECC_4_BITS = 0, //  0, 4,
    ECC_6_BITS,     //  1, 6,
    ECC_8_BITS,     //  2, 8,
    ECC_10_BITS,    //  3, 10,
    ECC_12_BITS,    //  4, 12,
    ECC_14_BITS,    //  5, 14,
    ECC_16_BITS,    //  6, 16,
    ECC_18_BITS,    //  7, 18,
    ECC_20_BITS,    //  8, 20,
    ECC_22_BITS,    //  9, 22,
    ECC_24_BITS,    // 10, 24,
    ECC_28_BITS,    // 11, 28,
    ECC_32_BITS,    // 12, 32,
    ECC_36_BITS,    // 13, 36,
    ECC_40_BITS,    // 14, 40,
    ECC_44_BITS,    // 15, 44,
    ECC_48_BITS,    // 16, 48,
    ECC_52_BITS,    // 17, 52, 
    ECC_56_BITS,    // 18, 56,
    ECC_60_BITS,    // 19, 60,
    ECC_BITS_MASK = 0x1F
} ECC_LEVEL_E;

#endif //_NFIECC_H
